@@ -10,6 +10,9 @@
 #include "wconfig.h"
 
 #include "WINGs.h"
+#include "WINGsP.h"
+#include "userdefaults.h"
+
 
 typedef struct W_UserDefaults {
 	WMPropList *defaults;
@@ -55,7 +58,11 @@ const char *wusergnusteppath()
 		/* Value have been already computed, re-use it */
 		return path;
 
+#ifdef HAVE_SECURE_GETENV
+	gspath = secure_getenv("GNUSTEP_USER_ROOT");
+#else
 	gspath = getenv("GNUSTEP_USER_ROOT");
+#endif
 	if (gspath) {
 		gspath = wexpandpath(gspath);
 		if (gspath) {
@@ -87,13 +94,10 @@ char *wdefaultspathfordomain(const char *domain)
 	slen = strlen(gspath) + strlen(DEFAULTS_DIR) + strlen(domain) + 4;
 	path = wmalloc(slen);
 
-	if (wstrlcpy(path, gspath, slen) >= slen ||
-	    wstrlcat(path, DEFAULTS_DIR, slen) >= slen ||
-	    wstrlcat(path, "/", slen) >= slen ||
-	    wstrlcat(path, domain, slen) >= slen) {
-		wfree(path);
-		return NULL;
-	}
+	strcpy(path, gspath);
+	strcat(path, DEFAULTS_DIR);
+	strcat(path, "/");
+	strcat(path, domain);
 
 	return path;
 }
@@ -114,9 +118,19 @@ char *wglobaldefaultspathfordomain(const char *domain)
 	return t;
 }
 
-static void
-saveDefaultsChanges(void)
+void w_save_defaults_changes(void)
 {
+	if (WMApplication.applicationName == NULL) {
+		/*
+		 * This means that the user has properly exited by calling the
+		 * function 'WMReleaseApplication' (which has already called us)
+		 * but we're being called again by the fallback 'atexit' method
+		 * (the legacy way of saving changes on exit which is kept for
+		 * application that would forget to call 'WMReleaseApplication')
+		 */
+		return;
+	}
+
 	/* save the user defaults databases */
 	synchronizeUserDefaults(NULL);
 }
@@ -127,7 +141,7 @@ static void registerSaveOnExit(void)
 	static Bool registeredSaveOnExit = False;
 
 	if (!registeredSaveOnExit) {
-		atexit(saveDefaultsChanges);
+		atexit(w_save_defaults_changes);
 		registeredSaveOnExit = True;
 	}
 }
@@ -136,6 +150,9 @@ static void synchronizeUserDefaults(void *foo)
 {
 	UserDefaults *database = sharedUserDefaults;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) foo;
+
 	while (database) {
 		if (!database->dontSync)
 			WMSynchronizeUserDefaults(database);
@@ -290,8 +307,7 @@ WMUserDefaults *WMGetStandardUserDefaults(void)
 	if (!domain)
 		domain = WMCreatePLDictionary(NULL, NULL);
 
-	if (path)
-		wfree(path);
+	wfree(path);
 
 	defaults->appDomain = domain;
 
@@ -575,7 +591,7 @@ void WMSetUDFloatForKey(WMUserDefaults * database, float value, const char *defa
 	WMPropList *object;
 	char buffer[128];
 
-	sprintf(buffer, "%f", value);
+	sprintf(buffer, "%f", (double)value);
 	object = WMCreatePLString(buffer);
 
 	WMSetUDObjectForKey(database, object, defaultName);
