@@ -43,8 +43,6 @@ typedef struct W_FontPanel {
 #define MIN_UPPER_HEIGHT	20
 #define MIN_LOWER_HEIGHT	140
 
-#define MAX_FONTS_TO_RETRIEVE	2000
-
 #define BUTTON_SPACE_HEIGHT 40
 
 #define MIN_WIDTH	250
@@ -85,6 +83,10 @@ static void listFamilies(WMScreen * scr, WMFontPanel * panel);
 
 static void splitViewConstrainCallback(WMSplitView * sPtr, int indView, int *min, int *max)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) sPtr;
+	(void) max;
+
 	if (indView == 0)
 		*min = MIN_UPPER_HEIGHT;
 	else
@@ -138,6 +140,9 @@ static void closeWindow(WMWidget * w, void *data)
 {
 	FontPanel *panel = (FontPanel *) data;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	WMHideFontPanel(panel);
 }
 
@@ -145,12 +150,19 @@ static void setClickedAction(WMWidget * w, void *data)
 {
 	FontPanel *panel = (FontPanel *) data;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	if (panel->action)
 		(*panel->action) (panel, panel->data);
 }
 
 static void revertClickedAction(WMWidget * w, void *data)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+	(void) data;
+
 	/*FontPanel *panel = (FontPanel*)data; */
 	/* XXX TODO */
 }
@@ -428,7 +440,7 @@ static void addSizeToTypeface(Typeface * face, int size)
 	if (size == 0) {
 		int j;
 
-		for (j = 0; j < sizeof(scalableFontSizes) / sizeof(scalableFontSizes[0]); j++) {
+		for (j = 0; j < wlengthof(scalableFontSizes); j++) {
 			size = scalableFontSizes[j];
 
 			if (!WMCountInArray(face->sizes, (void *)(uintptr_t) size)) {
@@ -676,6 +688,9 @@ static void typefaceClick(WMWidget * w, void *data)
 	int sizei = -1;
 	void *size;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	osize = WMGetTextFieldText(panel->sizT);
 
 	item = WMGetListSelectedItem(panel->typLs);
@@ -684,8 +699,10 @@ static void typefaceClick(WMWidget * w, void *data)
 	WMClearList(panel->sizLs);
 
 	WM_ITERATE_ARRAY(face->sizes, size, i) {
-		if ((uintptr_t)size != 0) {
-			sprintf(buffer, "%li", (uintptr_t)size);
+		if (size != NULL) {
+			int size_int = (int) size;
+
+			sprintf(buffer, "%i", size_int);
 
 			WMAddListItem(panel->sizLs, buffer);
 		}
@@ -715,6 +732,9 @@ static void sizeClick(WMWidget * w, void *data)
 	FontPanel *panel = (FontPanel *) data;
 	WMListItem *item;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	item = WMGetListSelectedItem(panel->sizLs);
 	WMSetTextFieldText(panel->sizT, item->text);
 
@@ -780,8 +800,11 @@ static void setFontPanelFontName(FontPanel * panel, const char *family, const ch
 
 	WM_ITERATE_ARRAY(face->sizes, vsize, i) {
 		char buffer[32];
-		if ((uintptr_t)vsize != 0) {
-			sprintf(buffer, "%li", (uintptr_t)vsize);
+
+		if (vsize != NULL) {
+			int size_int = (int) vsize;
+
+			sprintf(buffer, "%i", size_int);
 
 			WMAddListItem(panel->sizLs, buffer);
 		}
@@ -792,7 +815,6 @@ static void setFontPanelFontName(FontPanel * panel, const char *family, const ch
 	sz = WMFindRowOfListItemWithTitle(panel->sizLs, asize);
 
 	if (sz < 0) {
-		sz = 4;
 		return;
 	}
 
