@@ -64,8 +64,6 @@ typedef struct W_Scroller {
 	} flags;
 } Scroller;
 
-#define DEFAULT_HEIGHT		60
-#define DEFAULT_WIDTH		SCROLLER_WIDTH
 #define DEFAULT_ARROWS_POSITION	WSAMinEnd
 
 #define BUTTON_SIZE             ((SCROLLER_WIDTH) - 4)
@@ -108,7 +106,7 @@ WMScroller *WMCreateScroller(WMWidget * parent)
 	WMCreateEventHandler(sPtr->view, ExposureMask | StructureNotifyMask
 			     | ClientMessageMask, handleEvents, sPtr);
 
-	W_ResizeView(sPtr->view, DEFAULT_WIDTH, DEFAULT_WIDTH);
+	W_ResizeView(sPtr->view, SCROLLER_WIDTH, SCROLLER_WIDTH);
 	sPtr->flags.arrowsPosition = DEFAULT_ARROWS_POSITION;
 
 	WMCreateEventHandler(sPtr->view, ButtonPressMask | ButtonReleaseMask
@@ -134,6 +132,9 @@ static void willResizeScroller(W_ViewDelegate * self, WMView * view, unsigned in
 {
 	WMScroller *sPtr = (WMScroller *) view->self;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	if (*width > *height) {
 		sPtr->flags.horizontal = 1;
 		*height = SCROLLER_WIDTH;
@@ -167,10 +168,10 @@ void WMSetScrollerParameters(WMScroller * sPtr, float floatValue, float knobProp
 
 	assert(!isnan(floatValue));
 
-	if (floatValue < 0.0)
-		sPtr->floatValue = 0.0;
-	else if (floatValue > 1.0)
-		sPtr->floatValue = 1.0;
+	if (floatValue < 0.0F)
+		sPtr->floatValue = 0.0F;
+	else if (floatValue > 1.0F)
+		sPtr->floatValue = 1.0F;
 	else
 		sPtr->floatValue = floatValue;
 
@@ -179,9 +180,9 @@ void WMSetScrollerParameters(WMScroller * sPtr, float floatValue, float knobProp
 		sPtr->knobProportion = min_knob_proportion;
 		sPtr->flags.documentFullyVisible = 0;
 
-	} else if (knobProportion >= 1.0) {
+	} else if (knobProportion >= 1.0F) {
 
-		sPtr->knobProportion = 1.0;
+		sPtr->knobProportion = 1.0F;
 		sPtr->flags.documentFullyVisible = 1;
 
 	} else {
@@ -343,7 +344,7 @@ static int knobLength(Scroller * sPtr)
 		length -= 2 * (BUTTON_SIZE + 1);
 	}
 
-	tmp = (int)((float)length * sPtr->knobProportion + 0.5);
+	tmp = (int)((float)length * sPtr->knobProportion + 0.5F);
 	/* keep minimum size */
 	if (tmp < BUTTON_SIZE)
 		tmp = BUTTON_SIZE;
@@ -417,7 +418,7 @@ static void paintScroller(Scroller * sPtr)
 					       length - (int)(knobP + knobL), view->size.height - 4);
 		} else {
 			/* before */
-			if (knobP > 0.0)
+			if (knobP > 0.0F)
 				XFillRectangle(scr->display, d, scr->stippleGC,
 					       2, ofs, view->size.width - 4, (int)knobP);
 
