@@ -89,6 +89,7 @@ static const struct {
 	{ "LBCMaximizeKey", N_("Maximize active window left bottom corner") },
 	{ "RBCMaximizeKey", N_("Maximize active window right bottom corner") },
 	{ "MaximusKey",     N_("Maximus: Tiled maximization ") },
+	{ "OmnipresentKey", N_("Toggle window omnipresent status") },
 	{ "RaiseKey",       N_("Raise active window") },
 	{ "LowerKey",       N_("Lower active window") },
 	{ "RaiseLowerKey",  N_("Raise/Lower window under mouse pointer") },
@@ -101,6 +102,7 @@ static const struct {
 	{ "GroupPrevKey",   N_("Focus previous group window") },
 
 	/* Workspace Related */
+	{ "WorkspaceMapKey",  N_("Open workspace pager") },
 	{ "NextWorkspaceKey", N_("Switch to next workspace") },
 	{ "PrevWorkspaceKey", N_("Switch to previous workspace") },
 	{ "LastWorkspaceKey", N_("Switch to last used workspace") },
@@ -147,6 +149,7 @@ static const struct {
 	/* Misc. */
 	{ "WindowRelaunchKey", N_("Launch new instance of application") },
 	{ "ScreenSwitchKey",   N_("Switch to Next Screen/Monitor") },
+	{ "RunKey",            N_("Run application") },
 	{ "DockRaiseLowerKey", N_("Raise/Lower Dock") },
 	{ "ClipRaiseLowerKey", N_("Raise/Lower Clip") }
 #ifdef XKB_MODELOCK
@@ -262,21 +265,57 @@ static void XConvertCase(register KeySym sym, KeySym * lower, KeySym * upper)
 }
 #endif
 
+static int NumLockMask(Display *dpy)
+{
+	int i, mask;
+	XModifierKeymap *map;
+	static int mask_table[8] = {
+		ShiftMask, LockMask, ControlMask, Mod1Mask,
+		Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask
+	};
+	KeyCode numlock_keycode = XKeysymToKeycode(dpy, XK_Num_Lock);
+
+	if (numlock_keycode == NoSymbol)
+		return 0;
+
+	map = XGetModifierMapping(dpy);
+	if (!map)
+		return 0;
+
+	mask = 0;
+	for (i = 0; i < 8 * map->max_keypermod; i++) {
+		if (map->modifiermap[i] == numlock_keycode && mask == 0) {
+			mask = mask_table[i/map->max_keypermod];
+			break;
+		}
+	}
+
+	if (map)
+		XFreeModifiermap(map);
+
+	return mask;
+}
+
 char *capture_shortcut(Display *dpy, Bool *capturing, Bool convert_case)
 {
 	XEvent ev;
 	KeySym ksym, lksym, uksym;
 	char buffer[64];
 	char *key = NULL;
+	unsigned int numlock_mask;
 
 	while (*capturing) {
 		XAllowEvents(dpy, AsyncKeyboard, CurrentTime);
 		WMNextEvent(dpy, &ev);
 		if (ev.type == KeyPress && ev.xkey.keycode != 0) {
+			numlock_mask = NumLockMask(dpy);
+
 			if (xext_xkb_supported)
-				ksym = XkbKeycodeToKeysym(dpy, ev.xkey.keycode, 0, 0);
+				/* conditional mask check to get numeric keypad keys */
+				ksym = XkbKeycodeToKeysym(dpy, ev.xkey.keycode, 0, ev.xkey.state & numlock_mask?1:0);
 			else
 				ksym = XKeycodeToKeysym(dpy, ev.xkey.keycode, 0);
+
 			if (!IsModifierKey(ksym)) {
 				if (convert_case) {
 					XConvertCase(ksym, &lksym, &uksym);
@@ -297,28 +336,28 @@ char *capture_shortcut(Display *dpy, Bool *capturing, Bool convert_case)
 
 	buffer[0] = 0;
 
-	if (ev.xkey.state & ControlMask) {
+	if (ev.xkey.state & ControlMask)
 		strcat(buffer, "Control+");
-	}
-	if (ev.xkey.state & ShiftMask) {
+
+	if (ev.xkey.state & ShiftMask)
 		strcat(buffer, "Shift+");
-	}
-	if (ev.xkey.state & Mod1Mask) {
+
+	if ((numlock_mask != Mod1Mask) && (ev.xkey.state & Mod1Mask))
 		strcat(buffer, "Mod1+");
-	}
-	if (ev.xkey.state & Mod2Mask) {
+
+	if ((numlock_mask != Mod2Mask) && (ev.xkey.state & Mod2Mask))
 		strcat(buffer, "Mod2+");
-	}
-	if (ev.xkey.state & Mod3Mask) {
+
+	if ((numlock_mask != Mod3Mask) && (ev.xkey.state & Mod3Mask))
 		strcat(buffer, "Mod3+");
-	}
-	if (ev.xkey.state & Mod4Mask) {
+
+	if ((numlock_mask != Mod4Mask) && (ev.xkey.state & Mod4Mask))
 		strcat(buffer, "Mod4+");
-	}
-	if (ev.xkey.state & Mod5Mask) {
+
+	if ((numlock_mask != Mod5Mask) && (ev.xkey.state & Mod5Mask))
 		strcat(buffer, "Mod5+");
-	}
-	strcat(buffer, key);
+
+	wstrlcat(buffer, key, sizeof(buffer));
 
 	return wstrdup(buffer);
 }
@@ -362,6 +401,9 @@ static void clearShortcut(WMWidget * w, void *data)
 	_Panel *panel = (_Panel *) data;
 	int row = WMGetListSelectedItemRow(panel->actLs);
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	WMSetTextFieldText(panel->shoT, NULL);
 
 	if (row >= 0) {
@@ -377,6 +419,9 @@ static void typedKeys(void *observerData, WMNotification * notification)
 	_Panel *panel = (_Panel *) observerData;
 	int row = WMGetListSelectedItemRow(panel->actLs);
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) notification;
+
 	if (row < 0)
 		return;
 
@@ -470,8 +515,8 @@ static void createPanel(Panel * p)
 
 	/* **************** Actions **************** */
 	panel->actL = WMCreateLabel(panel->box);
-	WMResizeWidget(panel->actL, 280, 20);
-	WMMoveWidget(panel->actL, 20, 10);
+	WMResizeWidget(panel->actL, 314, 20);
+	WMMoveWidget(panel->actL, 9, 9);
 	WMSetLabelFont(panel->actL, boldFont);
 	WMSetLabelText(panel->actL, _("Actions"));
 	WMSetLabelRelief(panel->actL, WRSunken);
@@ -482,12 +527,12 @@ static void createPanel(Panel * p)
 	WMSetLabelTextColor(panel->actL, panel->white);
 
 	panel->actLs = WMCreateList(panel->box);
-	WMResizeWidget(panel->actLs, 280, 190);
-	WMMoveWidget(panel->actLs, 20, 32);
+	WMResizeWidget(panel->actLs, 314, 191);
+	WMMoveWidget(panel->actLs, 9, 31);
 	WMSetListUserDrawProc(panel->actLs, paintItem);
 	WMHangData(panel->actLs, panel);
 
-	for (i = 0; i < sizeof(keyOptions)/sizeof(keyOptions[0]); i++) {
+	for (i = 0; i < wlengthof(keyOptions); i++) {
 		WMAddListItem(panel->actLs, _(keyOptions[i].title));
 	}
 	WMSetListAction(panel->actLs, listClick, panel);
@@ -498,30 +543,30 @@ static void createPanel(Panel * p)
     /***************** Shortcut ****************/
 
 	panel->shoF = WMCreateFrame(panel->box);
-	WMResizeWidget(panel->shoF, 190, 210);
-	WMMoveWidget(panel->shoF, 315, 10);
+	WMResizeWidget(panel->shoF, 178, 214);
+	WMMoveWidget(panel->shoF, 333, 8);
 	WMSetFrameTitle(panel->shoF, _("Shortcut"));
 
 	panel->shoT = WMCreateTextField(panel->shoF);
 	WMResizeWidget(panel->shoT, 160, 20);
-	WMMoveWidget(panel->shoT, 15, 65);
+	WMMoveWidget(panel->shoT, 9, 65);
 	WMAddNotificationObserver(typedKeys, panel, WMTextDidChangeNotification, panel->shoT);
 
 	panel->cleB = WMCreateCommandButton(panel->shoF);
 	WMResizeWidget(panel->cleB, 75, 24);
-	WMMoveWidget(panel->cleB, 15, 95);
+	WMMoveWidget(panel->cleB, 9, 95);
 	WMSetButtonText(panel->cleB, _("Clear"));
 	WMSetButtonAction(panel->cleB, clearShortcut, panel);
 
 	panel->defB = WMCreateCommandButton(panel->shoF);
 	WMResizeWidget(panel->defB, 75, 24);
-	WMMoveWidget(panel->defB, 100, 95);
+	WMMoveWidget(panel->defB, 94, 95);
 	WMSetButtonText(panel->defB, _("Capture"));
 	WMSetButtonAction(panel->defB, captureClick, panel);
 
 	panel->instructionsL = WMCreateLabel(panel->shoF);
 	WMResizeWidget(panel->instructionsL, 160, 55);
-	WMMoveWidget(panel->instructionsL, 15, 140);
+	WMMoveWidget(panel->instructionsL, 9, 140);
 	WMSetLabelTextAlignment(panel->instructionsL, WACenter);
 	WMSetLabelWraps(panel->instructionsL, True);
 	WMSetLabelText(panel->instructionsL, _("Click on Capture to interactively define the shortcut key."));
@@ -559,7 +604,7 @@ static void storeData(_Panel * panel)
 	}
 }
 
-Panel *InitKeyboardShortcuts(WMScreen * scr, WMWidget * parent)
+Panel *InitKeyboardShortcuts(WMWidget *parent)
 {
 	_Panel *panel;
 
