@@ -357,10 +357,9 @@ static void sliderChangeCallback(WMWidget * w, void *data)
 {
 	TexturePanel *panel = (TexturePanel *) data;
 	RHSVColor hsv;
-	int row, rows;
+	int i, row, rows;
 	WMListItem *item;
 	RColor **colors;
-	int i;
 	RImage *image;
 	WMScreen *scr = WMWidgetScreen(w);
 
@@ -368,7 +367,20 @@ static void sliderChangeCallback(WMWidget * w, void *data)
 	hsv.saturation = WMGetSliderValue(panel->gsatS);
 	hsv.value = WMGetSliderValue(panel->gvalS);
 
+	rows = WMGetListNumberOfRows(panel->gcolL);
 	row = WMGetListSelectedItemRow(panel->gcolL);
+
+	if (row < 0 && rows > 0) {
+		row = 0;
+		WMSelectListItem(panel->gcolL, row);
+		item = WMGetListItem(panel->gcolL, row);
+		RRGBtoHSV((RColor *) item->clientData, &hsv);
+
+		WMSetSliderValue(panel->ghueS, hsv.hue);
+		WMSetSliderValue(panel->gsatS, hsv.saturation);
+		WMSetSliderValue(panel->gvalS, hsv.value);
+	}
+
 	if (row >= 0) {
 		RColor *rgb;
 
@@ -382,6 +394,7 @@ static void sliderChangeCallback(WMWidget * w, void *data)
 	}
 
 	if (w == panel->ghueS) {
+		updateHueSlider(panel->ghueS, panel->listFont, &hsv);
 		updateSVSlider(panel->gsatS, True, panel->listFont, &hsv);
 		updateSVSlider(panel->gvalS, False, panel->listFont, &hsv);
 	} else if (w == panel->gsatS) {
@@ -392,7 +405,6 @@ static void sliderChangeCallback(WMWidget * w, void *data)
 		updateSVSlider(panel->gsatS, True, panel->listFont, &hsv);
 	}
 
-	rows = WMGetListNumberOfRows(panel->gcolL);
 	if (rows == 0)
 		return;
 
@@ -460,6 +472,9 @@ static void gradAddCallback(WMWidget * w, void *data)
 	int row;
 	RColor *rgb;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	row = WMGetListSelectedItemRow(panel->gcolL) + 1;
 	item = WMInsertListItem(panel->gcolL, row, "00,00,00");
 	rgb = wmalloc(sizeof(RColor));
@@ -500,7 +515,10 @@ static void gradDeleteCallback(WMWidget * w, void *data)
 {
 	TexturePanel *panel = (TexturePanel *) data;
 	WMListItem *item;
-	int row;
+	int row, rows;
+
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
 
 	row = WMGetListSelectedItemRow(panel->gcolL);
 	if (row < 0)
@@ -510,9 +528,14 @@ static void gradDeleteCallback(WMWidget * w, void *data)
 	wfree(item->clientData);
 
 	WMRemoveListItem(panel->gcolL, row);
+	if (row > 0)
+		WMSelectListItem(panel->gcolL, row - 1);
+	else {
+		rows = WMGetListNumberOfRows(panel->gcolL);
+		if (rows > 0)
+			WMSelectListItem(panel->gcolL, 0);
 
-	WMSelectListItem(panel->gcolL, row - 1);
-
+	}
 	updateGradButtons(panel);
 
 	gradClickCallback(panel->gcolL, panel);
@@ -524,6 +547,9 @@ static void gradDeleteCallback(WMWidget * w, void *data)
 
 static void colorWellObserver(void *self, WMNotification * n)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) n;
+
 	updateSGradButtons(self);
 }
 
@@ -531,6 +557,9 @@ static void opaqChangeCallback(WMWidget * w, void *data)
 {
 	TexturePanel *panel = (TexturePanel *) data;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	updateTGradImage(panel);
 }
 
@@ -702,11 +731,6 @@ static void changeTypeCallback(WMWidget * w, void *data)
  * Public functions
  *--------------------------------------------------------------------------
  */
-void DestroyTexturePanel(TexturePanel * panel)
-{
-
-}
-
 void ShowTexturePanel(TexturePanel * panel)
 {
 	Display *dpy = WMScreenDisplay(WMWidgetScreen(panel->win));
@@ -817,8 +841,8 @@ void SetTexturePanelTexture(TexturePanel * panel, const char *name, WMPropList *
 			wfree(panel->imageFile);
 		panel->imageFile = wstrdup(WMGetFromPLString(WMGetFromPLArray(texture, 1)));
 
-		i = 180;
-		sscanf(WMGetFromPLString(WMGetFromPLArray(texture, 2)), "%i", &i);
+		if (sscanf(WMGetFromPLString(WMGetFromPLArray(texture, 2)), "%i", &i) != 1)
+			i = 180;
 		WMSetSliderValue(panel->topaS, i);
 
 		p = WMGetFromPLArray(texture, 3);
