@@ -4,6 +4,7 @@
  *
  *  Copyright (c) 1997-2003 Alfredo K. Kojima
  *  Copyright (c) 1998-2003 Dan Pascu
+ *  Copyright (c) 2014 Window Maker Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,33 +51,37 @@
 #include "misc.h"
 #include "event.h"
 
-/****** Global Variables ******/
 
-int ignore_wks_change = 0;
-extern Time LastTimestamp;
-extern Time LastFocusChange;
-extern Atom _XA_WM_TAKE_FOCUS;
+#ifndef HAVE_FLOAT_MATHFUNC
+#define sinf(x) ((float)sin((double)(x)))
+#define cosf(x) ((float)cos((double)(x)))
+#define sqrtf(x) ((float)sqrt((double)(x)))
+#define atan2f(y, x) ((float)atan((double)(y) / (double)(x)))
+#endif
 
 static void find_Maximus_geometry(WWindow *wwin, WArea usableArea, int *new_x, int *new_y,
 				  unsigned int *new_width, unsigned int *new_height);
 static void save_old_geometry(WWindow *wwin, int directions);
 
 /******* Local Variables *******/
+#ifdef USE_ANIMATIONS
 static struct {
 	int steps;
 	int delay;
 } shadePars[5] = {
-	{
-	SHADE_STEPS_UF, SHADE_DELAY_UF}, {
-	SHADE_STEPS_F, SHADE_DELAY_F}, {
-	SHADE_STEPS_M, SHADE_DELAY_M}, {
-	SHADE_STEPS_S, SHADE_DELAY_S}, {
-SHADE_STEPS_US, SHADE_DELAY_US}};
+	{ SHADE_STEPS_UF, SHADE_DELAY_UF },
+	{ SHADE_STEPS_F, SHADE_DELAY_F },
+	{ SHADE_STEPS_M, SHADE_DELAY_M },
+	{ SHADE_STEPS_S, SHADE_DELAY_S },
+	{ SHADE_STEPS_US, SHADE_DELAY_US }
+};
+
+#define SHADE_STEPS	shadePars[(int)wPreferences.shade_speed].steps
+#define SHADE_DELAY	shadePars[(int)wPreferences.shade_speed].delay
+#endif
 
 #define UNSHADE         0
 #define SHADE           1
-#define SHADE_STEPS	shadePars[(int)wPreferences.shade_speed].steps
-#define SHADE_DELAY	shadePars[(int)wPreferences.shade_speed].delay
 
 static int compareTimes(Time t1, Time t2)
 {
@@ -87,22 +92,30 @@ static int compareTimes(Time t1, Time t2)
 	return (diff < 60000) ? 1 : -1;
 }
 
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 static void shade_animate(WWindow *wwin, Bool what);
 #else
-static void shade_animate(WWindow *wwin, Bool what) { }
+static inline void shade_animate(WWindow *wwin, Bool what)
+{
+	/*
+	 * This function is empty on purpose, so tell the compiler
+	 * to not warn about parameters being not used
+	 */
+	(void) wwin;
+	(void) what;
+}
 #endif
 
 /*
  *----------------------------------------------------------------------
  * wSetFocusTo--
- * 	Changes the window focus to the one passed as argument.
+ *	Changes the window focus to the one passed as argument.
  * If the window to focus is not already focused, it will be brought
  * to the head of the list of windows. Previously focused window is
  * unfocused.
  *
  * Side effects:
- * 	Window list may be reordered and the window focus is changed.
+ *	Window list may be reordered and the window focus is changed.
  *
  *----------------------------------------------------------------------
  */
@@ -112,11 +125,11 @@ void wSetFocusTo(WScreen *scr, WWindow *wwin)
 
 	WWindow *old_focused;
 	WWindow *focused = scr->focused_window;
-	Time timestamp = LastTimestamp;
+	Time timestamp = w_global.timestamp.last_event;
 	WApplication *oapp = NULL, *napp = NULL;
 	int wasfocused;
 
-	if (scr->flags.ignore_focus_events || compareTimes(LastFocusChange, timestamp) > 0)
+	if (scr->flags.ignore_focus_events || compareTimes(w_global.timestamp.focus_change, timestamp) > 0)
 		return;
 
 	if (!old_scr)
@@ -124,7 +137,7 @@ void wSetFocusTo(WScreen *scr, WWindow *wwin)
 
 	old_focused = old_scr->focused_window;
 
-	LastFocusChange = timestamp;
+	w_global.timestamp.focus_change = timestamp;
 
 	if (old_focused)
 		oapp = wApplicationOf(old_focused->main_window);
@@ -174,7 +187,7 @@ void wSetFocusTo(WScreen *scr, WWindow *wwin)
 
 		XFlush(dpy);
 		if (wwin->protocols.TAKE_FOCUS)
-			wClientSendProtocol(wwin, _XA_WM_TAKE_FOCUS, timestamp);
+			wClientSendProtocol(wwin, w_global.atom.wm.take_focus, timestamp);
 
 		XSync(dpy, False);
 	} else {
@@ -251,7 +264,7 @@ void wShadeWindow(WWindow *wwin)
 
 	WMPostNotificationName(WMNChangedState, wwin, "shade");
 
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 	if (!wwin->screen_ptr->flags.startup) {
 		/* Catch up with events not processed while animation was running */
 		ProcessPendingEvents();
@@ -290,9 +303,9 @@ void wUnshadeWindow(WWindow *wwin)
 static void save_old_geometry(WWindow *wwin, int directions)
 {
 	/* never been saved? */
-	if (! wwin->old_geometry.width)
+	if (!wwin->old_geometry.width)
 		directions |= SAVE_GEOMETRY_X | SAVE_GEOMETRY_WIDTH;
-	if (! wwin->old_geometry.height)
+	if (!wwin->old_geometry.height)
 		directions |= SAVE_GEOMETRY_Y | SAVE_GEOMETRY_HEIGHT;
 
 	if (directions & SAVE_GEOMETRY_X)
@@ -353,6 +366,7 @@ void wMaximizeWindow(WWindow *wwin, int directions)
 	WArea usableArea, totalArea;
 	Bool has_border = 1;
 	int adj_size;
+	WScreen *scr = wwin->screen_ptr;
 
 	if (!IS_RESIZABLE(wwin))
 		return;
@@ -360,15 +374,18 @@ void wMaximizeWindow(WWindow *wwin, int directions)
 	if (!HAS_BORDER(wwin))
 		has_border = 0;
 
+	if (wPreferences.drag_maximized_window == DRAGMAX_NOMOVE)
+		wwin->client_flags.no_movable = 1;
+
 	/* the size to adjust the geometry */
-	adj_size = wwin->screen_ptr->frame_border_width * 2 * has_border;
+	adj_size = scr->frame_border_width * 2 * has_border;
 
 	/* save old coordinates before we change the current values */
 	if (!wwin->flags.maximized)
 		save_old_geometry(wwin, SAVE_GEOMETRY_ALL);
 
-	totalArea.x2 = wwin->screen_ptr->scr_width;
-	totalArea.y2 = wwin->screen_ptr->scr_height;
+	totalArea.x2 = scr->scr_width;
+	totalArea.y2 = scr->scr_height;
 	totalArea.x1 = 0;
 	totalArea.y1 = 0;
 	usableArea = totalArea;
@@ -385,12 +402,12 @@ void wMaximizeWindow(WWindow *wwin, int directions)
 		usableArea = wGetUsableAreaForHead(scr, head, &totalArea, True);
 	}
 
+
 	/* Only save directions, not kbd or xinerama hints */
 	directions &= (MAX_HORIZONTAL | MAX_VERTICAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_TOPHALF | MAX_BOTTOMHALF | MAX_MAXIMUS);
 
-	if (WFLAGP(wwin, full_maximize)) {
+	if (WFLAGP(wwin, full_maximize))
 		usableArea = totalArea;
-	}
 	half_scr_width = (usableArea.x2 - usableArea.x1)/2;
 	half_scr_height = (usableArea.y2 - usableArea.y1)/2;
 
@@ -581,8 +598,7 @@ static void find_Maximus_geometry(WWindow *wwin, WArea usableArea, int *new_x, i
 		remember_geometry(wwin, &orig.left, &orig.top, &orig.width, &orig.height);
 		orig.bottom = orig.top + orig.height;
 		orig.right = orig.left + orig.width;
-	}
-	else
+	} else
 		set_window_coords(wwin, &orig);
 
 	/* Try to fully maximize first, then readjust later */
@@ -662,7 +678,7 @@ static void find_Maximus_geometry(WWindow *wwin, WArea usableArea, int *new_x, i
 	*new_x = new.left;
 	*new_y = new.top;
 	/* xcalc needs -7 here, but other apps don't */
-	*new_height = new.bottom - new.top - adjust_height - 1;;
+	*new_height = new.bottom - new.top - adjust_height - 1;
 	*new_width = new.right - new.left;
 }
 
@@ -678,6 +694,9 @@ void wUnmaximizeWindow(WWindow *wwin)
 		wUnshadeWindow(wwin);
 	}
 
+	if (wPreferences.drag_maximized_window == DRAGMAX_NOMOVE)
+		wwin->client_flags.no_movable = 0;
+
 	/* Use old coordinates if they are set, current values otherwise */
 	remember_geometry(wwin, &x, &y, &w, &h);
 
@@ -707,7 +726,8 @@ void wFullscreenWindow(WWindow *wwin)
 
 	wWindowConfigureBorders(wwin);
 
-	ChangeStackingLevel(wwin->frame->core, WMFullscreenLevel);
+	ChangeStackingLevel(wwin->frame->core, WMNormalLevel);
+	wRaiseFrame(wwin->frame->core);
 
 	wwin->bfs_geometry.x = wwin->frame_x;
 	wwin->bfs_geometry.y = wwin->frame_y;
@@ -718,6 +738,9 @@ void wFullscreenWindow(WWindow *wwin)
 	rect = wGetRectForHead(wwin->screen_ptr, head);
 	wWindowConfigure(wwin, rect.pos.x, rect.pos.y, rect.size.width, rect.size.height);
 
+	wwin->screen_ptr->bfs_focused_window = wwin->screen_ptr->focused_window;
+	wSetFocusTo(wwin->screen_ptr, wwin);
+
 	WMPostNotificationName(WMNChangedState, wwin, "fullscreen");
 }
 
@@ -728,15 +751,10 @@ void wUnfullscreenWindow(WWindow *wwin)
 
 	wwin->flags.fullscreen = False;
 
-	if (wwin->frame->core->stacking->window_level == WMFullscreenLevel) {
-		if (WFLAGP(wwin, sunken)) {
-			ChangeStackingLevel(wwin->frame->core, WMSunkenLevel);
-		} else if (WFLAGP(wwin, floating)) {
-			ChangeStackingLevel(wwin->frame->core, WMFloatingLevel);
-		} else {
-			ChangeStackingLevel(wwin->frame->core, WMNormalLevel);
-		}
-	}
+	if (WFLAGP(wwin, sunken))
+		ChangeStackingLevel(wwin->frame->core, WMSunkenLevel);
+	else if (WFLAGP(wwin, floating))
+		ChangeStackingLevel(wwin->frame->core, WMFloatingLevel);
 
 	wWindowConfigure(wwin, wwin->bfs_geometry.x, wwin->bfs_geometry.y,
 			 wwin->bfs_geometry.width, wwin->bfs_geometry.height);
@@ -748,9 +766,14 @@ void wUnfullscreenWindow(WWindow *wwin)
 	 */
 
 	WMPostNotificationName(WMNChangedState, wwin, "fullscreen");
+
+	if (wwin->screen_ptr->bfs_focused_window) {
+		wSetFocusTo(wwin->screen_ptr, wwin->screen_ptr->bfs_focused_window);
+		wwin->screen_ptr->bfs_focused_window = NULL;
+	}
 }
 
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 static void animateResizeFlip(WScreen *scr, int x, int y, int w, int h, int fx, int fy, int fw, int fh, int steps)
 {
 #define FRAMES (MINIATURIZE_ANIMATION_FRAMES_F)
@@ -776,8 +799,8 @@ static void animateResizeFlip(WScreen *scr, int x, int y, int w, int h, int fx,
 		if (angle > final_angle)
 			angle = final_angle;
 
-		dx = (cw / 10) - ((cw / 5) * sin(angle));
-		dch = (ch / 2) * cos(angle);
+		dx = (cw / 10) - ((cw / 5) * sinf(angle));
+		dch = (ch / 2) * cosf(angle);
 		midy = cy + (ch / 2);
 
 		points[0].x = cx + dx;
@@ -841,19 +864,19 @@ animateResizeTwist(WScreen *scr, int x, int y, int w, int h, int fx, int fy, int
 		if (angle > final_angle)
 			angle = final_angle;
 
-		a = atan(ch / cw);
-		d = sqrt((cw / 2) * (cw / 2) + (ch / 2) * (ch / 2));
+		a = atan2f(ch, cw);
+		d = sqrtf((cw / 2) * (cw / 2) + (ch / 2) * (ch / 2));
 
-		points[0].x = cx + cos(angle - a) * d;
-		points[0].y = cy + sin(angle - a) * d;
-		points[1].x = cx + cos(angle + a) * d;
-		points[1].y = cy + sin(angle + a) * d;
-		points[2].x = cx + cos(angle - a + WM_PI) * d;
-		points[2].y = cy + sin(angle - a + WM_PI) * d;
-		points[3].x = cx + cos(angle + a + WM_PI) * d;
-		points[3].y = cy + sin(angle + a + WM_PI) * d;
-		points[4].x = cx + cos(angle - a) * d;
-		points[4].y = cy + sin(angle - a) * d;
+		points[0].x = cx + cosf(angle - a) * d;
+		points[0].y = cy + sinf(angle - a) * d;
+		points[1].x = cx + cosf(angle + a) * d;
+		points[1].y = cy + sinf(angle + a) * d;
+		points[2].x = cx + cosf(angle - a + (float)WM_PI) * d;
+		points[2].y = cy + sinf(angle - a + (float)WM_PI) * d;
+		points[3].x = cx + cosf(angle + a + (float)WM_PI) * d;
+		points[3].y = cy + sinf(angle + a + (float)WM_PI) * d;
+		points[4].x = cx + cosf(angle - a) * d;
+		points[4].y = cy + sinf(angle - a) * d;
 		XGrabServer(dpy);
 		XDrawLines(dpy, scr->root_win, scr->frame_gc, points, 5, CoordModeOrigin);
 		XFlush(dpy);
@@ -918,15 +941,13 @@ static void animateResizeZoom(WScreen *scr, int x, int y, int w, int h, int fx,
 		}
 	}
 
-	for (j = 0; j < FRAMES; j++) {
+	for (j = 0; j < FRAMES; j++)
 		XDrawRectangle(dpy, scr->root_win, scr->frame_gc, (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
-	}
 	XFlush(dpy);
 	wusleep(MINIATURIZE_ANIMATION_DELAY_Z);
 
-	for (j = 0; j < FRAMES; j++) {
+	for (j = 0; j < FRAMES; j++)
 		XDrawRectangle(dpy, scr->root_win, scr->frame_gc, (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
-	}
 
 	XUngrabServer(dpy);
 }
@@ -941,9 +962,8 @@ void animateResize(WScreen *scr, int x, int y, int w, int h, int fx, int fy, int
 	if (style == WIS_NONE)
 		return;
 
-	if (style == WIS_RANDOM) {
+	if (style == WIS_RANDOM)
 		style = rand() % 3;
-	}
 
 	switch (style) {
 	case WIS_TWIST:
@@ -964,7 +984,7 @@ void animateResize(WScreen *scr, int x, int y, int w, int h, int fx, int fy, int
 		break;
 	}
 }
-#endif				/* ANIMATIONS */
+#endif /* USE_ANIMATIONS */
 
 static void flushExpose(void)
 {
@@ -986,11 +1006,10 @@ static void unmapTransientsFor(WWindow *wwin)
 		    && (tmp->flags.mapped || wwin->screen_ptr->flags.startup || tmp->flags.shaded)) {
 			unmapTransientsFor(tmp);
 			tmp->flags.miniaturized = 1;
-			if (!tmp->flags.shaded) {
+			if (!tmp->flags.shaded)
 				wWindowUnmap(tmp);
-			} else {
+			else
 				XUnmapWindow(dpy, tmp->frame->core->window);
-			}
 			/*
 			   if (!tmp->flags.shaded)
 			 */
@@ -1013,11 +1032,10 @@ static void mapTransientsFor(WWindow *wwin)
 		    && tmp->icon == NULL) {
 			mapTransientsFor(tmp);
 			tmp->flags.miniaturized = 0;
-			if (!tmp->flags.shaded) {
+			if (!tmp->flags.shaded)
 				wWindowMap(tmp);
-			} else {
+			else
 				XMapWindow(dpy, tmp->frame->core->window);
-			}
 			tmp->flags.semi_focused = 0;
 			/*
 			   if (!tmp->flags.shaded)
@@ -1030,7 +1048,7 @@ static void mapTransientsFor(WWindow *wwin)
 	}
 }
 
-static WWindow *recursiveTransientFor(WWindow * wwin)
+static WWindow *recursiveTransientFor(WWindow *wwin)
 {
 	int i;
 
@@ -1045,14 +1063,44 @@ static WWindow *recursiveTransientFor(WWindow * wwin)
 		i--;
 	}
 	if (i == 0 && wwin) {
-		wwarning("%s has a severely broken WM_TRANSIENT_FOR hint.", wwin->frame->title);
+		wwarning(_("window \"%s\" has a severely broken WM_TRANSIENT_FOR hint"),
+		         wwin->frame->title);
 		return NULL;
 	}
 
 	return wwin;
 }
 
-void wIconifyWindow(WWindow * wwin)
+#ifdef USE_ANIMATIONS
+static int getAnimationGeometry(WWindow *wwin, int *ix, int *iy, int *iw, int *ih)
+{
+	if (wwin->screen_ptr->flags.startup || wPreferences.no_animations
+		|| wwin->flags.skip_next_animation || wwin->icon == NULL)
+		return 0;
+
+	if (!wPreferences.disable_miniwindows && !wwin->flags.net_handle_icon) {
+		*ix = wwin->icon_x;
+		*iy = wwin->icon_y;
+		*iw = wwin->icon->core->width;
+		*ih = wwin->icon->core->height;
+	} else {
+		if (wwin->flags.net_handle_icon) {
+			*ix = wwin->icon_x;
+			*iy = wwin->icon_y;
+			*iw = wwin->icon_w;
+			*ih = wwin->icon_h;
+		} else {
+			*ix = 0;
+			*iy = 0;
+			*iw = wwin->screen_ptr->scr_width;
+			*ih = wwin->screen_ptr->scr_height;
+		}
+	}
+	return 1;
+}
+#endif	/* USE_ANIMATIONS */
+
+void wIconifyWindow(WWindow *wwin)
 {
 	XWindowAttributes attribs;
 	int present;
@@ -1086,6 +1134,49 @@ void wIconifyWindow(WWindow * wwin)
 
 		wwin->icon = icon_create_for_wwindow(wwin);
 		wwin->icon->mapped = 1;
+
+		/* extract the window screenshot everytime, as the option can be enable anytime */
+		if (wwin->client_win && wwin->flags.mapped) {
+			RImage *mini_preview;
+			XImage *pimg;
+			unsigned int w, h;
+			int x, y;
+			Window baz;
+
+			XRaiseWindow(dpy, wwin->frame->core->window);
+			XTranslateCoordinates(dpy, wwin->client_win, wwin->screen_ptr->root_win, 0, 0, &x, &y, &baz);
+
+			w = attribs.width;
+			h = attribs.height;
+
+			if (x - attribs.x + attribs.width > wwin->screen_ptr->scr_width)
+				w = wwin->screen_ptr->scr_width - x + attribs.x;
+
+			if (y - attribs.y + attribs.height > wwin->screen_ptr->scr_height)
+				h = wwin->screen_ptr->scr_height - y + attribs.y;
+
+			pimg = XGetImage(dpy, wwin->client_win, 0, 0, w, h, AllPlanes, ZPixmap);
+			if (pimg) {
+				mini_preview = RCreateImageFromXImage(wwin->screen_ptr->rcontext, pimg, NULL);
+				XDestroyImage(pimg);
+
+				if (mini_preview) {
+					set_icon_minipreview(wwin->icon, mini_preview);
+					RReleaseImage(mini_preview);
+				} else {
+					const char *title;
+					char title_buf[32];
+
+					if (wwin->frame->title) {
+						title = wwin->frame->title;
+					} else {
+						snprintf(title_buf, sizeof(title_buf), "(id=0x%lx)", wwin->client_win);
+						title = title_buf;
+					}
+					wwarning(_("creation of mini-preview failed for window \"%s\""), title);
+				}
+			}
+		}
 	}
 
 	wwin->flags.miniaturized = 1;
@@ -1095,6 +1186,9 @@ void wIconifyWindow(WWindow * wwin)
 	unmapTransientsFor(wwin);
 
 	if (present) {
+#ifdef USE_ANIMATIONS
+		int ix, iy, iw, ih;
+#endif
 		XUngrabPointer(dpy, CurrentTime);
 		wWindowUnmap(wwin);
 		/* let all Expose events arrive so that we can repaint
@@ -1107,32 +1201,10 @@ void wIconifyWindow(WWindow * wwin)
 			wClientSetState(wwin, IconicState, wwin->icon->icon_win);
 
 		flushExpose();
-#ifdef ANIMATIONS
-		if (!wwin->screen_ptr->flags.startup && !wwin->flags.skip_next_animation
-		    && !wPreferences.no_animations) {
-			int ix, iy, iw, ih;
-
-			if (!wPreferences.disable_miniwindows && !wwin->flags.net_handle_icon) {
-				ix = wwin->icon_x;
-				iy = wwin->icon_y;
-				iw = wwin->icon->core->width;
-				ih = wwin->icon->core->height;
-			} else {
-				if (wwin->flags.net_handle_icon) {
-					ix = wwin->icon_x;
-					iy = wwin->icon_y;
-					iw = wwin->icon_w;
-					ih = wwin->icon_h;
-				} else {
-					ix = 0;
-					iy = 0;
-					iw = wwin->screen_ptr->scr_width;
-					ih = wwin->screen_ptr->scr_height;
-				}
-			}
+#ifdef USE_ANIMATIONS
+		if (getAnimationGeometry(wwin, &ix, &iy, &iw, &ih))
 			animateResize(wwin->screen_ptr, wwin->frame_x, wwin->frame_y,
 				      wwin->frame->core->width, wwin->frame->core->height, ix, iy, iw, ih);
-		}
 #endif
 	}
 
@@ -1171,7 +1243,7 @@ void wIconifyWindow(WWindow * wwin)
 		} else if (wPreferences.focus_mode != WKF_CLICK) {
 			wSetFocusTo(wwin->screen_ptr, NULL);
 		}
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 		if (!wwin->screen_ptr->flags.startup) {
 			/* Catch up with events not processed while animation was running */
 			Window clientwin = wwin->client_win;
@@ -1179,9 +1251,8 @@ void wIconifyWindow(WWindow * wwin)
 			ProcessPendingEvents();
 
 			/* the window can disappear while ProcessPendingEvents() runs */
-			if (!wWindowFor(clientwin)) {
+			if (!wWindowFor(clientwin))
 				return;
-			}
 		}
 #endif
 	}
@@ -1202,7 +1273,7 @@ void wIconifyWindow(WWindow * wwin)
 void wDeiconifyWindow(WWindow *wwin)
 {
 	/* Let's avoid changing workspace while deiconifying */
-	ignore_wks_change = 1;
+	w_global.ignore_workspace_change = True;
 
 	/* we're hiding for show_desktop */
 	int netwm_hidden = wwin->flags.net_show_desktop &&
@@ -1212,7 +1283,7 @@ void wDeiconifyWindow(WWindow *wwin)
 		wWindowChangeWorkspace(wwin, wwin->screen_ptr->current_workspace);
 
 	if (!wwin->flags.miniaturized) {
-		ignore_wks_change = 0;
+		w_global.ignore_workspace_change = False;
 		return;
 	}
 
@@ -1223,7 +1294,7 @@ void wDeiconifyWindow(WWindow *wwin)
 			wDeiconifyWindow(owner);
 			wSetFocusTo(wwin->screen_ptr, wwin);
 			wRaiseFrame(wwin->frame->core);
-			ignore_wks_change = 0;
+			w_global.ignore_workspace_change = False;
 			return;
 		}
 	}
@@ -1248,35 +1319,13 @@ void wDeiconifyWindow(WWindow *wwin)
 
 	/* if the window is in another workspace, do it silently */
 	if (!netwm_hidden) {
-#ifdef ANIMATIONS
-		if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations
-		    && !wwin->flags.skip_next_animation && wwin->icon != NULL) {
-			int ix, iy, iw, ih;
-
-			if (!wPreferences.disable_miniwindows
-			    && !wwin->flags.net_handle_icon) {
-				ix = wwin->icon_x;
-				iy = wwin->icon_y;
-				iw = wwin->icon->core->width;
-				ih = wwin->icon->core->height;
-			} else {
-				if (wwin->flags.net_handle_icon) {
-					ix = wwin->icon_x;
-					iy = wwin->icon_y;
-					iw = wwin->icon_w;
-					ih = wwin->icon_h;
-				} else {
-					ix = 0;
-					iy = 0;
-					iw = wwin->screen_ptr->scr_width;
-					ih = wwin->screen_ptr->scr_height;
-				}
-			}
+#ifdef USE_ANIMATIONS
+		int ix, iy, iw, ih;
+		if (getAnimationGeometry(wwin, &ix, &iy, &iw, &ih))
 			animateResize(wwin->screen_ptr, ix, iy, iw, ih,
 				      wwin->frame_x, wwin->frame_y,
 				      wwin->frame->core->width, wwin->frame->core->height);
-		}
-#endif	/* ANIMATIONS */
+#endif
 		wwin->flags.skip_next_animation = 0;
 		XGrabServer(dpy);
 		if (!wwin->flags.shaded)
@@ -1293,7 +1342,7 @@ void wDeiconifyWindow(WWindow *wwin)
 	if (!wPreferences.disable_miniwindows && wwin->icon != NULL
 	    && !wwin->flags.net_handle_icon) {
 		RemoveFromStackList(wwin->icon->core);
-		/*    removeIconGrabs(wwin->icon); */
+		wSetFocusTo(wwin->screen_ptr, wwin);
 		wIconDestroy(wwin->icon);
 		wwin->icon = NULL;
 	}
@@ -1303,7 +1352,7 @@ void wDeiconifyWindow(WWindow *wwin)
 
 		wSetFocusTo(wwin->screen_ptr, wwin);
 
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 		if (!wwin->screen_ptr->flags.startup) {
 			/* Catch up with events not processed while animation was running */
 			Window clientwin = wwin->client_win;
@@ -1312,7 +1361,7 @@ void wDeiconifyWindow(WWindow *wwin)
 
 			/* the window can disappear while ProcessPendingEvents() runs */
 			if (!wWindowFor(clientwin)) {
-				ignore_wks_change = 0;
+				w_global.ignore_workspace_change = False;
 				return;
 			}
 		}
@@ -1328,7 +1377,7 @@ void wDeiconifyWindow(WWindow *wwin)
 	if (!netwm_hidden)
 		wUnshadeWindow(wwin);
 
-	ignore_wks_change = 0;
+	w_global.ignore_workspace_change = False;
 }
 
 static void hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
@@ -1344,9 +1393,8 @@ static void hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int a
 		return;
 	}
 
-	if (wwin->flags.inspector_open) {
+	if (wwin->flags.inspector_open)
 		wHideInspectorForWindow(wwin);
-	}
 
 	wwin->flags.hidden = 1;
 	wWindowUnmap(wwin);
@@ -1354,13 +1402,18 @@ static void hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int a
 	wClientSetState(wwin, IconicState, icon->icon_win);
 	flushExpose();
 
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 	if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations &&
 	    !wwin->flags.skip_next_animation && animate) {
 		animateResize(wwin->screen_ptr, wwin->frame_x, wwin->frame_y,
 			      wwin->frame->core->width, wwin->frame->core->height,
 			      icon_x, icon_y, icon->core->width, icon->core->height);
 	}
+#else
+	/* Tell the compiler it is normal that those parameters are not used in this case */
+	(void) icon_x;
+	(void) icon_y;
+	(void) animate;
 #endif
 	wwin->flags.skip_next_animation = 0;
 
@@ -1486,9 +1539,8 @@ void wHideApplication(WApplication *wapp)
 
 	while (wlist) {
 		if (wlist->main_window == wapp->main_window) {
-			if (wlist->flags.focused) {
+			if (wlist->flags.focused)
 				hadfocus = 1;
-			}
 			if (wapp->app_icon) {
 				hideWindow(wapp->app_icon->icon, wapp->app_icon->x_pos,
 					   wapp->app_icon->y_pos, wlist, animate);
@@ -1533,13 +1585,19 @@ static void unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int
 
 	wwin->flags.hidden = 0;
 
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 	if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations && animate) {
 		animateResize(wwin->screen_ptr, icon_x, icon_y,
 			      icon->core->width, icon->core->height,
 			      wwin->frame_x, wwin->frame_y,
 			      wwin->frame->core->width, wwin->frame->core->height);
 	}
+#else
+	/* Tell the compiler it is normal that those parameters are not used in this case */
+	(void) icon;
+	(void) icon_x;
+	(void) icon_y;
+	(void) animate;
 #endif
 	wwin->flags.skip_next_animation = 0;
 	if (wwin->screen_ptr->current_workspace == wwin->frame->workspace) {
@@ -1549,9 +1607,8 @@ static void unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int
 		wwin->flags.mapped = 1;
 		wRaiseFrame(wwin->frame->core);
 	}
-	if (wwin->flags.inspector_open) {
+	if (wwin->flags.inspector_open)
 		wUnhideInspectorForWindow(wwin);
-	}
 
 	WMPostNotificationName(WMNChangedState, wwin, "hide");
 }
@@ -1593,9 +1650,8 @@ void wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurren
 						int x, y;
 
 						PlaceIcon(scr, &x, &y, wGetHeadForWindow(wlist));
-						if (wlist->icon_x != x || wlist->icon_y != y) {
+						if (wlist->icon_x != x || wlist->icon_y != y)
 							XMoveWindow(dpy, wlist->icon->core->window, x, y);
-						}
 						wlist->icon_x = x;
 						wlist->icon_y = y;
 						XMapWindow(dpy, wlist->icon->core->window);
@@ -1606,9 +1662,8 @@ void wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurren
 				if (bringToCurrentWS)
 					wWindowChangeWorkspace(wlist, scr->current_workspace);
 				wlist->flags.hidden = 0;
-				if (miniwindows && wlist->frame->workspace == scr->current_workspace) {
+				if (miniwindows && wlist->frame->workspace == scr->current_workspace)
 					wDeiconifyWindow(wlist);
-				}
 				WMPostNotificationName(WMNChangedState, wlist, "hide");
 			} else if (wlist->flags.shaded) {
 				if (bringToCurrentWS)
@@ -1617,9 +1672,8 @@ void wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurren
 				wRaiseFrame(wlist->frame->core);
 				if (wlist->frame->workspace == scr->current_workspace) {
 					XMapWindow(dpy, wlist->frame->core->window);
-					if (miniwindows) {
+					if (miniwindows)
 						wUnshadeWindow(wlist);
-					}
 				}
 				WMPostNotificationName(WMNChangedState, wlist, "hide");
 			} else if (wlist->flags.hidden) {
@@ -1627,9 +1681,9 @@ void wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurren
 					     wapp->app_icon->y_pos, wlist, animate, bringToCurrentWS);
 				animate = False;
 			} else {
-				if (bringToCurrentWS && wlist->frame->workspace != scr->current_workspace) {
+				if (bringToCurrentWS && wlist->frame->workspace != scr->current_workspace)
 					wWindowChangeWorkspace(wlist, scr->current_workspace);
-				}
+
 				wRaiseFrame(wlist->frame->core);
 			}
 		}
@@ -1723,7 +1777,19 @@ void wArrangeIcons(WScreen *scr, Bool arrangeAll)
 
 	for (head = 0; head < heads; ++head) {
 		WArea area = wGetUsableAreaForHead(scr, head, NULL, False);
-		WMRect rect = wmkrect(area.x1, area.y1, area.x2 - area.x1, area.y2 - area.y1);
+		WMRect rect;
+
+		if (scr->dock) {
+			int offset = wPreferences.icon_size + DOCK_EXTRA_SPACE;
+
+			if (scr->dock->on_right_side)
+				area.x2 -= offset;
+			else
+				area.x1 += offset;
+		}
+
+		rect = wmkrect(area.x1, area.y1, area.x2 - area.x1, area.y2 - area.y1);
+
 		vars[head].pi = vars[head].si = 0;
 		vars[head].sx1 = rect.pos.x;
 		vars[head].sy1 = rect.pos.y;
@@ -1761,12 +1827,12 @@ void wArrangeIcons(WScreen *scr, Bool arrangeAll)
 	}
 
 #define X ((wPreferences.icon_yard & IY_VERT) \
-    ? vars[head].xo + vars[head].xs*(vars[head].si*isize) \
-    : vars[head].xo + vars[head].xs*(vars[head].pi*isize))
+	? vars[head].xo + vars[head].xs*(vars[head].si*isize) \
+	: vars[head].xo + vars[head].xs*(vars[head].pi*isize))
 
 #define Y ((wPreferences.icon_yard & IY_VERT) \
-    ? vars[head].yo + vars[head].ys*(vars[head].pi*isize) \
-    : vars[head].yo + vars[head].ys*(vars[head].si*isize))
+	? vars[head].yo + vars[head].ys*(vars[head].pi*isize) \
+	: vars[head].yo + vars[head].ys*(vars[head].si*isize))
 
 	/* arrange application icons */
 	aicon = scr->app_icon_list;
@@ -1782,10 +1848,10 @@ void wArrangeIcons(WScreen *scr, Bool arrangeAll)
 			head = wGetHeadForWindow(aicon->icon->owner);
 
 			if (aicon->x_pos != X || aicon->y_pos != Y) {
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 				if (!wPreferences.no_animations)
-					SlideWindow(aicon->icon->core->window, aicon->x_pos, aicon->y_pos, X, Y);
-#endif				/* ANIMATIONS */
+					slide_window(aicon->icon->core->window, aicon->x_pos, aicon->y_pos, X, Y);
+#endif /* USE_ANIMATIONS */
 			}
 			wAppIconMove(aicon, X, Y);
 			vars[head].pi++;
@@ -1824,9 +1890,8 @@ void wArrangeIcons(WScreen *scr, Bool arrangeAll)
 				}
 			}
 		}
-		if (arrangeAll) {
+		if (arrangeAll)
 			wwin->flags.icon_moved = 0;
-		}
 		/* we reversed the order, so we use next */
 		wwin = wwin->next;
 	}
@@ -1845,27 +1910,31 @@ void wSelectWindow(WWindow *wwin, Bool flag)
 		else
 			XSetWindowBorder(dpy, wwin->frame->core->window, scr->white_pixel);
 
-		if (!HAS_BORDER(wwin)) {
+		if (!HAS_BORDER(wwin))
 			XSetWindowBorderWidth(dpy, wwin->frame->core->window, wwin->screen_ptr->frame_border_width);
-		}
 
 		if (!scr->selected_windows)
 			scr->selected_windows = WMCreateArray(4);
 		WMAddToArray(scr->selected_windows, wwin);
 	} else {
 		wwin->flags.selected = 0;
-		if (wwin->frame->border_pixel)
-			XSetWindowBorder(dpy, wwin->frame->core->window, *wwin->frame->border_pixel);
-		else
-			XSetWindowBorder(dpy, wwin->frame->core->window, scr->frame_border_pixel);
+		if (wwin->flags.focused) {
+			if (wwin->frame->focused_border_pixel)
+				XSetWindowBorder(dpy, wwin->frame->core->window, *wwin->frame->focused_border_pixel);
+			else
+				XSetWindowBorder(dpy, wwin->frame->core->window, scr->frame_focused_border_pixel);
+		} else {
+			if (wwin->frame->border_pixel)
+				XSetWindowBorder(dpy, wwin->frame->core->window, *wwin->frame->border_pixel);
+			else
+				XSetWindowBorder(dpy, wwin->frame->core->window, scr->frame_border_pixel);
+		}
 
-		if (!HAS_BORDER(wwin)) {
+		if (!HAS_BORDER(wwin))
 			XSetWindowBorderWidth(dpy, wwin->frame->core->window, 0);
-		}
 
-		if (scr->selected_windows) {
+		if (scr->selected_windows)
 			WMRemoveFromArray(scr->selected_windows, wwin);
-		}
 	}
 }
 
@@ -1874,9 +1943,9 @@ void wMakeWindowVisible(WWindow *wwin)
 	if (wwin->frame->workspace != wwin->screen_ptr->current_workspace)
 		wWorkspaceChange(wwin->screen_ptr, wwin->frame->workspace);
 
-	if (wwin->flags.shaded) {
+	if (wwin->flags.shaded)
 		wUnshadeWindow(wwin);
-	}
+
 	if (wwin->flags.hidden) {
 		WApplication *app;
 
@@ -1900,7 +1969,7 @@ void wMakeWindowVisible(WWindow *wwin)
  * Do the animation while shading (called with what = SHADE)
  * or unshading (what = UNSHADE).
  */
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 static void shade_animate(WWindow *wwin, Bool what)
 {
 	int y, s, w, h;
@@ -1909,7 +1978,7 @@ static void shade_animate(WWindow *wwin, Bool what)
 	if (wwin->flags.skip_next_animation || wPreferences.no_animations)
 		return;
 
-	switch(what) {
+	switch (what) {
 	case SHADE:
 		if (!wwin->screen_ptr->flags.startup) {
 			/* do the shading animation */
@@ -1927,11 +1996,10 @@ static void shade_animate(WWindow *wwin, Bool what)
 				if (time(NULL) - time0 > MAX_ANIMATION_TIME)
 					break;
 
-				if (SHADE_DELAY > 0) {
-				wusleep(SHADE_DELAY * 1000L);
-				} else {
+				if (SHADE_DELAY > 0)
+					wusleep(SHADE_DELAY * 1000L);
+				else
 					wusleep(10);
-				}
 				h -= s;
 			y -= s;
 			}
@@ -1952,11 +2020,10 @@ static void shade_animate(WWindow *wwin, Bool what)
 				XResizeWindow(dpy, wwin->frame->core->window, w, h);
 				XMoveWindow(dpy, wwin->client_win, 0, y);
 				XFlush(dpy);
-				if (SHADE_DELAY > 0) {
+				if (SHADE_DELAY > 0)
 					wusleep(SHADE_DELAY * 2000L / 3);
-				} else {
+				else
 					wusleep(10);
-				}
 				h += s;
 				y += s;
 
