@@ -51,7 +51,7 @@
 #include "placement.h"
 #include "misc.h"
 #include "event.h"
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 #include "xdnd.h"
 #endif
 
@@ -60,11 +60,6 @@
  * using the classname/instancename
  */
 
-/**** Global variables ****/
-extern Cursor wCursor[WCUR_LAST];
-extern WDDomain *WDWindowAttributes;
-extern XContext wWinContext;
-
 #define MOD_MASK       wPreferences.modifier_mask
 #define ICON_SIZE      wPreferences.icon_size
 
@@ -79,7 +74,7 @@ static void create_appicon_from_dock(WWindow *wwin, WApplication *wapp, Window m
 /* This function is used if the application is a .app. It checks if it has an icon in it
  * like for example /usr/local/GNUstep/Applications/WPrefs.app/WPrefs.tiff
  */
-void wApplicationExtractDirPackIcon(WScreen * scr, const char *path, const char *wm_instance, const char *wm_class)
+void wApplicationExtractDirPackIcon(const char *path, const char *wm_instance, const char *wm_class)
 {
 	char *iconPath = NULL;
 	char *tmp = NULL;
@@ -87,7 +82,7 @@ void wApplicationExtractDirPackIcon(WScreen * scr, const char *path, const char
 	if (strstr(path, ".app")) {
 		tmp = wmalloc(strlen(path) + 16);
 
-		if (scr->flags.supports_tiff) {
+		if (wPreferences.supports_tiff) {
 			strcpy(tmp, path);
 			strcat(tmp, ".tiff");
 			if (access(tmp, R_OK) == 0)
@@ -131,11 +126,11 @@ WAppIcon *wAppIconCreateForDock(WScreen *scr, const char *command, const char *w
 	if (wm_instance)
 		aicon->wm_instance = wstrdup(wm_instance);
 
-	if (strcmp(wm_class, "WMDock") == 0 && wPreferences.flags.clip_merged_in_dock)
+	if (wPreferences.flags.clip_merged_in_dock && wm_class != NULL && strcmp(wm_class, "WMDock") == 0)
 		tile = TILE_CLIP;
 	aicon->icon = icon_create_for_dock(scr, command, wm_instance, wm_class, tile);
 
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 	wXDNDMakeAwareness(aicon->icon->core->window);
 #endif
 
@@ -202,15 +197,16 @@ void unpaint_app_icon(WApplication *wapp)
 void paint_app_icon(WApplication *wapp)
 {
 	WIcon *icon;
-	WScreen *scr = wapp->main_window_desc->screen_ptr;
+	WScreen *scr;
 	WDock *attracting_dock;
 	int x = 0, y = 0;
 	Bool update_icon = False;
 
-	if (!wapp || !wapp->app_icon)
+	if (!wapp || !wapp->app_icon || !wapp->main_window_desc)
 		return;
 
 	icon = wapp->app_icon->icon;
+	scr = wapp->main_window_desc->screen_ptr;
 	wapp->app_icon->main_window = wapp->main_window;
 
 	/* If the icon is docked, don't continue */
@@ -313,7 +309,7 @@ static WAppIcon *wAppIconCreate(WWindow *leader_win)
 		aicon->wm_instance = wstrdup(leader_win->wm_instance);
 
 	aicon->icon = icon_create_for_wwindow(leader_win);
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 	wXDNDMakeAwareness(aicon->icon->core->window);
 #endif
 
@@ -336,7 +332,7 @@ void wAppIconDestroy(WAppIcon * aicon)
 	wIconDestroy(aicon->icon);
 	if (aicon->command)
 		wfree(aicon->command);
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 	if (aicon->dnd_command)
 		wfree(aicon->dnd_command);
 #endif
@@ -377,7 +373,7 @@ void wAppIconMove(WAppIcon * aicon, int x, int y)
 }
 
 #ifdef WS_INDICATOR
-static void updateDockNumbers(WScreen * scr)
+static void updateDockNumbers(WScreen *scr)
 {
 	int length;
 	char *ws_numbers;
@@ -415,7 +411,7 @@ void wAppIconPaint(WAppIcon *aicon)
 	if (aicon->docked && scr->dock && scr->dock == aicon->dock && aicon->yindex == 0)
 		updateDockNumbers(scr);
 # endif
-	if (scr->dock_dots && aicon->docked && !aicon->running && aicon->command != NULL) {
+	if (aicon->docked && !aicon->running && aicon->command != NULL) {
 		XSetClipMask(dpy, scr->copy_gc, scr->dock_dots->mask);
 		XSetClipOrigin(dpy, scr->copy_gc, 0, 0);
 		XCopyArea(dpy, scr->dock_dots->image, aicon->icon->core->window,
@@ -490,6 +486,9 @@ static void relaunchCallback(WMenu * menu, WMenuEntry * entry)
 {
 	WApplication *wapp = (WApplication *) entry->clientdata;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) menu;
+
 	relaunchApplication(wapp);
 }
 
@@ -509,6 +508,9 @@ static void unhideHereCallback(WMenu * menu, WMenuEntry * entry)
 {
 	WApplication *wapp = (WApplication *) entry->clientdata;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) menu;
+
 	wUnhideApplication(wapp, False, True);
 }
 
@@ -519,6 +521,9 @@ static void setIconCallback(WMenu *menu, WMenuEntry *entry)
 	WScreen *scr;
 	int result;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) menu;
+
 	assert(icon != NULL);
 
 	if (icon->editing)
@@ -531,17 +536,16 @@ static void setIconCallback(WMenu *menu, WMenuEntry *entry)
 
 	result = wIconChooserDialog(scr, &file, icon->wm_instance, icon->wm_class);
 
-	if (result && !icon->destroyed) {
-		if (file && *file == 0) {
-			wfree(file);
-			file = NULL;
-		}
-		if (!wIconChangeImageFile(icon->icon, file)) {
-			wMessageDialog(scr, _("Error"),
-				       _("Could not open specified icon file"), _("OK"), NULL, NULL);
-		} else {
-			wDefaultChangeIcon(scr, icon->wm_instance, icon->wm_class, file);
-			wAppIconPaint(icon);
+	if (result) {
+		if (!icon->destroyed) {
+			if (!wIconChangeImageFile(icon->icon, file)) {
+				wMessageDialog(scr, _("Error"),
+				               _("Could not open specified icon file"),
+				               _("OK"), NULL, NULL);
+			} else {
+				wDefaultChangeIcon(icon->wm_instance, icon->wm_class, file);
+				wAppIconPaint(icon);
+			}
 		}
 		if (file)
 			wfree(file);
@@ -648,6 +652,9 @@ static void openApplicationMenu(WApplication * wapp, int x, int y)
 
 static void iconExpose(WObjDescriptor *desc, XEvent *event)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) event;
+
 	wAppIconPaint(desc->parent);
 }
 
@@ -742,7 +749,7 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 	WDock *lastDock = originalDock;
 	WDock *allDocks[scr->drawer_count + 2]; /* clip, dock and drawers (order determined at runtime) */
 	WDrawerChain *dc;
-	Bool done = False, dockable, ondock;
+	Bool dockable, ondock;
 	Bool grabbed = False;
 	Bool collapsed = False; /* Stores the collapsed state of lastDock, before the moving appicon entered it */
 	int superfluous = wPreferences.superfluous; /* we cache it to avoid problems */
@@ -809,9 +816,11 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 	}
 	if (!wPreferences.flags.nodock && scr->dock != originalDock)
 		allDocks[ i++ ] = scr->dock;
+
 	if (!wPreferences.flags.noclip &&
 	    originalDock != scr->workspaces[scr->current_workspace]->clip)
 		allDocks[ i++ ] = scr->workspaces[scr->current_workspace]->clip;
+
 	for ( ; i < scr->drawer_count + 2; i++) /* In case the clip, the dock, or both, are disabled */
 		allDocks[ i ] = NULL;
 
@@ -830,7 +839,7 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 	if (ondock)
 		XMapWindow(dpy, scr->dock_shadow);
 
-	while (!done) {
+	while (1) {
 		WMMaskEvent(dpy, PointerMotionMask | ButtonReleaseMask | ButtonPressMask
 			    | ButtonMotionMask | ExposureMask | EnterWindowMask, &ev);
 		switch (ev.type) {
@@ -852,7 +861,7 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 				    || abs(ofs_y - ev.xmotion.y) >= MOVE_THRESHOLD) {
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_MOVE], CurrentTime);
+								 wPreferences.cursor[WCUR_MOVE], CurrentTime);
 					grabbed = 1;
 				} else {
 					break;
@@ -864,6 +873,7 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 				for (i = 0; i < scr->workspace_count; i++) {
 					if (i == scr->current_workspace)
 						continue;
+
 					wDockShowIcons(scr->workspaces[i]->clip);
 					/* Note: if dock is collapsed (for instance, because it
 					   auto-collapses), its icons still won't show up */
@@ -876,7 +886,7 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 			wAppIconMove(aicon, x, y);
 
 			WDock *theNewDock = NULL;
-			if (!(ev.xmotion.state & MOD_MASK) || aicon->launching || aicon->lock) {
+			if (!(ev.xmotion.state & MOD_MASK) || aicon->launching || aicon->lock || originalDock == NULL) {
 				for (i = 0; dockable && i < scr->drawer_count + 2; i++) {
 					WDock *theDock = allDocks[i];
 					if (theDock == NULL)
@@ -957,7 +967,7 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 
 			Bool docked = False;
 			if (ondock) {
-				SlideWindow(icon->core->window, x, y, shad_x, shad_y);
+				slide_window(icon->core->window, x, y, shad_x, shad_y);
 				XUnmapWindow(dpy, scr->dock_shadow);
 				if (originalDock == NULL) { // docking an undocked appicon
 					docked = wDockAttachIcon(lastDock, aicon, ix, iy, False);
@@ -970,7 +980,7 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 							// Also fill the gap left in the drawer
 							wDrawerFillTheGap(lastDock, aicon, False);
 						}
-						SlideWindow(icon->core->window, x, y, oldX, oldY);
+						slide_window(icon->core->window, x, y, oldX, oldY);
 					}
 				}
 				else { // moving a docked appicon to a dock
@@ -1006,7 +1016,7 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 								// Trust the appicon is inserted at exactly the same place, so its oldX/oldY are consistent with its "new" location?
 							}
 
-							SlideWindow(icon->core->window, x, y, oldX, oldY);
+							slide_window(icon->core->window, x, y, oldX, oldY);
 							wDockReattachIcon(originalDock, aicon, aicon->xindex, aicon->yindex);
 						}
 						else {
@@ -1069,6 +1079,7 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 				for (i = 0; i < scr->workspace_count; i++) {
 					if (i == scr->current_workspace)
 						continue;
+
 					wDockHideIcons(scr->workspaces[i]->clip);
 				}
 			}
@@ -1078,13 +1089,12 @@ Bool wHandleAppIconMove(WAppIcon *aicon, XEvent *event)
 			return hasMoved;
 		}
 	}
-	return False; /* Never reached */
 }
 
 /* This function save the application icon and store the path in the Dictionary */
 static void wApplicationSaveIconPathFor(const char *iconPath, const char *wm_instance, const char *wm_class)
 {
-	WMPropList *dict = WDWindowAttributes->dictionary;
+	WMPropList *dict = w_global.domain.window_attr->dictionary;
 	WMPropList *adict, *key, *iconk;
 	WMPropList *val;
 	char *tmp;
@@ -1118,7 +1128,7 @@ static void wApplicationSaveIconPathFor(const char *iconPath, const char *wm_ins
 	WMReleasePropList(iconk);
 
 	if (val && !wPreferences.flags.noupdates)
-		UpdateDomainFile(WDWindowAttributes);
+		UpdateDomainFile(w_global.domain.window_attr);
 }
 
 static WAppIcon *findDockIconFor(WDock *dock, Window main_window)
@@ -1150,8 +1160,10 @@ static void create_appicon_from_dock(WWindow *wwin, WApplication *wapp, Window m
 		int i;
 		for (i = 0; i < scr->workspace_count; i++) {
 			WDock *dock = scr->workspaces[i]->clip;
+
 			if (dock)
 				wapp->app_icon = findDockIconFor(dock, main_window);
+
 			if (wapp->app_icon)
 				break;
 		}
@@ -1222,7 +1234,7 @@ WAppIcon *wAppIconFor(Window window)
 	if (window == None)
 		return NULL;
 
-	if (XFindContext(dpy, window, wWinContext, (XPointer *) & desc) == XCNOENT)
+	if (XFindContext(dpy, window, w_global.context.client_win, (XPointer *) & desc) == XCNOENT)
 		return NULL;
 
 	if (desc->parent_type == WCLASS_APPICON || desc->parent_type == WCLASS_DOCK_ICON)
