@@ -35,11 +35,8 @@
 #include "properties.h"
 #include "workspace.h"
 #include "dock.h"
+#include "defaults.h"
 
-/******** Global variables ********/
-
-extern XContext wAppWinContext;
-extern XContext wWinContext;
 
 /******** Local variables ********/
 
@@ -72,7 +69,7 @@ WApplication *wApplicationOf(Window window)
 
 	if (window == None)
 		return NULL;
-	if (XFindContext(dpy, window, wAppWinContext, (XPointer *) & wapp) != XCSUCCESS)
+	if (XFindContext(dpy, window, w_global.context.app_win, (XPointer *) & wapp) != XCSUCCESS)
 		return NULL;
 	return wapp;
 }
@@ -101,7 +98,7 @@ WApplication *wApplicationCreate(WWindow * wwin)
 		wapp->refcount++;
 		if (wapp->app_icon && wapp->app_icon->docked &&
 		    wapp->app_icon->relaunching && wapp->main_window_desc->fake_group)
-			wDockFinishLaunch(wapp->app_icon->dock, wapp->app_icon);
+			wDockFinishLaunch(wapp->app_icon);
 
 		return wapp;
 	}
@@ -139,24 +136,14 @@ WApplication *wApplicationCreate(WWindow * wwin)
 	wapp->flags.emulated = WFLAGP(wapp->main_window_desc, emulate_appicon);
 
 	/* application descriptor */
-	XSaveContext(dpy, main_window, wAppWinContext, (XPointer) wapp);
+	XSaveContext(dpy, main_window, w_global.context.app_win, (XPointer) wapp);
 
-	/* First try to create an icon from the dock or clip */
-	create_appicon_from_dock(wwin, wapp, main_window);
-
-	/*
-	 * In case it was not found in the dock, make it from scratch.
-	 * Note: makeAppIconFor() returns early if wapp->app_icon exists
-	 */
-	makeAppIconFor(wapp);
-
-	/* Save the app_icon in a file */
-	save_appicon(wapp->app_icon, False);
+	create_appicon_for_application(wapp, wwin);
 
 	return wapp;
 }
 
-void wApplicationDestroy(WApplication * wapp)
+void wApplicationDestroy(WApplication *wapp)
 {
 	WWindow *wwin;
 	WScreen *scr;
@@ -192,7 +179,7 @@ void wApplicationDestroy(WApplication * wapp)
 			wapp->prev->next = wapp->next;
 	}
 
-	XDeleteContext(dpy, wapp->main_window, wAppWinContext);
+	XDeleteContext(dpy, wapp->main_window, w_global.context.app_win);
 	wAppMenuDestroy(wapp->menu);
 
 	/* Remove application icon */
@@ -204,7 +191,7 @@ void wApplicationDestroy(WApplication * wapp)
 	if (wwin) {
 		/* undelete client window context that was deleted in
 		 * wWindowDestroy */
-		XSaveContext(dpy, wwin->client_win, wWinContext, (XPointer) & wwin->client_descriptor);
+		XSaveContext(dpy, wwin->client_win, w_global.context.client_win, (XPointer) & wwin->client_descriptor);
 	}
 	wfree(wapp);
 }
