@@ -2,6 +2,7 @@
  *  Window Maker window manager
  *
  *  Copyright (c) 1998-2003 Alfredo K. Kojima
+ *  Copyright (c) 2014 Window Maker Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,6 +43,7 @@
 #include "appicon.h"
 #include "workspace.h"
 #include "balloon.h"
+#include "misc.h"
 
 
 typedef struct _WBalloon {
@@ -59,6 +61,7 @@ typedef struct _WBalloon {
 	WMHandlerID timer;
 
 	Pixmap contents;
+	Pixmap mini_preview;
 
 	char mapped;
 	char ignoreTimer;
@@ -69,11 +72,6 @@ typedef struct _WBalloon {
 #define LEFT	0
 #define RIGHT	2
 
-#define TLEFT	(TOP|LEFT)
-#define TRIGHT 	(TOP|RIGHT)
-#define BLEFT	(BOTTOM|LEFT)
-#define BRIGHT	(BOTTOM|RIGHT)
-
 static int countLines(const char *text)
 {
 	const char *p = text;
@@ -87,7 +85,7 @@ static int countLines(const char *text)
 	return h;
 }
 
-static int getMaxStringWidth(WMFont * font, const char *text)
+static int getMaxStringWidth(WMFont *font, const char *text)
 {
 	const char *p = text;
 	const char *pb = p;
@@ -114,8 +112,7 @@ static int getMaxStringWidth(WMFont * font, const char *text)
 	return w;
 }
 
-static void
-drawMultiLineString(WMScreen * scr, Pixmap pixmap, WMColor * color,
+static void drawMultiLineString(WMScreen *scr, Pixmap pixmap, WMColor *color,
 		    WMFont *font, int x, int y, const char *text, int len)
 {
 	const char *p = text;
@@ -134,16 +131,15 @@ drawMultiLineString(WMScreen * scr, Pixmap pixmap, WMColor * color,
 		}
 		p++;
 	}
-	if (pos > 0) {
+	if (pos > 0)
 		WMDrawString(scr, pixmap, color, font, x, y + l * height, pb, pos);
-	}
 }
 
 #ifdef SHAPED_BALLOON
 
-#define 	SPACE	12
+#define SPACE 12
 
-static void drawBalloon(WScreen * scr, Pixmap bitmap, Pixmap pix, int x, int y, int w, int h, int side)
+static void drawBalloon(WScreen *scr, Pixmap bitmap, Pixmap pix, int x, int y, int w, int h, int side)
 {
 	GC bgc = scr->balloon->monoGC;
 	GC gc = scr->draw_gc;
@@ -233,7 +229,7 @@ static void drawBalloon(WScreen * scr, Pixmap bitmap, Pixmap pix, int x, int y,
 	XDrawLines(dpy, pix, gc, pt, 3, CoordModeOrigin);
 }
 
-static Pixmap makePixmap(WScreen * scr, int width, int height, int side, Pixmap * mask)
+static Pixmap makePixmap(WScreen *scr, int width, int height, int side, Pixmap *mask)
 {
 	WBalloon *bal = scr->balloon;
 	Pixmap bitmap;
@@ -242,9 +238,9 @@ static Pixmap makePixmap(WScreen * scr, int width, int height, int side, Pixmap
 
 	bitmap = XCreatePixmap(dpy, scr->root_win, width + SPACE, height + SPACE, 1);
 
-	if (!bal->monoGC) {
+	if (!bal->monoGC)
 		bal->monoGC = XCreateGC(dpy, bitmap, 0, NULL);
-	}
+
 	XSetForeground(dpy, bal->monoGC, 0);
 	XFillRectangle(dpy, bitmap, bal->monoGC, 0, 0, width + SPACE, height + SPACE);
 
@@ -252,11 +248,10 @@ static Pixmap makePixmap(WScreen * scr, int width, int height, int side, Pixmap
 	XSetForeground(dpy, scr->draw_gc, scr->black_pixel);
 	XFillRectangle(dpy, pixmap, scr->draw_gc, 0, 0, width + SPACE, height + SPACE);
 
-	if (side & BOTTOM) {
+	if (side & BOTTOM)
 		y = 0;
-	} else {
+	else
 		y = SPACE;
-	}
 	x = 0;
 
 	drawBalloon(scr, bitmap, pixmap, x, y, width, height, side);
@@ -376,25 +371,100 @@ static void showText(WScreen *scr, int x, int y, int h, int w, const char *text)
 }
 #endif				/* !SHAPED_BALLOON */
 
-static void showBalloon(WScreen * scr)
+static void show_minipreview(WScreen *scr, int x, int y, const char *title, Pixmap mini_preview)
+{
+	Pixmap pixmap;
+	WMFont *font = scr->info_text_font;
+	int width, height;
+	int titleHeight;
+	char *shortenTitle;
+
+	if (scr->balloon->contents)
+		XFreePixmap(dpy, scr->balloon->contents);
+
+	width  = wPreferences.minipreview_size;
+	height = wPreferences.minipreview_size;
+
+	if (wPreferences.miniwin_title_balloon) {
+		shortenTitle = ShrinkString(font, title, width - MINIPREVIEW_BORDER * 2);
+		titleHeight = countLines(shortenTitle) * WMFontHeight(font) + 4;
+		height += titleHeight;
+	} else {
+		shortenTitle = NULL;
+		titleHeight = 0;
+	}
+
+	if (x < 0)
+		x = 0;
+	else if (x + width > scr->scr_width - 1)
+		x = scr->scr_width - width - 1;
+
+	if (y - height - 2 < 0) {
+		y += wPreferences.icon_size;
+		if (y < 0)
+			y = 0;
+	} else {
+		y -= height + 2;
+	}
+
+	if (scr->window_title_texture[0])
+		XSetForeground(dpy, scr->draw_gc, scr->window_title_texture[0]->any.color.pixel);
+	else
+		XSetForeground(dpy, scr->draw_gc, scr->light_pixel);
+
+	pixmap = XCreatePixmap(dpy, scr->root_win, width, height, scr->w_depth);
+	XFillRectangle(dpy, pixmap, scr->draw_gc, 0, 0, width, height);
+
+	if (shortenTitle != NULL) {
+		drawMultiLineString(scr->wmscreen, pixmap, scr->window_title_color[0], font,
+						MINIPREVIEW_BORDER, MINIPREVIEW_BORDER, shortenTitle, strlen(shortenTitle));
+		wfree(shortenTitle);
+	}
+
+	XCopyArea(dpy, mini_preview, pixmap, scr->draw_gc,
+	          0, 0, (wPreferences.minipreview_size - 1 - MINIPREVIEW_BORDER * 2),
+	          (wPreferences.minipreview_size - 1 - MINIPREVIEW_BORDER * 2),
+	          MINIPREVIEW_BORDER, MINIPREVIEW_BORDER + titleHeight);
+
+#ifdef SHAPED_BALLOON
+	XShapeCombineMask(dpy, scr->balloon->window, ShapeBounding, 0, 0, None, ShapeSet);
+#endif
+	XResizeWindow(dpy, scr->balloon->window, width, height);
+	XMoveWindow(dpy, scr->balloon->window, x, y);
+
+	XSetWindowBackgroundPixmap(dpy, scr->balloon->window, pixmap);
+
+	XClearWindow(dpy, scr->balloon->window);
+	XMapRaised(dpy, scr->balloon->window);
+
+
+	scr->balloon->contents = pixmap;
+
+	scr->balloon->mapped = 1;
+}
+
+static void showBalloon(WScreen *scr)
 {
 	int x, y;
 	Window foow;
 	unsigned foo, w;
 
-	if (scr->balloon) {
-		scr->balloon->timer = NULL;
-		scr->balloon->ignoreTimer = 1;
-	}
+	scr->balloon->timer = NULL;
+	scr->balloon->ignoreTimer = 1;
 
 	if (!XGetGeometry(dpy, scr->balloon->objectWindow, &foow, &x, &y, &w, &foo, &foo, &foo)) {
 		scr->balloon->prevType = 0;
 		return;
 	}
-	showText(scr, x, y, scr->balloon->h, w, scr->balloon->text);
+
+	if (wPreferences.miniwin_preview_balloon && scr->balloon->mini_preview != None)
+		/* used to display either the mini-preview alone or the mini-preview with the title */
+		show_minipreview(scr, x, y, scr->balloon->text, scr->balloon->mini_preview);
+	else
+		showText(scr, x, y, scr->balloon->h, w, scr->balloon->text);
 }
 
-static void frameBalloon(WObjDescriptor * object)
+static void frameBalloon(WObjDescriptor *object)
 {
 	WFrameWindow *fwin = (WFrameWindow *) object->parent;
 	WScreen *scr = fwin->core->screen_ptr;
@@ -411,7 +481,7 @@ static void frameBalloon(WObjDescriptor * object)
 	}
 }
 
-static void miniwindowBalloon(WObjDescriptor * object)
+static void miniwindowBalloon(WObjDescriptor *object)
 {
 	WIcon *icon = (WIcon *) object->parent;
 	WScreen *scr = icon->core->screen_ptr;
@@ -422,7 +492,9 @@ static void miniwindowBalloon(WObjDescriptor * object)
 	}
 	scr->balloon->h = icon->core->height;
 	scr->balloon->text = wstrdup(icon->icon_name);
+	scr->balloon->mini_preview = icon->mini_preview;
 	scr->balloon->objectWindow = icon->core->window;
+
 	if ((scr->balloon->prevType == object->parent_type || scr->balloon->prevType == WCLASS_APPICON)
 	    && scr->balloon->ignoreTimer) {
 		XUnmapWindow(dpy, scr->balloon->window);
@@ -432,7 +504,7 @@ static void miniwindowBalloon(WObjDescriptor * object)
 	}
 }
 
-static void appiconBalloon(WObjDescriptor * object)
+static void appiconBalloon(WObjDescriptor *object)
 {
 	WAppIcon *aicon = (WAppIcon *) object->parent;
 	WScreen *scr = aicon->icon->core->screen_ptr;
@@ -447,14 +519,49 @@ static void appiconBalloon(WObjDescriptor * object)
 			return;
 		}
 	} else if (aicon->command && aicon->wm_class) {
-		int len = strlen(aicon->command) + strlen(aicon->wm_class) + 8;
+		int len;
+		WApplication *app;
+		unsigned int app_win_cnt = 0;
+		const char *display_name;
+
+		if (object->parent_type == WCLASS_DOCK_ICON) {
+			if (aicon->main_window) {
+				app = wApplicationOf(aicon->main_window);
+				if (app && app->main_window_desc && app->main_window_desc->fake_group)
+					app_win_cnt = app->main_window_desc->fake_group->retainCount - 1;
+			}
+		}
+
+		/*
+		 * Check to see if it is a GNUstep app, because in this case we use the instance
+		 * instead of the class, otherwise the user will not be able to distinguish what
+		 * is being refered.
+		 */
+		if (strcmp(aicon->wm_class, "GNUstep") == 0)
+			display_name = aicon->wm_instance;
+		else
+			display_name = aicon->wm_class;
+
+		len = strlen(aicon->command) + strlen(display_name) + 8;
+
+		if (app_win_cnt > 0)
+			len += 1 + snprintf(NULL, 0, "%u", app_win_cnt);
+
 		tmp = wmalloc(len);
-		snprintf(tmp, len, "%s\n(%s)", aicon->wm_class, aicon->command);
+		if (app_win_cnt > 0)
+			snprintf(tmp, len, "%u %s\n(%s)", app_win_cnt, display_name, aicon->command);
+		else
+			snprintf(tmp, len, "%s\n(%s)", aicon->wm_instance, aicon->command);
 		scr->balloon->text = tmp;
+
 	} else if (aicon->command) {
 		scr->balloon->text = wstrdup(aicon->command);
 	} else if (aicon->wm_class) {
-		scr->balloon->text = wstrdup(aicon->wm_class);
+		/* Check to see if it is a GNUstep App */
+		if (strcmp(aicon->wm_class, "GNUstep") == 0)
+			scr->balloon->text = wstrdup(aicon->wm_instance);
+		else
+			scr->balloon->text = wstrdup(aicon->wm_class);
 	} else {
 		wBalloonHide(scr);
 		return;
@@ -471,7 +578,7 @@ static void appiconBalloon(WObjDescriptor * object)
 	}
 }
 
-void wBalloonInitialize(WScreen * scr)
+void wBalloonInitialize(WScreen *scr)
 {
 	WBalloon *bal;
 	XSetWindowAttributes attribs;
@@ -497,7 +604,7 @@ void wBalloonInitialize(WScreen * scr)
 #endif
 }
 
-void wBalloonEnteredObject(WScreen * scr, WObjDescriptor * object)
+void wBalloonEnteredObject(WScreen *scr, WObjDescriptor *object)
 {
 	WBalloon *balloon = scr->balloon;
 
@@ -511,6 +618,8 @@ void wBalloonEnteredObject(WScreen * scr, WObjDescriptor * object)
 		wfree(scr->balloon->text);
 	scr->balloon->text = NULL;
 
+	scr->balloon->mini_preview = None;
+
 	if (!object) {
 		wBalloonHide(scr);
 		balloon->ignoreTimer = 0;
@@ -527,7 +636,7 @@ void wBalloonEnteredObject(WScreen * scr, WObjDescriptor * object)
 			appiconBalloon(object);
 		break;
 	case WCLASS_MINIWINDOW:
-		if (wPreferences.miniwin_balloon)
+		if (wPreferences.miniwin_title_balloon || wPreferences.miniwin_preview_balloon)
 			miniwindowBalloon(object);
 		break;
 	case WCLASS_APPICON:
@@ -541,7 +650,7 @@ void wBalloonEnteredObject(WScreen * scr, WObjDescriptor * object)
 	scr->balloon->prevType = object->parent_type;
 }
 
-void wBalloonHide(WScreen * scr)
+void wBalloonHide(WScreen *scr)
 {
 	if (scr) {
 		if (scr->balloon->mapped) {
