@@ -4,6 +4,7 @@
  *
  *  Copyright (c) 1997-2003 Alfredo K. Kojima
  *  Copyright (c) 1998-2003 Dan Pascu
+ *  Copyright (c) 2014 Window Maker Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,6 +35,7 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include <limits.h>
+#include <errno.h>
 
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
@@ -105,6 +107,9 @@ int wMessageDialog(WScreen *scr, const char *title, const char *message, const c
 
 static void toggleSaveSession(WMWidget *w, void *data)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) data;
+
 	wPreferences.save_session_on_exit = WMGetButtonSelected((WMButton *) w);
 }
 
@@ -257,7 +262,8 @@ ScanFiles(const char *dir, const char *prefix, unsigned acceptmask, unsigned dec
 	char *fullfilename, *suffix;
 
 	prefixlen = strlen(prefix);
-	if ((d = opendir(dir)) != NULL) {
+	d = opendir(dir);
+	if (d != NULL) {
 		while ((de = readdir(d)) != NULL) {
 			if (strlen(de->d_name) > prefixlen &&
 			    !strncmp(prefix, de->d_name, prefixlen) &&
@@ -272,7 +278,7 @@ ScanFiles(const char *dir, const char *prefix, unsigned acceptmask, unsigned dec
 						  de->d_name + prefixlen) == WANotFound) {
 					suffix = wstrdup(de->d_name + prefixlen);
 					if (sb.st_mode & S_IFDIR)
-						wstrappend(suffix,"/");
+						suffix = wstrappend(suffix, "/");
 					WMAddToArray(result, suffix);
 				}
 				wfree(fullfilename);
@@ -293,12 +299,14 @@ static WMArray *GenerateVariants(const char *complete)
 	while (*complete == ' ')
 		++complete;
 
-	if ((pos = strrchr(complete, ' ')) != NULL) {
+	pos = strrchr(complete, ' ');
+	if (pos != NULL) {
 		complete = pos + 1;
 		firstWord = False;
 	}
 
-	if ((pos = strrchr(complete, '/')) != NULL) {
+	pos = strrchr(complete, '/');
+	if (pos != NULL) {
 		tmp = wstrndup((char *)complete, pos - complete + 1);
 		if (*tmp == '~' && *(tmp + 1) == '/' && getenv("HOME")) {
 			dir = wstrdup(getenv("HOME"));
@@ -568,16 +576,11 @@ static void listPixmaps(WScreen *scr, WMList *lPtr, const char *path)
 	dir = opendir(apath);
 
 	if (!dir) {
-		char *msg;
-		char *tmp;
-		tmp = _("Could not open directory ");
-		msg = wmalloc(strlen(tmp) + strlen(path) + 6);
-		strcpy(msg, tmp);
-		strcat(msg, path);
-
-		wMessageDialog(scr, _("Error"), msg, _("OK"), NULL, NULL);
-		wfree(msg);
 		wfree(apath);
+		snprintf(pbuf, sizeof(pbuf),
+		         _("Could not open directory \"%s\":\n%s"),
+		         path, strerror(errno));
+		wMessageDialog(scr, _("Error"), pbuf, _("OK"), NULL, NULL);
 		return;
 	}
 
@@ -588,9 +591,13 @@ static void listPixmaps(WScreen *scr, WMList *lPtr, const char *path)
 		if (strcmp(dentry->d_name, ".") == 0 || strcmp(dentry->d_name, "..") == 0)
 			continue;
 
-		strcpy(pbuf, apath);
-		strcat(pbuf, "/");
-		strcat(pbuf, dentry->d_name);
+		if (wstrlcpy(pbuf, apath, sizeof(pbuf)) >= sizeof(pbuf) ||
+		    wstrlcat(pbuf, "/", sizeof(pbuf)) >= sizeof(pbuf) ||
+		    wstrlcat(pbuf, dentry->d_name, sizeof(pbuf)) >= sizeof(pbuf)) {
+			wwarning(_("full path for file \"%s\" in \"%s\" is longer than %d bytes, skipped"),
+			         dentry->d_name, path, (int) (sizeof(pbuf) - 1) );
+			continue;
+		}
 
 		if (stat(pbuf, &statb) < 0)
 			continue;
@@ -616,7 +623,8 @@ static void setViewedImage(IconPanel *panel, const char *file)
 	color.green = 0xaa;
 	color.blue = 0xae;
 	color.alpha = 0;
-	pixmap = WMCreateBlendedPixmapFromFile(WMWidgetScreen(panel->win), file, &color);
+	pixmap = WMCreateScaledBlendedPixmapFromFile(WMWidgetScreen(panel->win), file, &color, 75, 75);
+
 	if (!pixmap) {
 		WMSetButtonEnabled(panel->okButton, False);
 
@@ -660,13 +668,13 @@ static void listCallback(void *self, void *data)
 		if (item == NULL)
 			return;
 		path = item->text;
-		tmp = wexpandpath(path);
 
 		item = WMGetListSelectedItem(panel->iconList);
 		if (item == NULL)
 			return;
 		iconFile = item->text;
 
+		tmp = wexpandpath(path);
 		path = wmalloc(strlen(tmp) + strlen(iconFile) + 4);
 		strcpy(path, tmp);
 		strcat(path, "/");
@@ -715,6 +723,9 @@ static void drawIconProc(WMList * lPtr, int index, Drawable d, char *text, int s
 	RColor color;
 	int x, y, width, height, len;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) index;
+
 	if (!panel->preview)
 		return;
 
@@ -736,7 +747,7 @@ static void drawIconProc(WMList * lPtr, int index, Drawable d, char *text, int s
 	color.blue = WMBlueComponentOfColor(back) >> 8;
 	color.alpha = WMGetColorAlpha(back) >> 8;
 
-	pixmap = WMCreateBlendedPixmapFromFile(wmscr, file, &color);
+	pixmap = WMCreateScaledBlendedPixmapFromFile(wmscr, file, &color, width - 2, height - 2);
 	wfree(file);
 
 	if (!pixmap) {
@@ -1004,22 +1015,40 @@ Bool wIconChooserDialog(WScreen *scr, char **file, const char *instance, const c
 	XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
 	{
-		char *tmp;
-		int len = (instance ? strlen(instance) : 0)
-		    + (class ? strlen(class) : 0) + 32;
+		static const char *prefix = NULL;
+		char *title;
+		int len;
 		WMPoint center;
 
-		tmp = wmalloc(len);
+		if (prefix == NULL)
+			prefix = _("Icon Chooser");
 
-		if (tmp && (instance || class))
-			snprintf(tmp, len, "%s [%s.%s]", _("Icon Chooser"), instance, class);
-		else
-			strcpy(tmp, _("Icon Chooser"));
+		len = strlen(prefix)
+			+ 2	// " ["
+			+ (instance ? strlen(instance) : 0)
+			+ 1	// "."
+			+ (class ? strlen(class) : 0)
+			+ 1	// "]"
+			+ 1;	// final NUL
+
+		title = wmalloc(len);
+		strcpy(title, prefix);
+
+		if (instance || class) {
+			strcat(title, " [");
+			if (instance != NULL)
+				strcat(title, instance);
+			if (instance && class)
+				strcat(title, ".");
+			if (class != NULL)
+				strcat(title, class);
+			strcat(title, "]");
+		}
 
 		center = getCenter(scr, 450, 280);
 
-		wwin = wManageInternalWindow(scr, parent, None, tmp, center.x, center.y, 450, 280);
-		wfree(tmp);
+		wwin = wManageInternalWindow(scr, parent, None, title, center.x, center.y, 450, 280);
+		wfree(title);
 	}
 
 	/* put icon paths in the list */
@@ -1099,30 +1128,38 @@ typedef struct {
 
 #define COPYRIGHT_TEXT  \
     "Copyright \xc2\xa9 1997-2006 Alfredo K. Kojima\n"\
-    "Copyright \xc2\xa9 1998-2006 Dan Pascu"
+    "Copyright \xc2\xa9 1998-2006 Dan Pascu\n"\
+    "Copyright \xc2\xa9 2013-2014 Window Maker Developers Team"
 
-static InfoPanel *thePanel = NULL;
+static InfoPanel *infoPanel = NULL;
 
 static void destroyInfoPanel(WCoreWindow *foo, void *data, XEvent *event)
 {
-	WMUnmapWidget(thePanel);
-	wUnmanageWindow(thePanel->wwin, False, False);
-	WMDestroyWidget(thePanel->win);
-	wfree(thePanel);
-	thePanel = NULL;
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) foo;
+	(void) data;
+	(void) event;
+
+	WMUnmapWidget(infoPanel);
+	wUnmanageWindow(infoPanel->wwin, False, False);
+	WMDestroyWidget(infoPanel->win);
+	wfree(infoPanel);
+	infoPanel = NULL;
 }
 
-void wShowInfoPanel(WScreen * scr)
+void wShowInfoPanel(WScreen *scr)
 {
+	const int win_width = 382;
+	const int win_height = 250;
 	InfoPanel *panel;
 	WMPixmap *logo;
 	WMFont *font;
-	char *strbuf = NULL;
+	char *name, *strbuf = NULL;
 	const char *separator;
 	char buffer[256];
-	const char *name;
 	Window parent;
 	WWindow *wwin;
+	WMPoint center;
 	char **strl;
 	int i, width = 50, sepHeight;
 	char *visuals[] = {
@@ -1134,10 +1171,10 @@ void wShowInfoPanel(WScreen * scr)
 		"DirectColor"
 	};
 
-	if (thePanel) {
-		if (thePanel->scr == scr) {
-			wRaiseFrame(thePanel->wwin->frame->core);
-			wSetFocusTo(scr, thePanel->wwin);
+	if (infoPanel) {
+		if (infoPanel->scr == scr) {
+			wRaiseFrame(infoPanel->wwin->frame->core);
+			wSetFocusTo(scr, infoPanel->wwin);
 		}
 		return;
 	}
@@ -1147,7 +1184,7 @@ void wShowInfoPanel(WScreen * scr)
 	panel->scr = scr;
 
 	panel->win = WMCreateWindow(scr->wmscreen, "info");
-	WMResizeWidget(panel->win, 390, 230);
+	WMResizeWidget(panel->win, win_width, win_height);
 
 	logo = WMCreateApplicationIconBlendedPixmap(scr->wmscreen, (RColor *) NULL);
 	if (!logo) {
@@ -1206,8 +1243,8 @@ void wShowInfoPanel(WScreen * scr)
 	WMSetLabelWraps(panel->versionL, False);
 
 	panel->copyrL = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->copyrL, 360, 40);
-	WMMoveWidget(panel->copyrL, 15, 185);
+	WMResizeWidget(panel->copyrL, 360, 60);
+	WMMoveWidget(panel->copyrL, 15, 190);
 	WMSetLabelTextAlignment(panel->copyrL, WALeft);
 	WMSetLabelText(panel->copyrL, COPYRIGHT_TEXT);
 	font = WMSystemFontOfSize(scr->wmscreen, 11);
@@ -1265,7 +1302,7 @@ void wShowInfoPanel(WScreen * scr)
 	}
 #endif
 
-	strbuf = wstrappend(strbuf, _("Supported image formats: "));
+	strbuf = wstrappend(strbuf, _("Image formats: "));
 	strl = RSupportedFileFormats();
 	separator = NULL;
 	for (i = 0; strl[i] != NULL; i++) {
@@ -1278,19 +1315,19 @@ void wShowInfoPanel(WScreen * scr)
 	strbuf = wstrappend(strbuf, _("\nAdditional support for: "));
 	strbuf = wstrappend(strbuf, "WMSPEC");
 
-#ifdef HAVE_XRANDR
-	strbuf = wstrappend(strbuf, ", XRandR ");
-	if (has_randr)
-		strbuf = wstrappend(strbuf, _("(Supported)"));
-	else
-		strbuf = wstrappend(strbuf, _("(Unsupported)"));
-#endif
-
-#ifdef MWM_HINTS
+#ifdef USE_MWM_HINTS
 	strbuf = wstrappend(strbuf, ", MWM");
 #endif
 
-#ifdef XINERAMA
+#ifdef USE_DOCK_XDND
+	strbuf = wstrappend(strbuf, ", XDnD");
+#endif
+
+#ifdef USE_MAGICK
+	strbuf = wstrappend(strbuf, ", ImageMagick");
+#endif
+
+#ifdef USE_XINERAMA
 	strbuf = wstrappend(strbuf, _("\n"));
 #ifdef SOLARIS_XINERAMA
 	strbuf = wstrappend(strbuf, _("Solaris "));
@@ -1298,13 +1335,23 @@ void wShowInfoPanel(WScreen * scr)
 	strbuf = wstrappend(strbuf, _("Xinerama: "));
 	{
 		char tmp[128];
-		snprintf(tmp, sizeof(tmp) - 1, _("%d heads found."), scr->xine_info.count);
+		snprintf(tmp, sizeof(tmp) - 1, _("%d head(s) found."), scr->xine_info.count);
 		strbuf = wstrappend(strbuf, tmp);
 	}
 #endif
 
+#ifdef USE_RANDR
+	strbuf = wstrappend(strbuf, _("\n"));
+	strbuf = wstrappend(strbuf, "RandR: ");
+	if (w_global.xext.randr.supported)
+		strbuf = wstrappend(strbuf, _("supported"));
+	else
+		strbuf = wstrappend(strbuf, _("unsupported"));
+	strbuf = wstrappend(strbuf, ".");
+#endif
+
 	panel->infoL = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->infoL, 350, 75);
+	WMResizeWidget(panel->infoL, 350, 80);
 	WMMoveWidget(panel->infoL, 15, 115);
 	WMSetLabelText(panel->infoL, strbuf);
 	font = WMSystemFontOfSize(scr->wmscreen, 11);
@@ -1318,17 +1365,14 @@ void wShowInfoPanel(WScreen * scr)
 	WMRealizeWidget(panel->win);
 	WMMapSubwidgets(panel->win);
 
-	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 382, 230, 0, 0, 0);
+	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, win_width, win_height, 0, 0, 0);
 
 	XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
 	WMMapWidget(panel->win);
 
-	{
-		WMPoint center = getCenter(scr, 382, 230);
-
-		wwin = wManageInternalWindow(scr, parent, None, _("Info"), center.x, center.y, 382, 230);
-	}
+	center = getCenter(scr, win_width, win_height);
+	wwin = wManageInternalWindow(scr, parent, None, _("Info"), center.x, center.y, win_width, win_height);
 
 	WSETUFLAG(wwin, no_closable, 0);
 	WSETUFLAG(wwin, no_close_button, 0);
@@ -1342,7 +1386,7 @@ void wShowInfoPanel(WScreen * scr)
 	wWindowMap(wwin);
 
 	panel->wwin = wwin;
-	thePanel = panel;
+	infoPanel = panel;
 }
 
 /*
@@ -1353,11 +1397,8 @@ void wShowInfoPanel(WScreen * scr)
 
 typedef struct {
 	WScreen *scr;
-
 	WWindow *wwin;
-
 	WMWindow *win;
-
 	WMLabel *licenseL;
 } LegalPanel;
 
@@ -1365,22 +1406,27 @@ static LegalPanel *legalPanel = NULL;
 
 static void destroyLegalPanel(WCoreWindow * foo, void *data, XEvent * event)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) foo;
+	(void) data;
+	(void) event;
+
 	WMUnmapWidget(legalPanel->win);
-
 	WMDestroyWidget(legalPanel->win);
-
 	wUnmanageWindow(legalPanel->wwin, False, False);
-
 	wfree(legalPanel);
-
 	legalPanel = NULL;
 }
 
-void wShowLegalPanel(WScreen * scr)
+void wShowLegalPanel(WScreen *scr)
 {
+	const int win_width = 420;
+	const int win_height = 250;
+	const int margin = 10;
 	LegalPanel *panel;
 	Window parent;
 	WWindow *wwin;
+	WMPoint center;
 
 	if (legalPanel) {
 		if (legalPanel->scr == scr) {
@@ -1391,44 +1437,37 @@ void wShowLegalPanel(WScreen * scr)
 	}
 
 	panel = wmalloc(sizeof(LegalPanel));
-
 	panel->scr = scr;
-
 	panel->win = WMCreateWindow(scr->wmscreen, "legal");
-	WMResizeWidget(panel->win, 420, 250);
+	WMResizeWidget(panel->win, win_width, win_height);
 
 	panel->licenseL = WMCreateLabel(panel->win);
 	WMSetLabelWraps(panel->licenseL, True);
-	WMResizeWidget(panel->licenseL, 400, 230);
-	WMMoveWidget(panel->licenseL, 10, 10);
+	WMResizeWidget(panel->licenseL, win_width - (2 * margin), win_height - (2 * margin));
+	WMMoveWidget(panel->licenseL, margin, margin);
 	WMSetLabelTextAlignment(panel->licenseL, WALeft);
 	WMSetLabelText(panel->licenseL,
-		       _("    Window Maker is free software; you can redistribute it and/or\n"
-			 "modify it under the terms of the GNU General Public License as\n"
-			 "published by the Free Software Foundation; either version 2 of the\n"
+		       _("    Window Maker is free software; you can redistribute it and/or "
+			 "modify it under the terms of the GNU General Public License as "
+			 "published by the Free Software Foundation; either version 2 of the "
 			 "License, or (at your option) any later version.\n\n"
-			 "    Window Maker is distributed in the hope that it will be useful,\n"
-			 "but WITHOUT ANY WARRANTY; without even the implied warranty\n"
-			 "of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
+			 "    Window Maker is distributed in the hope that it will be useful, "
+			 "but WITHOUT ANY WARRANTY; without even the implied warranty "
+			 "of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. "
 			 "See the GNU General Public License for more details.\n\n"
-			 "    You should have received a copy of the GNU General Public\n"
-			 "License along with this program; if not, write to the Free Software\n"
-			 "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA\n"
+			 "    You should have received a copy of the GNU General Public "
+			 "License along with this program; if not, write to the Free Software "
+			 "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA"
 			 "02110-1301 USA."));
 	WMSetLabelRelief(panel->licenseL, WRGroove);
 
 	WMRealizeWidget(panel->win);
 	WMMapSubwidgets(panel->win);
 
-	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 420, 250, 0, 0, 0);
-
+	parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, win_width, win_height, 0, 0, 0);
 	XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
-
-	{
-		WMPoint center = getCenter(scr, 420, 250);
-
-		wwin = wManageInternalWindow(scr, parent, None, _("Legal"), center.x, center.y, 420, 250);
-	}
+	center = getCenter(scr, win_width, win_height);
+	wwin = wManageInternalWindow(scr, parent, None, _("Legal"), center.x, center.y, win_width, win_height);
 
 	WSETUFLAG(wwin, no_closable, 0);
 	WSETUFLAG(wwin, no_close_button, 0);
@@ -1438,13 +1477,10 @@ void wShowLegalPanel(WScreen * scr)
 	wFrameWindowHideButton(wwin->frame, WFF_LANGUAGE_BUTTON);
 #endif
 	wwin->frame->on_click_right = destroyLegalPanel;
-
 	panel->wwin = wwin;
 
 	WMMapWidget(panel->win);
-
 	wWindowMap(wwin);
-
 	legalPanel = panel;
 }
 
@@ -1454,8 +1490,6 @@ void wShowLegalPanel(WScreen * scr)
  ***********************************************************************
  */
 
-extern WDDomain *WDWindowAttributes;
-
 typedef struct _CrashPanel {
 	WMWindow *win;		/* main window */
 
@@ -1492,6 +1526,9 @@ static void okButtonCallback(void *self, void *clientData)
 {
 	CrashPanel *panel = (CrashPanel *) clientData;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	panel->done = True;
 }
 
@@ -1510,7 +1547,7 @@ static WMPixmap *getWindowMakerIconImage(WMScreen *scr)
 	char *path = NULL;
 
 	/* Get the Logo icon, without the default icon */
-	path = get_icon_filename(NULL, "Logo", "WMPanel", NULL, False);
+	path = get_icon_filename("Logo", "WMPanel", NULL, False);
 
 	if (path) {
 		RColor gray;
@@ -1540,8 +1577,6 @@ int wShowCrashingDialogPanel(int whatSig)
 	int action;
 	char buf[256];
 
-	panel = wmalloc(sizeof(CrashPanel));
-
 	screen_no = DefaultScreen(dpy);
 	scr_width = WidthOfScreen(ScreenOfDisplay(dpy, screen_no));
 	scr_height = HeightOfScreen(ScreenOfDisplay(dpy, screen_no));
@@ -1552,6 +1587,8 @@ int wShowCrashingDialogPanel(int whatSig)
 		return WMAbort;
 	}
 
+	panel = wmalloc(sizeof(CrashPanel));
+
 	panel->retKey = XKeysymToKeycode(dpy, XK_Return);
 
 	panel->win = WMCreateWindow(scr, "crashingDialog");
@@ -1591,9 +1628,11 @@ int wShowCrashingDialogPanel(int whatSig)
 	WMResizeWidget(panel->note2L, PWIDTH - 20, 100);
 	WMMoveWidget(panel->note2L, 10, 130);
 	WMSetLabelTextAlignment(panel->note2L, WALeft);
-	WMSetLabelText(panel->note2L,
-		       _(" This fatal error occured probably due to a bug."
-			 " Please fill the included BUGFORM and " "report it to bugs@windowmaker.info."));
+	snprintf(buf, sizeof(buf), /* Comment for the PO file: the %s is an email address */
+	         _(" This fatal error occured probably due to a bug."
+	           " Please fill the included BUGFORM and report it to %s."),
+	         PACKAGE_BUGREPORT);
+	WMSetLabelText(panel->note2L, buf);
 	WMSetLabelWraps(panel->note2L, True);
 
 	panel->whatF = WMCreateFrame(panel->win);
