@@ -3,6 +3,7 @@
  *  Window Maker window manager
  *
  *  Copyright (c) 1997-2003 Alfredo K. Kojima
+ *  Copyright (c) 2014 Window Maker Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,17 +33,19 @@
 #include <string.h>
 #include <strings.h>
 #include <time.h>
+#include <errno.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#ifdef SHAPE
+#include <X11/Xatom.h>
+#ifdef USE_XSHAPE
 # include <X11/extensions/shape.h>
 #endif
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 #include "xdnd.h"
 #endif
 
-#ifdef HAVE_XRANDR
+#ifdef USE_RANDR
 #include <X11/extensions/Xrandr.h>
 #endif
 
@@ -75,40 +78,11 @@
 #include "event.h"
 #include "winmenu.h"
 #include "switchmenu.h"
+#include "wsmap.h"
 
-/******** Global Variables **********/
-extern XContext wWinContext;
-extern XContext wVEdgeContext;
-
-extern Cursor wCursor[WCUR_LAST];
-
-extern WShortKey wKeyBindings[WKBD_LAST];
-extern int wScreenCount;
-extern Time LastTimestamp;
-extern Time LastFocusChange;
 
 #define MOD_MASK wPreferences.modifier_mask
 
-extern Atom _XA_WM_COLORMAP_NOTIFY;
-
-extern Atom _XA_WM_CHANGE_STATE;
-extern Atom _XA_WM_DELETE_WINDOW;
-extern Atom _XA_GNUSTEP_WM_ATTR;
-extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
-extern Atom _XA_GNUSTEP_TITLEBAR_STATE;
-extern Atom _XA_WINDOWMAKER_WM_FUNCTION;
-extern Atom _XA_WINDOWMAKER_COMMAND;
-extern Atom _XA_WM_IGNORE_FOCUS_EVENTS;
-
-#ifdef SHAPE
-extern Bool wShapeSupported;
-extern int wShapeEventBase;
-#endif
-
-#ifdef KEEP_XKB_LOCK_STATUS
-extern int wXkbEventBase;
-#endif
-
 /************ Local stuff ***********/
 
 static void saveTimestamp(XEvent *event);
@@ -129,20 +103,22 @@ static void handleKeyPress(XEvent *event);
 static void handleFocusIn(XEvent *event);
 static void handleMotionNotify(XEvent *event);
 static void handleVisibilityNotify(XEvent *event);
-static void handle_inotify_events(int fd, int wd);
+static void handle_inotify_events(void);
+static void handle_selection_request(XSelectionRequestEvent *event);
+static void handle_selection_clear(XSelectionClearEvent *event);
 static void wdelete_death_handler(WMagicNumber id);
 
 
-#ifdef SHAPE
+#ifdef USE_XSHAPE
 static void handleShapeNotify(XEvent *event);
 #endif
 
 #ifdef KEEP_XKB_LOCK_STATUS
-static void handleXkbIndicatorStateNotify(XEvent *event);
+static void handleXkbIndicatorStateNotify(XkbEvent *event);
 #endif
 
 /* real dead process handler */
-static void handleDeadProcess(void *foo);
+static void handleDeadProcess(void);
 
 typedef struct DeadProcesses {
 	pid_t pid;
@@ -195,7 +171,7 @@ static void wdelete_death_handler(WMagicNumber id)
 void DispatchEvent(XEvent * event)
 {
 	if (deathHandlers)
-		handleDeadProcess(NULL);
+		handleDeadProcess();
 
 	if (WCHECK_STATE(WSTATE_NEED_EXIT)) {
 		WCHANGE_STATE(WSTATE_EXITING);
@@ -295,12 +271,20 @@ void DispatchEvent(XEvent * event)
 		break;
 
 	case ConfigureNotify:
-#ifdef HAVE_XRANDR
+#ifdef USE_RANDR
 		if (event->xconfigure.window == DefaultRootWindow(dpy))
 			XRRUpdateConfiguration(event);
 #endif
 		break;
 
+	case SelectionRequest:
+		handle_selection_request(&event->xselectionrequest);
+		break;
+
+	case SelectionClear:
+		handle_selection_clear(&event->xselectionclear);
+		break;
+
 	default:
 		handleExtensions(event);
 		break;
@@ -321,12 +305,12 @@ void DispatchEvent(XEvent * event)
  * 	Calls wDefaultsCheckDomains if config database is updated
  *----------------------------------------------------------------------
  */
-/* allow 5 simultaneous events, with path + filenames up to 64 chars */
-#define BUFF_SIZE ((sizeof(struct inotify_event) + 64)*5)
-static void handle_inotify_events(int fd, int wd)
+static void handle_inotify_events(void)
 {
-	ssize_t eventQLength, i = 0;
-	char buff[BUFF_SIZE] = { 0 };
+	ssize_t eventQLength;
+	size_t i = 0;
+	/* Make room for at lease 5 simultaneous events, with path + filenames */
+	char buff[ (sizeof(struct inotify_event) + NAME_MAX + 1) * 5 ];
 	/* Check config only once per read of the event queue */
 	int oneShotFlag = 0;
 
@@ -337,7 +321,13 @@ static void handle_inotify_events(int fd, int wd)
 	 * occur as a result of an Xevent - so the event queue should never have more than
 	 * a few entries before a read().
 	 */
-	eventQLength = read(fd, buff, BUFF_SIZE);
+	eventQLength = read(w_global.inotify.fd_event_queue,
+	                    buff, sizeof(buff) );
+
+	if (eventQLength < 0) {
+		wwarning(_("read problem when trying to get INotify event: %s"), strerror(errno));
+		return;
+	}
 
 	/* check what events occured */
 	/* Should really check wd here too, but for now we only have one watch! */
@@ -350,17 +340,27 @@ static void handle_inotify_events(int fd, int wd)
 		if (pevent->mask & IN_DELETE_SELF) {
 			wwarning(_("the defaults database has been deleted!"
 				   " Restart Window Maker to create the database" " with the default settings"));
-			close(fd);
+
+			if (w_global.inotify.fd_event_queue >= 0) {
+				close(w_global.inotify.fd_event_queue);
+				w_global.inotify.fd_event_queue = -1;
+			}
 		}
 		if (pevent->mask & IN_UNMOUNT) {
 			wwarning(_("the unit containing the defaults database has"
 				   " been unmounted. Setting --static mode." " Any changes will not be saved."));
-			close(fd);
+
+			if (w_global.inotify.fd_event_queue >= 0) {
+				close(w_global.inotify.fd_event_queue);
+				w_global.inotify.fd_event_queue = -1;
+			}
+
 			wPreferences.flags.noupdates = 1;
 		}
 		if ((pevent->mask & IN_MODIFY) && oneShotFlag == 0) {
 			wwarning(_("Inotify: Reading config files in defaults database."));
 			wDefaultsCheckDomains(NULL);
+			oneShotFlag = 1;
 		}
 
 		/* move to next event in the buffer */
@@ -386,13 +386,11 @@ noreturn void EventLoop(void)
 {
 	XEvent event;
 #ifdef HAVE_INOTIFY
-	extern int inotifyFD;
-	extern int inotifyWD;
 	struct timeval time;
 	fd_set rfds;
 	int retVal = 0;
 
-	if (inotifyFD < 0 || inotifyWD < 0)
+	if (w_global.inotify.fd_event_queue < 0 || w_global.inotify.wd_defaults < 0)
 		retVal = -1;
 #endif
 
@@ -405,20 +403,21 @@ noreturn void EventLoop(void)
 			time.tv_sec = 0;
 			time.tv_usec = 0;
 			FD_ZERO(&rfds);
-			FD_SET(inotifyFD, &rfds);
+			FD_SET(w_global.inotify.fd_event_queue, &rfds);
 
 			/* check for available read data from inotify - don't block! */
-			retVal = select(inotifyFD + 1, &rfds, NULL, NULL, &time);
+			retVal = select(w_global.inotify.fd_event_queue + 1, &rfds, NULL, NULL, &time);
 
 			if (retVal < 0) {	/* an error has occured */
 				wwarning(_("select failed. The inotify instance will be closed."
 					   " Changes to the defaults database will require"
 					   " a restart to take effect."));
-				close(inotifyFD);
+				close(w_global.inotify.fd_event_queue);
+				w_global.inotify.fd_event_queue = -1;
 				continue;
 			}
-			if (FD_ISSET(inotifyFD, &rfds))
-				handle_inotify_events(inotifyFD, inotifyWD);
+			if (FD_ISSET(w_global.inotify.fd_event_queue, &rfds))
+				handle_inotify_events();
 		}
 #endif
 	}
@@ -485,7 +484,7 @@ void NotifyDeadProcess(pid_t pid, unsigned char status)
 	deadProcessPtr++;
 }
 
-static void handleDeadProcess(void *foo)
+static void handleDeadProcess(void)
 {
 	DeathHandler *tmp;
 	int i;
@@ -527,31 +526,31 @@ static void saveTimestamp(XEvent * event)
 	switch (event->type) {
 	case ButtonRelease:
 	case ButtonPress:
-		LastTimestamp = event->xbutton.time;
+		w_global.timestamp.last_event = event->xbutton.time;
 		break;
 	case KeyPress:
 	case KeyRelease:
-		LastTimestamp = event->xkey.time;
+		w_global.timestamp.last_event = event->xkey.time;
 		break;
 	case MotionNotify:
-		LastTimestamp = event->xmotion.time;
+		w_global.timestamp.last_event = event->xmotion.time;
 		break;
 	case PropertyNotify:
-		LastTimestamp = event->xproperty.time;
+		w_global.timestamp.last_event = event->xproperty.time;
 		break;
 	case EnterNotify:
 	case LeaveNotify:
-		LastTimestamp = event->xcrossing.time;
+		w_global.timestamp.last_event = event->xcrossing.time;
 		break;
 	case SelectionClear:
-		LastTimestamp = event->xselectionclear.time;
+		w_global.timestamp.last_event = event->xselectionclear.time;
 		break;
 	case SelectionRequest:
-		LastTimestamp = event->xselectionrequest.time;
+		w_global.timestamp.last_event = event->xselectionrequest.time;
 		break;
 	case SelectionNotify:
-		LastTimestamp = event->xselection.time;
-#ifdef XDND
+		w_global.timestamp.last_event = event->xselection.time;
+#ifdef USE_DOCK_XDND
 		wXDNDProcessSelection(event);
 #endif
 		break;
@@ -565,22 +564,18 @@ static int matchWindow(const void *item, const void *cdata)
 
 static void handleExtensions(XEvent * event)
 {
-#ifdef KEEP_XKB_LOCK_STATUS
-	XkbEvent *xkbevent;
-	xkbevent = (XkbEvent *) event;
-#endif				/*KEEP_XKB_LOCK_STATUS */
-#ifdef SHAPE
-	if (wShapeSupported && event->type == (wShapeEventBase + ShapeNotify)) {
+#ifdef USE_XSHAPE
+	if (w_global.xext.shape.supported && event->type == (w_global.xext.shape.event_base + ShapeNotify)) {
 		handleShapeNotify(event);
 	}
 #endif
 #ifdef KEEP_XKB_LOCK_STATUS
-	if (wPreferences.modelock && (xkbevent->type == wXkbEventBase)) {
-		handleXkbIndicatorStateNotify(event);
+	if (wPreferences.modelock && (event->type == w_global.xext.xkb.event_base)) {
+		handleXkbIndicatorStateNotify((XkbEvent *) event);
 	}
 #endif				/*KEEP_XKB_LOCK_STATUS */
-#ifdef HAVE_XRANDR
-	if (has_randr && event->type == (randr_event_base + RRScreenChangeNotify)) {
+#ifdef USE_RANDR
+	if (w_global.xext.randr.supported && event->type == (w_global.xext.randr.event_base + RRScreenChangeNotify)) {
 		/* From xrandr man page: "Clients must call back into Xlib using
 		 * XRRUpdateConfiguration when screen configuration change notify
 		 * events are generated */
@@ -598,7 +593,8 @@ static void handleMapRequest(XEvent * ev)
 	WScreen *scr = NULL;
 	Window window = ev->xmaprequest.window;
 
-	if ((wwin = wWindowFor(window))) {
+	wwin = wWindowFor(window);
+	if (wwin != NULL) {
 		if (wwin->flags.shaded) {
 			wUnshadeWindow(wwin);
 		}
@@ -717,7 +713,7 @@ static void handleExpose(XEvent * event)
 
 	while (XCheckTypedWindowEvent(dpy, event->xexpose.window, Expose, &ev)) ;
 
-	if (XFindContext(dpy, event->xexpose.window, wWinContext, (XPointer *) & desc) == XCNOENT) {
+	if (XFindContext(dpy, event->xexpose.window, w_global.context.client_win, (XPointer *) & desc) == XCNOENT) {
 		return;
 	}
 
@@ -726,8 +722,38 @@ static void handleExpose(XEvent * event)
 	}
 }
 
-static void executeButtonAction(WScreen * scr, XEvent * event, int action)
+static void executeWheelAction(WScreen *scr, XEvent *event, int action)
 {
+	WWindow *wwin;
+	Bool next_direction;
+
+	if (event->xbutton.button == Button5 || event->xbutton.button == Button6)
+		next_direction = False;
+	else
+		next_direction = True;
+
+	switch (action) {
+	case WA_SWITCH_WORKSPACES:
+		if (next_direction)
+			wWorkspaceRelativeChange(scr, 1);
+		else
+			wWorkspaceRelativeChange(scr, -1);
+		break;
+
+	case WA_SWITCH_WINDOWS:
+		wwin = scr->focused_window;
+		if (next_direction)
+			wWindowFocusNext(wwin, True);
+		else
+			wWindowFocusPrev(wwin, True);
+		break;
+	}
+}
+
+static void executeButtonAction(WScreen *scr, XEvent *event, int action)
+{
+	WWindow *wwin;
+
 	switch (action) {
 	case WA_SELECT_WINDOWS:
 		wUnselectWindows(scr);
@@ -752,7 +778,19 @@ static void executeButtonAction(WScreen * scr, XEvent * event, int action)
 				event->xbutton.window = scr->switch_menu->frame->core->window;
 		}
 		break;
-	default:
+	case WA_MOVE_PREVWORKSPACE:
+		wWorkspaceRelativeChange(scr, -1);
+		break;
+	case WA_MOVE_NEXTWORKSPACE:
+		wWorkspaceRelativeChange(scr, 1);
+		break;
+	case WA_MOVE_PREVWINDOW:
+		wwin = scr->focused_window;
+		wWindowFocusPrev(wwin, True);
+		break;
+	case WA_MOVE_NEXTWINDOW:
+		wwin = scr->focused_window;
+		wWindowFocusNext(wwin, True);
 		break;
 	}
 }
@@ -769,23 +807,31 @@ static void handleButtonPress(XEvent * event)
 	wBalloonHide(scr);
 #endif
 
-	if (event->xbutton.window == scr->root_win) {
+	if (!wPreferences.disable_root_mouse && event->xbutton.window == scr->root_win) {
 		if (event->xbutton.button == Button1 && wPreferences.mouse_button1 != WA_NONE) {
 			executeButtonAction(scr, event, wPreferences.mouse_button1);
 		} else if (event->xbutton.button == Button2 && wPreferences.mouse_button2 != WA_NONE) {
 			executeButtonAction(scr, event, wPreferences.mouse_button2);
 		} else if (event->xbutton.button == Button3 && wPreferences.mouse_button3 != WA_NONE) {
 			executeButtonAction(scr, event, wPreferences.mouse_button3);
-		} else if (event->xbutton.button == Button4 && wPreferences.mouse_wheel != WA_NONE) {
-			wWorkspaceRelativeChange(scr, 1);
-		} else if (event->xbutton.button == Button5 && wPreferences.mouse_wheel != WA_NONE) {
-			wWorkspaceRelativeChange(scr, -1);
+		} else if (event->xbutton.button == Button8 && wPreferences.mouse_button8 != WA_NONE) {
+			executeButtonAction(scr, event, wPreferences.mouse_button8);
+		}else if (event->xbutton.button == Button9 && wPreferences.mouse_button9 != WA_NONE) {
+			executeButtonAction(scr, event, wPreferences.mouse_button9);
+		} else if (event->xbutton.button == Button4 && wPreferences.mouse_wheel_scroll != WA_NONE) {
+			executeWheelAction(scr, event, wPreferences.mouse_wheel_scroll);
+		} else if (event->xbutton.button == Button5 && wPreferences.mouse_wheel_scroll != WA_NONE) {
+			executeWheelAction(scr, event, wPreferences.mouse_wheel_scroll);
+		} else if (event->xbutton.button == Button6 && wPreferences.mouse_wheel_tilt != WA_NONE) {
+			executeWheelAction(scr, event, wPreferences.mouse_wheel_tilt);
+		} else if (event->xbutton.button == Button7 && wPreferences.mouse_wheel_tilt != WA_NONE) {
+			executeWheelAction(scr, event, wPreferences.mouse_wheel_tilt);
 		}
 	}
 
 	desc = NULL;
-	if (XFindContext(dpy, event->xbutton.subwindow, wWinContext, (XPointer *) & desc) == XCNOENT) {
-		if (XFindContext(dpy, event->xbutton.window, wWinContext, (XPointer *) & desc) == XCNOENT) {
+	if (XFindContext(dpy, event->xbutton.subwindow, w_global.context.client_win, (XPointer *) & desc) == XCNOENT) {
+		if (XFindContext(dpy, event->xbutton.window, w_global.context.client_win, (XPointer *) & desc) == XCNOENT) {
 			return;
 		}
 	}
@@ -897,7 +943,8 @@ static void handleConfigureRequest(XEvent * event)
 {
 	WWindow *wwin;
 
-	if (!(wwin = wWindowFor(event->xconfigurerequest.window))) {
+	wwin = wWindowFor(event->xconfigurerequest.window);
+	if (wwin == NULL) {
 		/*
 		 * Configure request for unmapped window
 		 */
@@ -934,7 +981,7 @@ static void handleClientMessage(XEvent * event)
 	WObjDescriptor *desc;
 
 	/* handle transition from Normal to Iconic state */
-	if (event->xclient.message_type == _XA_WM_CHANGE_STATE
+	if (event->xclient.message_type == w_global.atom.wm.change_state
 	    && event->xclient.format == 32 && event->xclient.data.l[0] == IconicState) {
 
 		wwin = wWindowFor(event->xclient.window);
@@ -942,7 +989,7 @@ static void handleClientMessage(XEvent * event)
 			return;
 		if (!wwin->flags.miniaturized)
 			wIconifyWindow(wwin);
-	} else if (event->xclient.message_type == _XA_WM_COLORMAP_NOTIFY && event->xclient.format == 32) {
+	} else if (event->xclient.message_type == w_global.atom.wm.colormap_notify && event->xclient.format == 32) {
 		WScreen *scr = wScreenForRootWindow(event->xclient.window);
 
 		if (!scr)
@@ -953,7 +1000,7 @@ static void handleClientMessage(XEvent * event)
 		} else {	/* stopping */
 			wColormapAllowClientInstallation(scr, False);
 		}
-	} else if (event->xclient.message_type == _XA_WINDOWMAKER_COMMAND) {
+	} else if (event->xclient.message_type == w_global.atom.wmaker.command) {
 
 		char *command;
 		size_t len;
@@ -971,7 +1018,7 @@ static void handleClientMessage(XEvent * event)
 
 		wfree(command);
 
-	} else if (event->xclient.message_type == _XA_WINDOWMAKER_WM_FUNCTION) {
+	} else if (event->xclient.message_type == w_global.atom.wmaker.wm_function) {
 		WApplication *wapp;
 		int done = 0;
 		wapp = wApplicationOf(event->xclient.window);
@@ -1002,7 +1049,7 @@ static void handleClientMessage(XEvent * event)
 				}
 			}
 		}
-	} else if (event->xclient.message_type == _XA_GNUSTEP_WM_ATTR) {
+	} else if (event->xclient.message_type == w_global.atom.gnustep.wm_attr) {
 		wwin = wWindowFor(event->xclient.window);
 		if (!wwin)
 			return;
@@ -1017,7 +1064,7 @@ static void handleClientMessage(XEvent * event)
 			}
 			break;
 		}
-	} else if (event->xclient.message_type == _XA_GNUSTEP_TITLEBAR_STATE) {
+	} else if (event->xclient.message_type == w_global.atom.gnustep.titlebar_state) {
 		wwin = wWindowFor(event->xclient.window);
 		if (!wwin)
 			return;
@@ -1032,24 +1079,24 @@ static void handleClientMessage(XEvent * event)
 			wFrameWindowChangeState(wwin->frame, WS_FOCUSED);
 			break;
 		}
-	} else if (event->xclient.message_type == _XA_WM_IGNORE_FOCUS_EVENTS) {
+	} else if (event->xclient.message_type == w_global.atom.wm.ignore_focus_events) {
 		WScreen *scr = wScreenForRootWindow(event->xclient.window);
 		if (!scr)
 			return;
 		scr->flags.ignore_focus_events = event->xclient.data.l[0] ? 1 : 0;
 	} else if (wNETWMProcessClientMessage(&event->xclient)) {
 		/* do nothing */
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 	} else if (wXDNDProcessClientMessage(&event->xclient)) {
 		/* do nothing */
-#endif				/* XDND */
+#endif	/* USE_DOCK_XDND */
 	} else {
 		/*
 		 * Non-standard thing, but needed by OffiX DND.
 		 * For when the icon frame gets a ClientMessage
 		 * that should have gone to the icon_window.
 		 */
-		if (XFindContext(dpy, event->xbutton.window, wWinContext, (XPointer *) & desc) != XCNOENT) {
+		if (XFindContext(dpy, event->xbutton.window, w_global.context.client_win, (XPointer *) & desc) != XCNOENT) {
 			struct WIcon *icon = NULL;
 
 			if (desc->parent_type == WCLASS_MINIWINDOW) {
@@ -1099,7 +1146,7 @@ static void handleEnterNotify(XEvent * event)
 		}
 	}
 
-	if (XFindContext(dpy, event->xcrossing.window, wWinContext, (XPointer *) & desc) != XCNOENT) {
+	if (XFindContext(dpy, event->xcrossing.window, w_global.context.client_win, (XPointer *) & desc) != XCNOENT) {
 		if (desc->handle_enternotify)
 			(*desc->handle_enternotify) (desc, event);
 	}
@@ -1163,13 +1210,13 @@ static void handleLeaveNotify(XEvent * event)
 {
 	WObjDescriptor *desc = NULL;
 
-	if (XFindContext(dpy, event->xcrossing.window, wWinContext, (XPointer *) & desc) != XCNOENT) {
+	if (XFindContext(dpy, event->xcrossing.window, w_global.context.client_win, (XPointer *) & desc) != XCNOENT) {
 		if (desc->handle_leavenotify)
 			(*desc->handle_leavenotify) (desc, event);
 	}
 }
 
-#ifdef SHAPE
+#ifdef USE_XSHAPE
 static void handleShapeNotify(XEvent * event)
 {
 	XShapeEvent *shev = (XShapeEvent *) event;
@@ -1205,18 +1252,18 @@ static void handleShapeNotify(XEvent * event)
 		wWindowSetShape(wwin);
 	}
 }
-#endif				/* SHAPE */
+#endif				/* USE_XSHAPE */
 
 #ifdef KEEP_XKB_LOCK_STATUS
 /* please help ]d if you know what to do */
-static void handleXkbIndicatorStateNotify(XEvent *event)
+static void handleXkbIndicatorStateNotify(XkbEvent *event)
 {
 	WWindow *wwin;
 	WScreen *scr;
 	XkbStateRec staterec;
 	int i;
 
-	for (i = 0; i < wScreenCount; i++) {
+	for (i = 0; i < w_global.screen_count; i++) {
 		scr = wScreenWithNumber(i);
 		wwin = scr->focused_window;
 		if (wwin && wwin->flags.focused) {
@@ -1340,7 +1387,7 @@ static void handleKeyPress(XEvent * event)
 #endif				/*KEEP_XKB_LOCK_STATUS */
 
 	/* ignore CapsLock */
-	modifiers = event->xkey.state & ValidModMask;
+	modifiers = event->xkey.state & w_global.shortcut.modifiers_mask;
 
 	for (i = 0; i < WKBD_LAST; i++) {
 		if (wKeyBindings[i].keycode == 0)
@@ -1410,7 +1457,7 @@ static void handleKeyPress(XEvent * event)
 			CloseWindowMenu(scr);
 
 			if (wwin->protocols.MINIATURIZE_WINDOW)
-				wClientSendProtocol(wwin, _XA_GNUSTEP_WM_MINIATURIZE_WINDOW, event->xbutton.time);
+				wClientSendProtocol(wwin, w_global.atom.gnustep.wm_miniaturize_window, event->xbutton.time);
 			else {
 				wIconifyWindow(wwin);
 			}
@@ -1517,6 +1564,13 @@ static void handleKeyPress(XEvent * event)
 			handleMaximize(wwin, MAX_MAXIMUS | MAX_KEYBOARD);
 		}
 		break;
+	case WKBD_OMNIPRESENT:
+		if (ISMAPPED(wwin) && ISFOCUSED(wwin)) {
+			CloseWindowMenu(scr);
+
+			wWindowSetOmnipresent(wwin, !wwin->flags.omnipresent);
+		}
+		break;
 	case WKBD_RAISE:
 		if (ISMAPPED(wwin) && ISFOCUSED(wwin)) {
 			CloseWindowMenu(scr);
@@ -1558,7 +1612,7 @@ static void handleKeyPress(XEvent * event)
 		if (ISMAPPED(wwin) && ISFOCUSED(wwin) && !WFLAGP(wwin, no_closable)) {
 			CloseWindowMenu(scr);
 			if (wwin->protocols.DELETE_WINDOW)
-				wClientSendProtocol(wwin, _XA_WM_DELETE_WINDOW, event->xkey.time);
+				wClientSendProtocol(wwin, w_global.atom.wm.delete_window, event->xkey.time);
 		}
 		break;
 	case WKBD_SELECT:
@@ -1566,6 +1620,12 @@ static void handleKeyPress(XEvent * event)
 			wSelectWindow(wwin, !wwin->flags.selected);
 		}
 		break;
+
+	case WKBD_WORKSPACEMAP:
+		if (wPreferences.enable_workspace_pager)
+			StartWorkspaceMap(scr);
+		break;
+
 	case WKBD_FOCUSNEXT:
 		StartWindozeCycle(wwin, event, True, False);
 		break;
@@ -1718,17 +1778,17 @@ static void handleKeyPress(XEvent * event)
 		break;
 
 	case WKBD_SWITCH_SCREEN:
-		if (wScreenCount > 1) {
+		if (w_global.screen_count > 1) {
 			WScreen *scr2;
 			int i;
 
 			/* find index of this screen */
-			for (i = 0; i < wScreenCount; i++) {
+			for (i = 0; i < w_global.screen_count; i++) {
 				if (wScreenWithNumber(i) == scr)
 					break;
 			}
 			i++;
-			if (i >= wScreenCount) {
+			if (i >= w_global.screen_count) {
 				i = 0;
 			}
 			scr2 = wScreenWithNumber(i);
@@ -1740,6 +1800,26 @@ static void handleKeyPress(XEvent * event)
 		}
 		break;
 
+	case WKBD_RUN:
+	{
+		char *cmdline;
+
+		cmdline = ExpandOptions(scr, _("exec %A(Run,Type command to run:)"));
+
+		if (cmdline) {
+			XGrabPointer(dpy, scr->root_win, True, 0,
+			     GrabModeAsync, GrabModeAsync, None, wPreferences.cursor[WCUR_WAIT], CurrentTime);
+			XSync(dpy, False);
+
+			ExecuteShellCommand(scr, cmdline);
+			wfree(cmdline);
+
+			XUngrabPointer(dpy, CurrentTime);
+			XSync(dpy, False);
+		}
+		break;
+	}
+
 	case WKBD_NEXTWSLAYER:
 	case WKBD_PREVWSLAYER:
 		{
@@ -1803,7 +1883,7 @@ static void handleMotionNotify(XEvent * event)
 
 			menu = wMenuUnderPointer(scr);
 			if (menu != NULL)
-				wMenuScroll(menu, event);
+				wMenuScroll(menu);
 		}
 	}
 }
@@ -1817,3 +1897,98 @@ static void handleVisibilityNotify(XEvent * event)
 		return;
 	wwin->flags.obscured = (event->xvisibility.state == VisibilityFullyObscured);
 }
+
+static void handle_selection_request(XSelectionRequestEvent *event)
+{
+#ifdef USE_ICCCM_WMREPLACE
+	static Atom atom_version = None;
+	WScreen *scr;
+	XSelectionEvent notify;
+
+	/*
+	 * This event must be sent to the slection requester to not block him
+	 *
+	 * We create it with the answer 'there is no selection' by default
+	 */
+	notify.type = SelectionNotify;
+	notify.display = dpy;
+	notify.requestor = event->requestor;
+	notify.selection = event->selection;
+	notify.target = event->target;
+	notify.property = None; /* This says that there is no selection */
+	notify.time = event->time;
+
+	scr = wScreenForWindow(event->owner);
+	if (!scr)
+		goto not_our_selection;
+
+	if (event->owner != scr->info_window)
+		goto not_our_selection;
+
+	if (event->selection != scr->sn_atom)
+		goto not_our_selection;
+
+	if (atom_version == None)
+		atom_version = XInternAtom(dpy, "VERSION", False);
+
+	if (event->target == atom_version) {
+		static const long icccm_version[] = { 2, 0 };
+
+		/*
+		 * This protocol is defined in ICCCM 2.0:
+		 * http://www.x.org/releases/X11R7.7/doc/xorg-docs/icccm/icccm.html
+		 *  "Communication with the Window Manager by Means of Selections"
+		 */
+
+		/*
+		 * Setting the property means the content of the selection is available
+		 * According to the ICCCM spec, we need to support being asked for a property
+		 * set to 'None' for compatibility with old clients
+		 */
+		notify.property = (event->property == None)?(event->target):(event->property);
+
+		XChangeProperty(dpy, event->requestor, notify.property,
+		                XA_INTEGER, 32, PropModeReplace,
+		                (unsigned char *) icccm_version, wlengthof(icccm_version));
+	}
+
+ not_our_selection:
+	if (notify.property == None)
+		wwarning("received SelectionRequest(%s) for target=\"%s\" from requestor 0x%lX but we have no answer",
+		         XGetAtomName(dpy, event->selection), XGetAtomName(dpy, event->target), (long) event->requestor);
+
+	/* Send the answer to the requestor */
+	XSendEvent(dpy, event->requestor, False, 0L, (XEvent *) &notify);
+
+#else
+	/*
+	 * If the support for ICCCM window manager replacement was not enabled, we should not receive
+	 * this kind of event, so we just ignore it (Conceptually, we should reply with 'SelectionNotify'
+	 * event with property set to 'None' to tell that we don't have this selection, but that is a bit
+	 * costly for an event that shall never happen).
+	 */
+	(void) event;
+#endif
+}
+
+static void handle_selection_clear(XSelectionClearEvent *event)
+{
+#ifdef USE_ICCCM_WMREPLACE
+	WScreen *scr = wScreenForWindow(event->window);
+
+	if (!scr)
+		return;
+
+	if (event->selection != scr->sn_atom)
+		return;
+
+	wmessage(_("another window manager is replacing us!"));
+	Shutdown(WSExitMode);
+#else
+	/*
+	 * If the support for ICCCM window manager replacement was not enabled, we should not receive
+	 * this kind of event, so we simply do nothing.
+	 */
+	(void) event;
+#endif
+}
