@@ -54,11 +54,9 @@
 #define CACHE_ICON_PATH "/Library/WindowMaker/CachedPixmaps"
 #define ICON_BORDER 3
 
-extern Cursor wCursor[WCUR_LAST];
-
-static void miniwindowExpose(WObjDescriptor * desc, XEvent * event);
-static void miniwindowMouseDown(WObjDescriptor * desc, XEvent * event);
-static void miniwindowDblClick(WObjDescriptor * desc, XEvent * event);
+static void miniwindowExpose(WObjDescriptor *desc, XEvent *event);
+static void miniwindowMouseDown(WObjDescriptor *desc, XEvent *event);
+static void miniwindowDblClick(WObjDescriptor *desc, XEvent *event);
 
 static WIcon *icon_create_core(WScreen *scr, int coord_x, int coord_y);
 
@@ -95,6 +93,9 @@ static void tileObserver(void *self, WMNotification *notif)
 {
 	WIcon *icon = (WIcon *) self;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) notif;
+
 	update_icon_pixmap(icon);
 
 	XClearArea(dpy, icon->core->window, 0, 0, 1, 1, True);
@@ -198,7 +199,7 @@ static WIcon *icon_create_core(WScreen *scr, int coord_x, int coord_y)
 	return icon;
 }
 
-void wIconDestroy(WIcon * icon)
+void wIconDestroy(WIcon *icon)
 {
 	WCoreWindow *core = icon->core;
 	WScreen *scr = core->screen_ptr;
@@ -224,6 +225,9 @@ void wIconDestroy(WIcon * icon)
 	if (icon->pixmap)
 		XFreePixmap(dpy, icon->pixmap);
 
+	if (icon->mini_preview)
+		XFreePixmap(dpy, icon->mini_preview);
+
 	unset_icon_image(icon);
 
 	wCoreDestroy(icon->core);
@@ -324,10 +328,12 @@ void wIconChangeTitle(WIcon *icon, WWindow *wwin)
 	if (icon->icon_name != NULL)
 		XFree(icon->icon_name);
 
-	/* Set the new one, using two methods */
+	/* Set the new one, using two methods to identify
+	the icon name or switch back to window name */
 	icon->icon_name = wNETWMGetIconName(wwin->client_win);
 	if (!icon->icon_name)
-		wGetIconName(dpy, wwin->client_win, &icon->icon_name);
+		if (!wGetIconName(dpy, wwin->client_win, &icon->icon_name))
+			icon->icon_name = wNETWMGetWindowName(wwin->client_win);
 }
 
 RImage *wIconValidateIconSize(RImage *icon, int max_size)
@@ -353,38 +359,34 @@ RImage *wIconValidateIconSize(RImage *icon, int max_size)
 	return icon;
 }
 
-Bool wIconChangeImageFile(WIcon *icon, const char *file)
+int wIconChangeImageFile(WIcon *icon, const char *file)
 {
 	WScreen *scr = icon->core->screen_ptr;
 	char *path;
 	RImage *image = NULL;
-	int error = 0;
 
 	/* If no new image, don't do nothing */
 	if (!file)
-		return True;
+		return 1;
 
 	/* Find the new image */
 	path = FindImage(wPreferences.icon_path, file);
-	if (path)
-		image = get_rimage_from_file(scr, path, wPreferences.icon_size);
-	else
-		error = 1;
+	if (!path)
+		return 0;
 
-	/* New image! */
-	if (!error && image) {
-		/* Set the new image */
-		set_icon_image_from_image(icon, image);
-		icon->file = wstrdup(path);
-		update_icon_pixmap(icon);
-	} else {
-		error = 1;
+	image = get_rimage_from_file(scr, path, wPreferences.icon_size);
+	if (!image) {
+		wfree(path);
+		return 0;
 	}
 
-	if (path)
-		wfree(path);
+	/* Set the new image */
+	set_icon_image_from_image(icon, image);
+	icon->file = wstrdup(path);
+	update_icon_pixmap(icon);
 
-	return !error;
+	wfree(path);
+	return 1;
 }
 
 static char *get_name_for_wwin(WWindow *wwin)
@@ -466,7 +468,7 @@ static RImage *get_wwindow_image_from_wmhints(WWindow *wwin, WIcon *icon)
  * Side effects:
  * 	New directories might be created.
  */
-char *wIconStore(WIcon * icon)
+char *wIconStore(WIcon *icon)
 {
 	char *path, *dir_path, *file;
 	int len = 0;
@@ -540,7 +542,7 @@ void wIconSetHighlited(WIcon *icon, Bool flag)
 	update_icon_pixmap(icon);
 }
 
-void wIconSelect(WIcon * icon)
+void wIconSelect(WIcon *icon)
 {
 	WScreen *scr = icon->core->screen_ptr;
 	icon->selected = !icon->selected;
@@ -585,6 +587,23 @@ void set_icon_image_from_image(WIcon *icon, RImage *image)
 	icon->file_image = image;
 }
 
+void set_icon_minipreview(WIcon *icon, RImage *image)
+{
+	Pixmap tmp;
+	RImage *scaled_mini_preview;
+	WScreen *scr = icon->core->screen_ptr;
+
+	scaled_mini_preview = RSmoothScaleImage(image, wPreferences.minipreview_size - 2 * MINIPREVIEW_BORDER,
+	                                  wPreferences.minipreview_size - 2 * MINIPREVIEW_BORDER);
+
+	if (RConvertImage(scr->rcontext, scaled_mini_preview, &tmp)) {
+		if (icon->mini_preview != None)
+			XFreePixmap(dpy, icon->mini_preview);
+		icon->mini_preview = tmp;
+	}
+	RReleaseImage(scaled_mini_preview);
+}
+
 void wIconUpdate(WIcon *icon)
 {
 	WWindow *wwin = NULL;
@@ -721,7 +740,7 @@ static void set_dockapp_in_icon(WIcon *icon)
 	    (attr.all_event_masks & ButtonPressMask))
 		wHackedGrabButton(Button1, MOD_MASK, icon->core->window, True,
 				  ButtonPressMask, GrabModeSync, GrabModeAsync,
-				  None, wCursor[WCUR_ARROW]);
+				  None, wPreferences.cursor[WCUR_ARROW]);
 }
 
 /* Get the RImage from the XWindow wm_hints */
@@ -788,21 +807,27 @@ void wIconPaint(WIcon *icon)
 
 /******************************************************************/
 
-static void miniwindowExpose(WObjDescriptor * desc, XEvent * event)
+static void miniwindowExpose(WObjDescriptor *desc, XEvent *event)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) event;
+
 	wIconPaint(desc->parent);
 }
 
-static void miniwindowDblClick(WObjDescriptor * desc, XEvent * event)
+static void miniwindowDblClick(WObjDescriptor *desc, XEvent *event)
 {
 	WIcon *icon = desc->parent;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) event;
+
 	assert(icon->owner != NULL);
 
 	wDeiconifyWindow(icon->owner);
 }
 
-static void miniwindowMouseDown(WObjDescriptor * desc, XEvent * event)
+static void miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
 {
 	WIcon *icon = desc->parent;
 	WWindow *wwin = icon->owner;
@@ -862,7 +887,7 @@ static void miniwindowMouseDown(WObjDescriptor * desc, XEvent * event)
 				    || abs(dy - ev.xmotion.y) >= MOVE_THRESHOLD) {
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_MOVE], CurrentTime);
+								 wPreferences.cursor[WCUR_MOVE], CurrentTime);
 					grabbed = 1;
 				} else {
 					break;
@@ -903,7 +928,7 @@ void set_icon_image_from_database(WIcon *icon, const char *wm_instance, const ch
 {
 	char *file = NULL;
 
-	file = get_icon_filename(icon->core->screen_ptr, wm_instance, wm_class, command, False);
+	file = get_icon_filename(wm_instance, wm_class, command, False);
 	if (file) {
 		icon->file = wstrdup(file);
 		icon->file_image = get_rimage_from_file(icon->core->screen_ptr, icon->file, wPreferences.icon_size);
