@@ -2,6 +2,7 @@
  *  Window Maker window manager
  *
  *  Copyright (c) 1997-2003 Alfredo K. Kojima
+ *  Copyright (c) 2014 Window Maker Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +22,23 @@
 #ifndef WMKEYBIND_H
 #define WMKEYBIND_H
 
+/* <X11/X.h> doesn't define these, even though XFree supports them */
+#ifndef Button6
+#define Button6 6
+#endif
+
+#ifndef Button7
+#define Button7 7
+#endif
+
+#ifndef Button8
+#define Button8 8
+#endif
+
+#ifndef Button9
+#define Button9 9
+#endif
+
 enum {
 	/* anywhere */
 	WKBD_ROOTMENU,
@@ -45,11 +63,13 @@ enum {
 	WKBD_RBCMAXIMIZE,
 	WKBD_MAXIMUS,
 	WKBD_SELECT,
+	WKBD_OMNIPRESENT,
 	WKBD_RAISE,
 	WKBD_LOWER,
 	WKBD_RAISELOWER,
 	WKBD_MOVERESIZE,
 	WKBD_SHADE,
+	WKBD_WORKSPACEMAP,
 	WKBD_FOCUSNEXT,
 	WKBD_FOCUSPREV,
 	WKBD_GROUPNEXT,
@@ -116,6 +136,9 @@ enum {
 	/* screen */
 	WKBD_SWITCH_SCREEN,
 
+	/* open "run" dialog */
+	WKBD_RUN,
+
 #ifdef KEEP_XKB_LOCK_STATUS
 	WKBD_TOGGLE,
 #endif
@@ -128,6 +151,12 @@ typedef struct WShortKey {
     KeyCode keycode;
 } WShortKey;
 
+/* ---[ Global Variables ]------------------------------------------------ */
+
+extern WShortKey wKeyBindings[WKBD_LAST];
+
+/* ---[ Functions ]------------------------------------------------------- */
+
 void wKeyboardInitialize(void);
 
 #endif /* WMKEYBIND_H */
