@@ -64,6 +64,7 @@
 #endif
 
 /****** Global Variables ******/
+struct wmaker_global_variables w_global;
 
 /* general info */
 
@@ -71,84 +72,10 @@ Display *dpy;
 
 char *ProgName;
 
-unsigned int ValidModMask = 0xff;
-
-#ifdef HAVE_INOTIFY
-int inotifyFD;
-int inotifyWD;
-#endif
-/* locale to use. NULL==POSIX or C */
-char *Locale = NULL;
-
-int wScreenCount = 0;
-
 struct WPreferences wPreferences;
 
 WShortKey wKeyBindings[WKBD_LAST];
 
-/* defaults domains */
-WDDomain *WDWindowMaker = NULL;
-WDDomain *WDWindowAttributes = NULL;
-WDDomain *WDRootMenu = NULL;
-
-/* XContexts */
-XContext wWinContext;
-XContext wAppWinContext;
-XContext wStackContext;
-XContext wVEdgeContext;
-
-/* Atoms */
-Atom _XA_WM_STATE;
-Atom _XA_WM_CHANGE_STATE;
-Atom _XA_WM_PROTOCOLS;
-Atom _XA_WM_TAKE_FOCUS;
-Atom _XA_WM_DELETE_WINDOW;
-Atom _XA_WM_SAVE_YOURSELF;
-Atom _XA_WM_CLIENT_LEADER;
-Atom _XA_WM_COLORMAP_WINDOWS;
-Atom _XA_WM_COLORMAP_NOTIFY;
-
-Atom _XA_GNUSTEP_WM_ATTR;
-Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
-Atom _XA_GNUSTEP_WM_RESIZEBAR;
-Atom _XA_GNUSTEP_TITLEBAR_STATE;
-
-Atom _XA_WINDOWMAKER_MENU;
-Atom _XA_WINDOWMAKER_WM_PROTOCOLS;
-Atom _XA_WINDOWMAKER_STATE;
-
-Atom _XA_WINDOWMAKER_WM_FUNCTION;
-Atom _XA_WINDOWMAKER_NOTICEBOARD;
-Atom _XA_WINDOWMAKER_COMMAND;
-
-Atom _XA_WINDOWMAKER_ICON_SIZE;
-Atom _XA_WINDOWMAKER_ICON_TILE;
-
-Atom _XA_WM_IGNORE_FOCUS_EVENTS;
-
-/* cursors */
-Cursor wCursor[WCUR_LAST];
-
-/* last event timestamp for XSetInputFocus */
-Time LastTimestamp = CurrentTime;
-/* timestamp on the last time we did XSetInputFocus() */
-Time LastFocusChange = CurrentTime;
-
-#ifdef SHAPE
-Bool wShapeSupported;
-int wShapeEventBase;
-#endif
-
-#ifdef KEEP_XKB_LOCK_STATUS
-Bool wXkbSupported;
-int wXkbEventBase;
-#endif
-
-/* special flags */
-char WProgramSigState = 0;
-char WProgramState = WSTATE_NORMAL;
-char WDelayedActionSet = 0;
-
 /* notifications */
 const char WMNManaged[] = "WMNManaged";
 const char WMNUnmanaged[] = "WMNUnmanaged";
@@ -211,7 +138,7 @@ static void setWVisualID(int screen, int val)
 		int oldlen = wVisualID_len;
 
 		wVisualID_len = screen + 1;
-		wVisualID = (int *)realloc(wVisualID, wVisualID_len * sizeof(int));
+		wVisualID = (int *)wrealloc(wVisualID, wVisualID_len * sizeof(int));
 		for (i = oldlen; i < wVisualID_len; i++) {
 			wVisualID[i] = -1;
 		}
@@ -274,6 +201,9 @@ noreturn void Exit(int status)
 	if (dpy)
 		XCloseDisplay(dpy);
 
+	RShutdown(); /* wrlib clean exit */
+	wutil_shutdown();  /* WUtil clean-up */
+
 	exit(status);
 }
 
@@ -316,9 +246,22 @@ void SetupEnvironment(WScreen * scr)
 		int len = strlen(DisplayName) + 64;
 		tmp = wmalloc(len);
 		snprintf(tmp, len, "DISPLAY=%s", XDisplayName(DisplayName));
-		ptr = strchr(strchr(tmp, ':'), '.');
-		if (ptr)
-			*ptr = 0;
+
+		/* Search from the end to be compatible with ipv6 address */
+		ptr = strrchr(tmp, ':');
+		if (ptr == NULL) {
+			static Bool message_already_displayed = False;
+
+			if (!message_already_displayed)
+				wwarning(_("the display name has an unexpected syntax: \"%s\""),
+				         XDisplayName(DisplayName));
+			message_already_displayed = True;
+		} else {
+			/* If found, remove the screen specification from the display variable */
+			ptr = strchr(ptr, '.');
+			if (ptr)
+				*ptr = 0;
+		}
 		snprintf(buf, sizeof(buf), ".%i", scr->screen);
 		strcat(tmp, buf);
 		putenv(tmp);
@@ -334,8 +277,13 @@ typedef struct {
 	char *command;
 } _tuple;
 
-static void shellCommandHandler(pid_t pid, unsigned char status, _tuple * data)
+static void shellCommandHandler(pid_t pid, unsigned int status, void *client_data)
 {
+	_tuple *data = (_tuple *) client_data;
+
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) pid;
+
 	if (status == 127) {
 		char *buffer;
 
@@ -390,7 +338,7 @@ void ExecuteShellCommand(WScreen *scr, const char *command)
 		data->scr = scr;
 		data->command = wstrdup(command);
 
-		wAddDeathHandler(pid, (WDeathHandler *) shellCommandHandler, data);
+		wAddDeathHandler(pid, shellCommandHandler, data);
 	}
 }
 
@@ -449,7 +397,7 @@ Bool RelaunchWindow(WWindow *wwin)
 		data->command = wtokenjoin(argv, argc);
 
 		/* not actually a shell command */
-		wAddDeathHandler(pid, (WDeathHandler *) shellCommandHandler, data);
+		wAddDeathHandler(pid, shellCommandHandler, data);
 
 		XFreeStringList(argv);
 	}
@@ -469,7 +417,7 @@ noreturn void wAbort(Bool dumpCore)
 	int i;
 	WScreen *scr;
 
-	for (i = 0; i < wScreenCount; i++) {
+	for (i = 0; i < w_global.screen_count; i++) {
 		scr = wScreenWithNumber(i);
 		if (scr)
 			RestoreDesktop(scr);
@@ -487,9 +435,13 @@ static void print_help(void)
 	puts(_("The Window Maker window manager for the X window system"));
 	puts("");
 	puts(_(" -display host:dpy	display to use"));
+#ifdef USE_ICCCM_WMREPLACE
+	puts(_(" --replace		replace running window manager"));
+#endif
 	puts(_(" --no-dock		do not open the application Dock"));
 	puts(_(" --no-clip		do not open the workspace Clip"));
 	puts(_(" --no-autolaunch	do not autolaunch applications"));
+	puts(_(" --no-drawer		disable drawers in the dock"));
 	puts(_(" --dont-restore		do not restore saved session"));
 
 	puts(_(" --locale locale	locale to use"));
@@ -499,6 +451,7 @@ static void print_help(void)
 #ifndef HAVE_INOTIFY
 	puts(_(" --no-polling		do not periodically check for configuration updates"));
 #endif
+	puts(_(" --global_defaults_path	print the path for default config and exit"));
 	puts(_(" --version		print version and exit"));
 	puts(_(" --help			show this message"));
 }
@@ -532,8 +485,9 @@ static void check_defaults(void)
 static void inotifyWatchConfig(void)
 {
 	char *watchPath = NULL;
-	inotifyFD = inotify_init();	/* Initialise an inotify instance */
-	if (inotifyFD < 0) {
+
+	w_global.inotify.fd_event_queue = inotify_init();	/* Initialise an inotify instance */
+	if (w_global.inotify.fd_event_queue < 0) {
 		wwarning(_("could not initialise an inotify instance."
 			   " Changes to the defaults database will require"
 			   " a restart to take effect. Check your kernel!"));
@@ -543,12 +497,13 @@ static void inotifyWatchConfig(void)
 		 * but we might want more in the future so check all events for now.
 		 * The individual events are checked for in event.c.
 		 */
-		inotifyWD = inotify_add_watch(inotifyFD, watchPath, IN_ALL_EVENTS);
-		if (inotifyWD < 0) {
+		w_global.inotify.wd_defaults = inotify_add_watch(w_global.inotify.fd_event_queue, watchPath, IN_ALL_EVENTS);
+		if (w_global.inotify.wd_defaults < 0) {
 			wwarning(_("could not add an inotify watch on path %s."
-				   "Changes to the defaults database will require"
+				   " Changes to the defaults database will require"
 				   " a restart to take effect."), watchPath);
-			close(inotifyFD);
+			close(w_global.inotify.fd_event_queue);
+			w_global.inotify.fd_event_queue = -1;
 		}
 	}
 	wfree(watchPath);
@@ -596,6 +551,14 @@ int main(int argc, char **argv)
 	int i_am_the_monitor, i, len;
 	char *str, *alt;
 
+	memset(&w_global, 0, sizeof(w_global));
+	w_global.program.state = WSTATE_NORMAL;
+	w_global.program.signal_state = WSTATE_NORMAL;
+	w_global.timestamp.last_event = CurrentTime;
+	w_global.timestamp.focus_change = CurrentTime;
+	w_global.ignore_workspace_change = False;
+	w_global.shortcut.modifiers_mask = 0xff;
+
 	/* setup common stuff for the monitor and wmaker itself */
 	WMInitializeApplication("WindowMaker", &argc, argv);
 
@@ -691,6 +654,10 @@ static int real_main(int argc, char **argv)
 				wPreferences.flags.noclip = 1;
 			} else if (strcmp(argv[i], "-nodrawer") == 0 || strcmp(argv[i], "--no-drawer") == 0) {
 				wPreferences.flags.nodrawer = 1;
+#ifdef USE_ICCCM_WMREPLACE
+			} else if (strcmp(argv[i], "-replace") == 0 || strcmp(argv[i], "--replace") == 0) {
+				wPreferences.flags.replace = 1;
+#endif
 			} else if (strcmp(argv[i], "-version") == 0 || strcmp(argv[i], "--version") == 0) {
 				printf("Window Maker %s\n", VERSION);
 				exit(0);
@@ -703,7 +670,7 @@ static int real_main(int argc, char **argv)
 					wwarning(_("too few arguments for %s"), argv[i - 1]);
 					exit(0);
 				}
-				Locale = argv[i];
+				w_global.locale = argv[i];
 			} else if (strcmp(argv[i], "-display") == 0 || strcmp(argv[i], "--display") == 0) {
 				i++;
 				if (i >= argc) {
@@ -721,12 +688,14 @@ static int real_main(int argc, char **argv)
 					wwarning(_("bad value for visualid: \"%s\""), argv[i]);
 					exit(0);
 				}
-			} else if (strcmp(argv[i], "-static") == 0 || strcmp(argv[i], "--static") == 0
-#ifndef HAVE_INOTIFY
-				    || strcmp(argv[i], "--no-polling") == 0
-#endif
-				    ) {
+			} else if (strcmp(argv[i], "-static") == 0 || strcmp(argv[i], "--static") == 0) {
 				wPreferences.flags.noupdates = 1;
+			} else if (strcmp(argv[i], "--no-polling") == 0) {
+#ifndef HAVE_INOTIFY
+				wPreferences.flags.noupdates = 1;
+#else
+				wmessage(_("your version of Window Maker was compiled with INotify support, so \"--no-polling\" has no effect"));
+#endif
 			} else if (strcmp(argv[i], "--help") == 0) {
 				print_help();
 				exit(0);
@@ -743,19 +712,19 @@ static int real_main(int argc, char **argv)
 		check_defaults();
 	}
 
-	if (Locale) {
-		setenv("LANG", Locale, 1);
+	if (w_global.locale) {
+		setenv("LANG", w_global.locale, 1);
 	} else {
-		Locale = getenv("LC_ALL");
-		if (!Locale) {
-			Locale = getenv("LANG");
+		w_global.locale = getenv("LC_ALL");
+		if (!w_global.locale) {
+			w_global.locale = getenv("LANG");
 		}
 	}
 
 	setlocale(LC_ALL, "");
 
-	if (!Locale || strcmp(Locale, "C") == 0 || strcmp(Locale, "POSIX") == 0)
-		Locale = NULL;
+	if (!w_global.locale || strcmp(w_global.locale, "C") == 0 || strcmp(w_global.locale, "POSIX") == 0)
+		w_global.locale = NULL;
 #ifdef I18N
 	if (getenv("NLSPATH")) {
 		bindtextdomain("WindowMaker", getenv("NLSPATH"));
@@ -783,11 +752,11 @@ static int real_main(int argc, char **argv)
 	}
 #endif
 
-	if (Locale) {
+	if (w_global.locale) {
 		char *ptr;
 
-		Locale = wstrdup(Locale);
-		ptr = strchr(Locale, '.');
+		w_global.locale = wstrdup(w_global.locale);
+		ptr = strchr(w_global.locale, '.');
 		if (ptr)
 			*ptr = 0;
 	}
@@ -832,7 +801,7 @@ static int real_main(int argc, char **argv)
 	wXModifierInitialize();
 	StartUp(!multiHead);
 
-	if (wScreenCount == 1)
+	if (w_global.screen_count == 1)
 		multiHead = False;
 
 	execInitScript();
