@@ -45,11 +45,6 @@
 #include "rootmenu.h"
 #include "switchmenu.h"
 
-/****** Global Variables ******/
-
-extern Cursor wCursor[WCUR_LAST];
-
-extern XContext wWinContext;
 
 #define MOD_MASK wPreferences.modifier_mask
 
@@ -61,10 +56,6 @@ extern XContext wWinContext;
 
 /***** Local Stuff ******/
 
-#define WSS_ROOTMENU	(1<<0)
-#define WSS_SWITCHMENU	(1<<1)
-#define WSS_WSMENU	(1<<2)
-
 static struct {
 	int steps;
 	int delay;
@@ -321,10 +312,10 @@ void wMenuEntrySetCascade(WMenu * menu, WMenuEntry * entry, WMenu * cascade)
 	if (!done) {
 		entry->cascade = menu->cascade_no;
 
-		menu->cascades = wrealloc(menu->cascades, sizeof(WMenu) * (menu->cascade_no + 1));
+		menu->cascades = wrealloc(menu->cascades, sizeof(menu->cascades[0]) * (menu->cascade_no + 1));
 		menu->cascades[menu->cascade_no++] = cascade;
 
-		brother->cascades = wrealloc(brother->cascades, sizeof(WMenu) * (brother->cascade_no + 1));
+		brother->cascades = wrealloc(brother->cascades, sizeof(brother->cascades[0]) * (brother->cascade_no + 1));
 		brother->cascades[brother->cascade_no++] = cascade->brother;
 	}
 
@@ -1278,7 +1269,7 @@ static WMenu *findMenu(WScreen * scr, int *x_ret, int *y_ret)
 	if (win == None)
 		return NULL;
 
-	if (XFindContext(dpy, win, wWinContext, (XPointer *) & desc) == XCNOENT)
+	if (XFindContext(dpy, win, w_global.context.client_win, (XPointer *) & desc) == XCNOENT)
 		return NULL;
 
 	if (desc->parent_type == WCLASS_MENU) {
@@ -1400,7 +1391,7 @@ WMenu *wMenuUnderPointer(WScreen * screen)
 	if (win == None)
 		return NULL;
 
-	if (XFindContext(dpy, win, wWinContext, (XPointer *) & desc) == XCNOENT)
+	if (XFindContext(dpy, win, w_global.context.client_win, (XPointer *) & desc) == XCNOENT)
 		return NULL;
 
 	if (desc->parent_type == WCLASS_MENU)
@@ -1548,15 +1539,17 @@ typedef struct _delay {
 	int ox, oy;
 } _delay;
 
-static void leaving(_delay * dl)
+static void callback_leaving(void *user_param)
 {
+	_delay *dl = (_delay *) user_param;
+
 	wMenuMove(dl->menu, dl->ox, dl->oy, True);
 	dl->menu->jump_back = NULL;
 	dl->menu->menu->screen_ptr->flags.jump_back_pending = 0;
 	wfree(dl);
 }
 
-void wMenuScroll(WMenu * menu, XEvent * event)
+void wMenuScroll(WMenu *menu)
 {
 	WMenu *smenu;
 	WMenu *omenu = parentMenu(menu);
@@ -1657,6 +1650,7 @@ void wMenuScroll(WMenu * menu, XEvent * event)
 
 	if (jump_back) {
 		_delay *delayer;
+
 		if (!omenu->jump_back) {
 			delayer = wmalloc(sizeof(_delay));
 			delayer->menu = omenu;
@@ -1666,12 +1660,15 @@ void wMenuScroll(WMenu * menu, XEvent * event)
 			scr->flags.jump_back_pending = 1;
 		} else
 			delayer = omenu->jump_back;
-		WMAddTimerHandler(MENU_JUMP_BACK_DELAY, (WMCallback *) leaving, delayer);
+		WMAddTimerHandler(MENU_JUMP_BACK_DELAY, callback_leaving, delayer);
 	}
 }
 
 static void menuExpose(WObjDescriptor * desc, XEvent * event)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) event;
+
 	wMenuPaint(desc->parent);
 }
 
@@ -2096,6 +2093,9 @@ static void menuTitleDoubleClick(WCoreWindow * sender, void *data, XEvent * even
 	WMenu *menu = data;
 	int lower;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) sender;
+
 	if (event->xbutton.state & MOD_MASK) {
 		if (menu->flags.lowered) {
 			lower = 0;
@@ -2116,6 +2116,9 @@ static void menuTitleMouseDown(WCoreWindow * sender, void *data, XEvent * event)
 	int i, lower;
 	Bool started;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) sender;
+
 	/* can't touch the menu copy */
 	if (menu->flags.brother)
 		return;
@@ -2185,7 +2188,7 @@ static void menuTitleMouseDown(WCoreWindow * sender, void *data, XEvent * event)
 						     ButtonMotionMask | ButtonReleaseMask
 						     | ButtonPressMask,
 						     GrabModeAsync, GrabModeAsync, None,
-						     wCursor[WCUR_MOVE], CurrentTime);
+						     wPreferences.cursor[WCUR_MOVE], CurrentTime);
 				}
 			}
 			break;
@@ -2223,6 +2226,10 @@ static void menuCloseClick(WCoreWindow * sender, void *data, XEvent * event)
 	WMenu *parent = menu->parent;
 	int i;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) sender;
+	(void) event;
+
 	if (parent) {
 		for (i = 0; i < parent->cascade_no; i++) {
 			/* find the entry that points to the copy */
@@ -2371,7 +2378,7 @@ static Bool getMenuInfo(WMPropList * info, int *x, int *y, Bool * lowered)
 	return True;
 }
 
-static int restoreMenu(WScreen * scr, WMPropList * menu, int which)
+static int restoreMenu(WScreen *scr, WMPropList *menu)
 {
 	int x, y;
 	Bool lowered = False;
@@ -2383,10 +2390,8 @@ static int restoreMenu(WScreen * scr, WMPropList * menu, int which)
 	if (!getMenuInfo(menu, &x, &y, &lowered))
 		return False;
 
-	if (which & WSS_SWITCHMENU) {
-		OpenSwitchMenu(scr, x, y, False);
-		pmenu = scr->switch_menu;
-	}
+	OpenSwitchMenu(scr, x, y, False);
+	pmenu = scr->switch_menu;
 
 	if (pmenu) {
 		int width = MENUW(pmenu);
@@ -2497,7 +2502,7 @@ void wMenuRestoreState(WScreen * scr)
 	skey = WMCreatePLString("SwitchMenu");
 	menu = WMGetFromPLDictionary(menus, skey);
 	WMReleasePropList(skey);
-	restoreMenu(scr, menu, WSS_SWITCHMENU);
+	restoreMenu(scr, menu);
 
 	if (!scr->root_menu) {
 		OpenRootMenu(scr, scr->scr_width * 2, 0, False);
