@@ -28,27 +28,37 @@
 #include <string.h>
 #include <strings.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <stdarg.h>
 #include <pwd.h>
 #include <math.h>
 #include <time.h>
+#include <errno.h>
+
+#include <X11/XKBlib.h>
 
 #include <WINGs/WUtil.h>
 #include <wraster.h>
 
+#include "window.h"
+#include "misc.h"
 #include "WindowMaker.h"
 #include "GNUstep.h"
 #include "screen.h"
 #include "wcore.h"
 #include "window.h"
 #include "framewin.h"
-#include "funcs.h"
 #include "dialog.h"
 #include "xutil.h"
 #include "xmodifier.h"
+#include "main.h"
+#include "event.h"
 
-/**** global variables *****/
-extern WPreferences wPreferences;
+
+#define ICON_SIZE wPreferences.icon_size
+
+/**** Local prototypes *****/
+static void UnescapeWM_CLASS(const char *str, char **name, char **class);
 
 /* XFetchName Wrapper */
 Bool wFetchName(Display *dpy, Window win, char **winname)
@@ -131,25 +141,34 @@ static void eatExpose(void)
 
 void move_window(Window win, int from_x, int from_y, int to_x, int to_y)
 {
-#ifdef ANIMATIONS
+#ifdef USE_ANIMATIONS
 	if (wPreferences.no_animations)
 		XMoveWindow(dpy, win, to_x, to_y);
 	else
-		SlideWindow(win, from_x, from_y, to_x, to_y);
+		slide_window(win, from_x, from_y, to_x, to_y);
 #else
 	XMoveWindow(dpy, win, to_x, to_y);
+
+	/* Tell the compiler it is normal that those parameters are not used in this case */
+	(void) from_x;
+	(void) from_y;
 #endif
 }
 
-void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
+/* wins is an array of Window, sorted from left to right, the first is
+ * going to be moved from (from_x,from_y) to (to_x,to_y) and the
+ * following windows are going to be offset by (ICON_SIZE*i,0) */
+void slide_windows(Window wins[], int n, int from_x, int from_y, int to_x, int to_y)
 {
 	time_t time0 = time(NULL);
-	float dx, dy, x = from_x, y = from_y, sx, sy, px, py;
-	int dx_is_bigger = 0;
+	float dx, dy, x = from_x, y = from_y, px, py;
+	Bool is_dx_nul, is_dy_nul;
+	int dx_is_bigger = 0, dx_int, dy_int;
 	int slide_delay, slide_steps, slide_slowdown;
+	int i;
 
 	/* animation parameters */
-	static struct {
+	static const struct {
 		int delay;
 		int steps;
 		int slowdown;
@@ -165,12 +184,14 @@ void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 	slide_steps = apars[(int)wPreferences.icon_slide_speed].steps;
 	slide_delay = apars[(int)wPreferences.icon_slide_speed].delay;
 
-	dx = (float)(to_x - from_x);
-	dy = (float)(to_y - from_y);
-	sx = (dx == 0 ? 0 : fabs(dx) / dx);
-	sy = (dy == 0 ? 0 : fabs(dy) / dy);
+	dx_int = to_x - from_x;
+	dy_int = to_y - from_y;
+	is_dx_nul = (dx_int == 0);
+	is_dy_nul = (dy_int == 0);
+	dx = (float) dx_int;
+	dy = (float) dy_int;
 
-	if (fabs(dx) > fabs(dy)) {
+	if (abs(dx_int) > abs(dy_int)) {
 		dx_is_bigger = 1;
 	}
 
@@ -180,17 +201,18 @@ void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 			px = slide_steps;
 		else if (px > -slide_steps && px < 0)
 			px = -slide_steps;
-		py = (sx == 0 ? 0 : px * dy / dx);
+		py = (is_dx_nul ? 0.0F : px * dy / dx);
 	} else {
 		py = dy / slide_slowdown;
 		if (py < slide_steps && py > 0)
 			py = slide_steps;
 		else if (py > -slide_steps && py < 0)
 			py = -slide_steps;
-		px = (sy == 0 ? 0 : py * dx / dy);
+		px = (is_dy_nul ? 0.0F : py * dx / dy);
 	}
 
-	while (x != to_x || y != to_y) {
+	while (((int)x) != to_x ||
+			 ((int)y) != to_y) {
 		x += px;
 		y += py;
 		if ((px < 0 && (int)x < to_x) || (px > 0 && (int)x > to_x))
@@ -199,39 +221,43 @@ void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 			y = (float)to_y;
 
 		if (dx_is_bigger) {
-			px = px * (1.0 - 1 / (float)slide_slowdown);
+			px = px * (1.0F - 1 / (float)slide_slowdown);
 			if (px < slide_steps && px > 0)
 				px = slide_steps;
 			else if (px > -slide_steps && px < 0)
 				px = -slide_steps;
-			py = (sx == 0 ? 0 : px * dy / dx);
+			py = (is_dx_nul ? 0.0F : px * dy / dx);
 		} else {
-			py = py * (1.0 - 1 / (float)slide_slowdown);
+			py = py * (1.0F - 1 / (float)slide_slowdown);
 			if (py < slide_steps && py > 0)
 				py = slide_steps;
 			else if (py > -slide_steps && py < 0)
 				py = -slide_steps;
-			px = (sy == 0 ? 0 : py * dx / dy);
+			px = (is_dy_nul ? 0.0F : py * dx / dy);
 		}
 
-		XMoveWindow(dpy, win, (int)x, (int)y);
+		for (i = 0; i < n; i++) {
+			XMoveWindow(dpy, wins[i], (int)x + i * ICON_SIZE, (int)y);
+		}
 		XFlush(dpy);
 		if (slide_delay > 0) {
 			wusleep(slide_delay * 1000L);
 		} else {
-			wusleep(10);
+			wusleep(1000L);
 		}
 		if (time(NULL) - time0 > MAX_ANIMATION_TIME)
 			break;
 	}
-	XMoveWindow(dpy, win, to_x, to_y);
+	for (i = 0; i < n; i++) {
+		XMoveWindow(dpy, wins[i], to_x + i * ICON_SIZE, to_y);
+	}
 
 	XSync(dpy, 0);
 	/* compress expose events */
 	eatExpose();
 }
 
-char *ShrinkString(WMFont * font, char *string, int width)
+char *ShrinkString(WMFont *font, const char *string, int width)
 {
 	int w, w1 = 0;
 	int p;
@@ -255,7 +281,6 @@ char *ShrinkString(WMFont * font, char *string, int width)
 		p = strlen(text);
 		w1 = WMWidthOfString(font, text, p);
 		if (w1 > width) {
-			w1 = 0;
 			p = 0;
 			*pos = ' ';
 			*text = 0;
@@ -271,7 +296,7 @@ char *ShrinkString(WMFont * font, char *string, int width)
 	}
 	strcat(text, "...");
 	width -= WMWidthOfString(font, "...", 3);
-	pos = string;
+
 	p1 = 0;
 	p2 = p;
 	t = (p2 - p1) / 2;
@@ -291,7 +316,7 @@ char *ShrinkString(WMFont * font, char *string, int width)
 	return text;
 }
 
-char *FindImage(char *paths, char *file)
+char *FindImage(const char *paths, const char *file)
 {
 	char *tmp, *path = NULL;
 
@@ -410,7 +435,7 @@ static char *getselection(WScreen * scr)
 }
 
 static char*
-parseuserinputpart(char *line, int *ptr, char *endchars)
+parseuserinputpart(const char *line, int *ptr, const char *endchars)
 {
 	int depth = 0, begin;
 	char *value = NULL;
@@ -434,7 +459,7 @@ parseuserinputpart(char *line, int *ptr, char *endchars)
 }
 
 static char*
-getuserinput(WScreen *scr, char *line, int *ptr, Bool advanced)
+getuserinput(WScreen *scr, const char *line, int *ptr, Bool advanced)
 {
     char *ret = NULL, *title = NULL, *prompt = NULL, *name = NULL;
     int rv;
@@ -482,15 +507,12 @@ getuserinput(WScreen *scr, char *line, int *ptr, Bool advanced)
  * OPTION	etc.	NORMAL		%<input>
  */
 #define TMPBUFSIZE 64
-char *ExpandOptions(WScreen * scr, char *cmdline)
+char *ExpandOptions(WScreen *scr, const char *cmdline)
 {
 	int ptr, optr, state, len, olen;
 	char *out, *nout;
 	char *selection = NULL;
 	char *user_input = NULL;
-#ifdef XDND
-	char *dropped_thing = NULL;
-#endif
 	char tmpbuf[TMPBUFSIZE];
 	int slen;
 
@@ -551,7 +573,7 @@ char *ExpandOptions(WScreen * scr, char *cmdline)
 					olen += slen;
 					nout = realloc(out, olen);
 					if (!nout) {
-						wwarning(_("out of memory during expansion of \"%%w\""));
+						wwarning(_("out of memory during expansion of '%s' for command \"%s\""), "%w", cmdline);
 						goto error;
 					}
 					out = nout;
@@ -568,7 +590,7 @@ char *ExpandOptions(WScreen * scr, char *cmdline)
 				olen += slen;
 				nout = realloc(out, olen);
 				if (!nout) {
-					wwarning(_("out of memory during expansion of \"%%W\""));
+					wwarning(_("out of memory during expansion of '%s' for command \"%s\""), "%W", cmdline);
 					goto error;
 				}
 				out = nout;
@@ -585,7 +607,7 @@ char *ExpandOptions(WScreen * scr, char *cmdline)
 					olen += slen;
 					nout = realloc(out, olen);
 					if (!nout) {
-						wwarning(_("out of memory during expansion of \"%%a\""));
+						wwarning(_("out of memory during expansion of '%s' for command \"%s\""), "%a", cmdline);
 						goto error;
 					}
 					out = nout;
@@ -598,30 +620,24 @@ char *ExpandOptions(WScreen * scr, char *cmdline)
 				}
 				break;
 
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 			case 'd':
-				if (scr->xdestring) {
-					dropped_thing = wstrdup(scr->xdestring);
-				}
-				if (!dropped_thing) {
-					dropped_thing = get_dnd_selection(scr);
-				}
-				if (!dropped_thing) {
+				if (!scr->xdestring) {
 					scr->flags.dnd_data_convertion_status = 1;
 					goto error;
 				}
-				slen = strlen(dropped_thing);
+				slen = strlen(scr->xdestring);
 				olen += slen;
 				nout = realloc(out, olen);
 				if (!nout) {
-					wwarning(_("out of memory during expansion of \"%%d\""));
+					wwarning(_("out of memory during expansion of '%s' for command \"%s\""), "%d", cmdline);
 					goto error;
 				}
 				out = nout;
-				strcat(out, dropped_thing);
+				strcat(out, scr->xdestring);
 				optr += slen;
 				break;
-#endif				/* XDND */
+#endif	/* USE_DOCK_XDND */
 
 			case 's':
 				if (!selection) {
@@ -635,7 +651,7 @@ char *ExpandOptions(WScreen * scr, char *cmdline)
 				olen += slen;
 				nout = realloc(out, olen);
 				if (!nout) {
-					wwarning(_("out of memory during expansion of \"%%s\""));
+					wwarning(_("out of memory during expansion of '%s' for command \"%s\""), "%s", cmdline);
 					goto error;
 				}
 				out = nout;
@@ -663,7 +679,7 @@ char *ExpandOptions(WScreen * scr, char *cmdline)
 	return NULL;
 }
 
-void ParseWindowName(WMPropList * value, char **winstance, char **wclass, char *where)
+void ParseWindowName(WMPropList *value, char **winstance, char **wclass, const char *where)
 {
 	char *name;
 
@@ -708,70 +724,82 @@ static char *keysymToString(KeySym keysym, unsigned int state)
 }
 #endif
 
-char *GetShortcutString(char *text)
+char *GetShortcutString(const char *shortcut)
 {
 	char *buffer = NULL;
-	char *k;
-	int modmask = 0;
-	/*    KeySym ksym; */
-	int control = 0;
-	char *tmp;
+	char *k, *tmp, *text;
 
-	tmp = text = wstrdup(text);
+	tmp = text = wstrdup(shortcut);
 
 	/* get modifiers */
 	while ((k = strchr(text, '+')) != NULL) {
 		int mod;
+		const char *lbl;
 
 		*k = 0;
 		mod = wXModifierFromKey(text);
 		if (mod < 0) {
 			return wstrdup("bug");
 		}
-
-		modmask |= mod;
-
-		if (strcasecmp(text, "Meta") == 0) {
-			buffer = wstrappend(buffer, "M+");
-		} else if (strcasecmp(text, "Alt") == 0) {
-			buffer = wstrappend(buffer, "A+");
-		} else if (strcasecmp(text, "Shift") == 0) {
-			buffer = wstrappend(buffer, "Sh+");
-		} else if (strcasecmp(text, "Mod1") == 0) {
-			buffer = wstrappend(buffer, "M1+");
-		} else if (strcasecmp(text, "Mod2") == 0) {
-			buffer = wstrappend(buffer, "M2+");
-		} else if (strcasecmp(text, "Mod3") == 0) {
-			buffer = wstrappend(buffer, "M3+");
-		} else if (strcasecmp(text, "Mod4") == 0) {
-			buffer = wstrappend(buffer, "M4+");
-		} else if (strcasecmp(text, "Mod5") == 0) {
-			buffer = wstrappend(buffer, "M5+");
-		} else if (strcasecmp(text, "Control") == 0) {
-			control = 1;
-		} else {
+		lbl = wXModifierToShortcutLabel(mod);
+		if (lbl)
+			buffer = wstrappend(buffer, lbl);
+		else
 			buffer = wstrappend(buffer, text);
-		}
 		text = k + 1;
 	}
 
-	if (control) {
-		buffer = wstrappend(buffer, "^");
-	}
 	buffer = wstrappend(buffer, text);
-
-	/* get key */
-	/*    ksym = XStringToKeysym(text);
-	   tmp = keysymToString(ksym, modmask);
-	   puts(tmp);
-	   buffer = wstrappend(buffer, tmp);
-	 */
 	wfree(tmp);
 
 	return buffer;
 }
 
-char *EscapeWM_CLASS(char *name, char *class)
+char *GetShortcutKey(WShortKey key)
+{
+	const char *key_name;
+	char buffer[256];
+	char *wr;
+
+	void append_string(const char *text)
+	{
+		const char *string = text;
+
+		while (*string) {
+			if (wr >= buffer + sizeof(buffer) - 1)
+				break;
+			*wr++ = *string++;
+		}
+	}
+
+	void append_modifier(int modifier_index, const char *fallback_name)
+	{
+		if (wPreferences.modifier_labels[modifier_index]) {
+			append_string(wPreferences.modifier_labels[modifier_index]);
+		} else {
+			append_string(fallback_name);
+		}
+	}
+
+	key_name = XKeysymToString(XkbKeycodeToKeysym(dpy, key.keycode, 0, 0));
+	if (!key_name)
+		return NULL;
+
+	wr = buffer;
+	if (key.modifier & ControlMask) append_modifier(1, "Control+");
+	if (key.modifier & ShiftMask)   append_modifier(0, "Shift+");
+	if (key.modifier & Mod1Mask)    append_modifier(2, "Mod1+");
+	if (key.modifier & Mod2Mask)    append_modifier(3, "Mod2+");
+	if (key.modifier & Mod3Mask)    append_modifier(4, "Mod3+");
+	if (key.modifier & Mod4Mask)    append_modifier(5, "Mod4+");
+	if (key.modifier & Mod5Mask)    append_modifier(6, "Mod5+");
+	append_string(key_name);
+	*wr = '\0';
+
+	return GetShortcutString(buffer);
+}
+
+char *EscapeWM_CLASS(const char *name, const char *class)
 {
 	char *ret;
 	char *ename = NULL, *eclass = NULL;
@@ -826,19 +854,17 @@ char *EscapeWM_CLASS(char *name, char *class)
 	return ret;
 }
 
-void UnescapeWM_CLASS(char *str, char **name, char **class)
+static void UnescapeWM_CLASS(const char *str, char **name, char **class)
 {
 	int i, j, k, dot;
+	int length_of_name;
 
 	j = strlen(str);
-	*name = wmalloc(j);
-	**name = 0;
-	*class = wmalloc(j);
-	**class = 0;
 
 	/* separate string in 2 parts */
+	length_of_name = 0;
 	dot = -1;
-	for (i = 0; i < j; i++) {
+	for (i = 0; i < j; i++, length_of_name++) {
 		if (str[i] == '\\') {
 			i++;
 			continue;
@@ -848,36 +874,108 @@ void UnescapeWM_CLASS(char *str, char **name, char **class)
 		}
 	}
 
-	/* unescape strings */
-	for (i = 0, k = 0; i < dot; i++) {
-		if (str[i] == '\\') {
-			continue;
-		} else {
-			(*name)[k++] = str[i];
+	/* unescape the name */
+	if (length_of_name > 0) {
+		*name = wmalloc(length_of_name + 1);
+		for (i = 0, k = 0; i < dot; i++) {
+			if (str[i] != '\\')
+				(*name)[k++] = str[i];
 		}
-	}
-	(*name)[k] = 0;
-
-	for (i = dot + 1, k = 0; i < j; i++) {
-		if (str[i] == '\\') {
-			continue;
-		} else {
-			(*class)[k++] = str[i];
-		}
-	}
-	(*class)[k] = 0;
-
-	if (!*name) {
-		wfree(*name);
+		(*name)[k] = '\0';
+	} else {
 		*name = NULL;
 	}
-	if (!*class) {
-		wfree(*class);
+
+	/* unescape the class */
+	if (dot < j-1) {
+		*class = wmalloc(j - (dot + 1) + 1);
+		for (i = dot + 1, k = 0; i < j; i++) {
+			if (str[i] != '\\')
+				(*class)[k++] = str[i];
+		}
+		(*class)[k] = 0;
+	} else {
 		*class = NULL;
 	}
 }
 
-void SendHelperMessage(WScreen * scr, char type, int workspace, char *msg)
+static void track_bg_helper_death(pid_t pid, unsigned int status, void *client_data)
+{
+	WScreen *scr = (WScreen *) client_data;
+
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) pid;
+	(void) status;
+
+	close(scr->helper_fd);
+	scr->helper_fd = 0;
+	scr->helper_pid = 0;
+	scr->flags.backimage_helper_launched = 0;
+}
+
+Bool start_bg_helper(WScreen *scr)
+{
+	pid_t pid;
+	int filedes[2];
+
+	if (pipe(filedes) < 0) {
+		werror(_("%s failed, can't set workspace specific background image (%s)"),
+		       "pipe()", strerror(errno));
+		return False;
+	}
+
+	pid = fork();
+	if (pid < 0) {
+		werror(_("%s failed, can't set workspace specific background image (%s)"),
+		       "fork()", strerror(errno));
+		close(filedes[0]);
+		close(filedes[1]);
+		return False;
+
+	} else if (pid == 0) {
+		const char *dither;
+
+		/* We don't need this side of the pipe in the child process */
+		close(filedes[1]);
+
+		SetupEnvironment(scr);
+
+		close(STDIN_FILENO);
+		if (dup2(filedes[0], STDIN_FILENO) < 0) {
+			werror(_("%s failed, can't set workspace specific background image (%s)"),
+			       "dup2()", strerror(errno));
+			exit(1);
+		}
+		close(filedes[0]);
+
+		dither = wPreferences.no_dithering ? "-m" : "-d";
+		if (wPreferences.smooth_workspace_back)
+			execlp("wmsetbg", "wmsetbg", "-helper", "-S", dither, NULL);
+		else
+			execlp("wmsetbg", "wmsetbg", "-helper", dither, NULL);
+
+		werror(_("could not execute \"%s\": %s"), "wmsetbg", strerror(errno));
+		exit(1);
+
+	} else {
+		/* We don't need this side of the pipe in the parent process */
+		close(filedes[0]);
+
+		if (fcntl(filedes[1], F_SETFD, FD_CLOEXEC) < 0)
+			wwarning(_("could not set close-on-exec flag for bg_helper's communication file handle (%s)"),
+			         strerror(errno));
+
+		scr->helper_fd = filedes[1];
+		scr->helper_pid = pid;
+		scr->flags.backimage_helper_launched = 1;
+
+		wAddDeathHandler(pid, track_bg_helper_death, scr);
+
+		return True;
+	}
+}
+
+void SendHelperMessage(WScreen *scr, char type, int workspace, const char *msg)
 {
 	char *buffer;
 	int len;
@@ -942,7 +1040,7 @@ Bool UpdateDomainFile(WDDomain * domain)
 	return result;
 }
 
-char *StrConcatDot(char *a, char *b)
+char *StrConcatDot(const char *a, const char *b)
 {
 	int len;
 	char *str;
