@@ -50,18 +50,14 @@
 
 #define LEFT            1
 #define RIGHT           2
-#define HORIZONTAL      (LEFT|RIGHT)
 #define UP              4
 #define DOWN            8
-#define VERTICAL        (UP|DOWN)
 
 /* True if window currently has a border. This also includes borderless
  * windows which are currently selected
  */
 #define HAS_BORDER_WITH_SELECT(w) ((w)->flags.selected || HAS_BORDER(w))
 
-/****** Global Variables ******/
-extern Cursor wCursor[WCUR_LAST];
 
 /*
  *----------------------------------------------------------------------
@@ -564,7 +560,7 @@ static void crossWorkspace(WScreen * scr, WWindow * wwin, int opaque_move, int n
 	if (!opaque_move) {
 		XGrabPointer(dpy, scr->root_win, True, PointerMotionMask
 			     | ButtonReleaseMask | ButtonPressMask, GrabModeAsync,
-			     GrabModeAsync, None, wCursor[WCUR_MOVE], CurrentTime);
+			     GrabModeAsync, None, wPreferences.cursor[WCUR_MOVE], CurrentTime);
 	}
 }
 
@@ -590,6 +586,18 @@ typedef struct {
 	int calcX, calcY;	/* calculated position of window */
 	int omouseX, omouseY;	/* old mouse position */
 	int mouseX, mouseY;	/* last known position of the pointer */
+
+	enum {
+		SNAP_NONE,
+		SNAP_LEFT,
+		SNAP_RIGHT,
+		SNAP_TOP,
+		SNAP_BOTTOM,
+		SNAP_TOPLEFT,
+		SNAP_TOPRIGHT,
+		SNAP_BOTTOMLEFT,
+		SNAP_BOTTOMRIGHT
+	} snap;
 } MoveData;
 
 #define WTOP(w) (w)->frame_y
@@ -651,7 +659,7 @@ static int compareWBottom(const void *a, const void *b)
 		return 0;
 }
 
-static void updateResistance(WWindow * wwin, MoveData * data, int newX, int newY)
+static void updateResistance(MoveData *data, int newX, int newY)
 {
 	int i;
 	int newX2 = newX + data->winWidth;
@@ -766,10 +774,10 @@ static void updateMoveData(WWindow * wwin, MoveData * data)
 
 	/* order from closest to the border of the screen to farthest */
 
-	qsort(data->topList, data->count, sizeof(WWindow **), compareWTop);
-	qsort(data->leftList, data->count, sizeof(WWindow **), compareWLeft);
-	qsort(data->rightList, data->count, sizeof(WWindow **), compareWRight);
-	qsort(data->bottomList, data->count, sizeof(WWindow **), compareWBottom);
+	qsort(data->topList, data->count, sizeof(data->topList[0]), compareWTop);
+	qsort(data->leftList, data->count, sizeof(data->leftList[0]), compareWLeft);
+	qsort(data->rightList, data->count, sizeof(data->rightList[0]), compareWRight);
+	qsort(data->bottomList, data->count, sizeof(data->bottomList[0]), compareWBottom);
 
 	/* figure the position of the window relative to the others */
 
@@ -831,6 +839,8 @@ static void initMoveData(WWindow * wwin, MoveData * data)
 
 	data->winWidth = wwin->frame->core->width + (HAS_BORDER_WITH_SELECT(wwin) ? 2 * wwin->screen_ptr->frame_border_width : 0);
 	data->winHeight = wwin->frame->core->height + (HAS_BORDER_WITH_SELECT(wwin) ? 2 * wwin->screen_ptr->frame_border_width : 0);
+
+	data->snap = SNAP_NONE;
 }
 
 static Bool checkWorkspaceChange(WWindow * wwin, MoveData * data, Bool opaqueMove)
@@ -840,23 +850,17 @@ static Bool checkWorkspaceChange(WWindow * wwin, MoveData * data, Bool opaqueMov
 
 	if (data->mouseX <= 1) {
 		if (scr->current_workspace > 0) {
-
 			crossWorkspace(scr, wwin, opaqueMove, scr->current_workspace - 1, True);
 			changed = True;
 			data->rubCount = 0;
-
 		} else if (scr->current_workspace == 0 && wPreferences.ws_cycle) {
-
 			crossWorkspace(scr, wwin, opaqueMove, scr->workspace_count - 1, True);
 			changed = True;
 			data->rubCount = 0;
 		}
 	} else if (data->mouseX >= scr->scr_width - 2) {
-
 		if (scr->current_workspace == scr->workspace_count - 1) {
-
 			if (wPreferences.ws_cycle || scr->workspace_count == MAX_WORKSPACES) {
-
 				crossWorkspace(scr, wwin, opaqueMove, 0, False);
 				changed = True;
 				data->rubCount = 0;
@@ -885,7 +889,6 @@ static Bool checkWorkspaceChange(WWindow * wwin, MoveData * data, Bool opaqueMov
 				data->rubCount = 0;
 			}
 		} else if (scr->current_workspace < scr->workspace_count) {
-
 			/* go to next workspace */
 			crossWorkspace(scr, wwin, opaqueMove, scr->current_workspace + 1, False);
 			changed = True;
@@ -1186,13 +1189,130 @@ updateWindowPosition(WWindow * wwin, MoveData * data, Bool doResistance,
 
 	/* recalc relative window position */
 	if (doResistance && (data->realX != newX || data->realY != newY)) {
-		updateResistance(wwin, data, newX, newY);
+		updateResistance(data, newX, newY);
 	}
 
 	data->realX = newX;
 	data->realY = newY;
 }
 
+static void draw_snap_frame(WWindow *wwin, int direction)
+{
+	WScreen *scr;
+
+	scr = wwin->screen_ptr;
+
+	switch (direction) {
+	case SNAP_LEFT:
+		drawTransparentFrame(wwin, 0, 0, scr->scr_width/2, scr->scr_height);
+		break;
+
+	case SNAP_RIGHT:
+		drawTransparentFrame(wwin, scr->scr_width/2, 0, scr->scr_width/2, scr->scr_height);
+		break;
+
+	case SNAP_TOP:
+		drawTransparentFrame(wwin, 0, 0, scr->scr_width, scr->scr_height/2);
+		break;
+
+	case SNAP_BOTTOM:
+		drawTransparentFrame(wwin, 0, scr->scr_height/2, scr->scr_width, scr->scr_height/2);
+		break;
+
+	case SNAP_TOPLEFT:
+		drawTransparentFrame(wwin, 0, 0, scr->scr_width/2, scr->scr_height/2);
+		break;
+
+	case SNAP_TOPRIGHT:
+		drawTransparentFrame(wwin, scr->scr_width/2, 0, scr->scr_width/2, scr->scr_height/2);
+		break;
+
+	case SNAP_BOTTOMLEFT:
+		drawTransparentFrame(wwin, 0, scr->scr_height/2, scr->scr_width/2, scr->scr_height/2);
+		break;
+
+	case SNAP_BOTTOMRIGHT:
+		drawTransparentFrame(wwin, scr->scr_width/2, scr->scr_height/2,
+				     scr->scr_width/2, scr->scr_height/2);
+		break;
+	}
+}
+
+static int get_snap_direction(WScreen *scr, int x, int y)
+{
+	int edge, corner;
+
+	edge = wPreferences.snap_edge_detect;
+	corner = wPreferences.snap_corner_detect;
+
+	if (x < corner && y < corner)
+		return SNAP_TOPLEFT;
+	if (x < corner && y >= scr->scr_height - corner)
+		return SNAP_BOTTOMLEFT;
+	if (x < edge)
+		return SNAP_LEFT;
+
+	if (x >= scr->scr_width - corner && y < corner)
+		return SNAP_TOPRIGHT;
+	if (x >= scr->scr_width - corner && y >= scr->scr_height - corner)
+		return SNAP_BOTTOMRIGHT;
+	if (x >= scr->scr_width - edge)
+		return SNAP_RIGHT;
+
+	if (y < edge)
+		return SNAP_TOP;
+	if (y >= scr->scr_height - edge)
+		return SNAP_BOTTOM;
+	return SNAP_NONE;
+}
+
+static void do_snap(WWindow *wwin, MoveData *data, Bool opaqueMove)
+{
+	int directions;
+	WScreen *scr;
+
+	directions = 0;
+	scr = wwin->screen_ptr;
+
+	/* erase frames */
+	if (!opaqueMove)
+		drawFrames(wwin, scr->selected_windows, data->realX - wwin->frame_x, data->realY - wwin->frame_y);
+	draw_snap_frame(wwin, data->snap);
+
+	switch (data->snap) {
+	case SNAP_NONE:
+		return;
+	case SNAP_LEFT:
+		directions = MAX_VERTICAL | MAX_LEFTHALF;
+		break;
+	case SNAP_RIGHT:
+		directions = MAX_VERTICAL | MAX_RIGHTHALF;
+		break;
+	case SNAP_TOP:
+		directions = MAX_HORIZONTAL | MAX_TOPHALF;
+		break;
+	case SNAP_BOTTOM:
+		directions = MAX_HORIZONTAL | MAX_BOTTOMHALF;
+		break;
+	case SNAP_TOPLEFT:
+		directions = MAX_TOPHALF | MAX_LEFTHALF;
+		break;
+	case SNAP_TOPRIGHT:
+		directions = MAX_TOPHALF | MAX_RIGHTHALF;
+		break;
+	case SNAP_BOTTOMLEFT:
+		directions = MAX_BOTTOMHALF | MAX_LEFTHALF;
+		break;
+	case SNAP_BOTTOMRIGHT:
+		directions = MAX_BOTTOMHALF | MAX_RIGHTHALF;
+		break;
+	}
+
+	if (directions)
+		handleMaximize(wwin, directions);
+	data->snap = SNAP_NONE;
+}
+
 #define _KS KEY_CONTROL_WINDOW_WEIGHT
 
 #define MOVABLE_BIT	0x01
@@ -1242,7 +1362,7 @@ int wKeyboardMoveResizeWindow(WWindow * wwin)
 	XGrabServer(dpy);
 	XGrabPointer(dpy, scr->root_win, True, PointerMotionMask
 		     | ButtonReleaseMask | ButtonPressMask, GrabModeAsync,
-		     GrabModeAsync, None, wCursor[WCUR_DEFAULT], CurrentTime);
+		     GrabModeAsync, None, wPreferences.cursor[WCUR_NORMAL], CurrentTime);
 
 
 
@@ -1325,6 +1445,9 @@ int wKeyboardMoveResizeWindow(WWindow * wwin)
 				keysym = XLookupKeysym(&event.xkey, 0);
 				switch (keysym) {
 				case XK_Return:
+#ifdef XK_KP_Enter
+				case XK_KP_Enter:
+#endif
 					done = 2;
 					break;
 				case XK_Escape:
@@ -1396,16 +1519,18 @@ int wKeyboardMoveResizeWindow(WWindow * wwin)
 
 				if (wPreferences.ws_cycle) {
 					if (src_x + off_x + ww < 20) {
-						if (!scr->current_workspace) {
+						if (!scr->current_workspace)
 							wWorkspaceChange(scr, scr->workspace_count - 1);
-						} else
+						else
 							wWorkspaceChange(scr, scr->current_workspace - 1);
+
 						off_x += scr_width;
 					} else if (src_x + off_x + 20 > scr_width) {
-						if (scr->current_workspace == scr->workspace_count - 1) {
+						if (scr->current_workspace == scr->workspace_count - 1)
 							wWorkspaceChange(scr, 0);
-						} else
+						else
 							wWorkspaceChange(scr, scr->current_workspace + 1);
+
 						off_x -= scr_width;
 					}
 				} else {
@@ -1476,11 +1601,7 @@ int wKeyboardMoveResizeWindow(WWindow * wwin)
 				};
 
 		if (done) {
-			scr->keymove_tick = 0;
-			/*
-			   WMDeleteTimerWithClientData(&looper);
-			 */
-			if (!opaqueMoveResize) {/*ctrlmode=> resize    */
+			if (!opaqueMoveResize) {	/* ctrlmode => resize */
 				if (wwin->flags.shaded || scr->selected_windows) {
 					if (scr->selected_windows)
 						drawFrames(wwin, scr->selected_windows, off_x, off_y);
@@ -1489,7 +1610,7 @@ int wKeyboardMoveResizeWindow(WWindow * wwin)
 				} else {
 					drawTransparentFrame(wwin, src_x + off_x, src_y + off_y, ww, wh);
 				}
-			};
+			}
 
 			if (ctrlmode) {
 				showGeometry(wwin, src_x + off_x, src_y + off_y, src_x + off_x + ww,
@@ -1653,12 +1774,41 @@ int wMouseMoveWindow(WWindow * wwin, XEvent * ev)
 			break;
 
 		case MotionNotify:
+			if (IS_RESIZABLE(wwin) && wPreferences.window_snapping) {
+				int snap_direction;
+
+				snap_direction = get_snap_direction(scr, moveData.mouseX, moveData.mouseY);
+
+				if (!wPreferences.no_autowrap &&
+				    snap_direction != SNAP_TOP &&
+				    snap_direction != SNAP_BOTTOM)
+					snap_direction = SNAP_NONE;
+
+				if (moveData.snap != snap_direction) {
+					/* erase old frame */
+					if (moveData.snap)
+						draw_snap_frame(wwin, moveData.snap);
+					/* draw new frame */
+					if (snap_direction)
+						draw_snap_frame(wwin, snap_direction);
+					moveData.snap = snap_direction;
+				}
+			}
+
 			if (started) {
+				/* erase snap frame */
+				if (moveData.snap)
+					draw_snap_frame(wwin, moveData.snap);
+
 				updateWindowPosition(wwin, &moveData,
 						     scr->selected_windows == NULL
 						     && wPreferences.edge_resistance > 0,
 						     opaqueMove, event.xmotion.x_root, event.xmotion.y_root);
 
+				/* redraw snap frame */
+				if (moveData.snap)
+					draw_snap_frame(wwin, moveData.snap);
+
 				if (!warped && !wPreferences.no_autowrap) {
 					int oldWorkspace = scr->current_workspace;
 
@@ -1694,9 +1844,29 @@ int wMouseMoveWindow(WWindow * wwin, XEvent * ev)
 			} else if (abs(ev->xmotion.x_root - event.xmotion.x_root) >= MOVE_THRESHOLD
 				   || abs(ev->xmotion.y_root - event.xmotion.y_root) >= MOVE_THRESHOLD) {
 
+				if (wwin->flags.maximized) {
+					if (wPreferences.drag_maximized_window == DRAGMAX_RESTORE) {
+						float titlebar_ratio;
+						int new_x, new_y;
+
+						titlebar_ratio = (moveData.mouseX - wwin->frame_x) /
+							(float)wwin->frame->core->width;
+						new_y = wwin->frame_y;
+						wUnmaximizeWindow(wwin);
+						new_x = moveData.mouseX - titlebar_ratio * wwin->frame->core->width;
+						wWindowMove(wwin, new_x, new_y);
+						moveData.realX = moveData.calcX = wwin->frame_x;
+						moveData.realY = moveData.calcY = wwin->frame_y;
+					}
+					if (wPreferences.drag_maximized_window == DRAGMAX_UNMAXIMIZE) {
+						wwin->flags.maximized = 0;
+						wwin->flags.old_maximized = 0;
+					}
+				}
+
 				XChangeActivePointerGrab(dpy, ButtonMotionMask
 							 | ButtonReleaseMask | ButtonPressMask,
-							 wCursor[WCUR_MOVE], CurrentTime);
+							 wPreferences.cursor[WCUR_MOVE], CurrentTime);
 				started = 1;
 				XGrabKeyboard(dpy, root, False, GrabModeAsync, GrabModeAsync, CurrentTime);
 
@@ -1714,6 +1884,7 @@ int wMouseMoveWindow(WWindow * wwin, XEvent * ev)
 						showPosition(wwin, moveData.realX, moveData.realY);
 				}
 			}
+
 			break;
 
 		case ButtonPress:
@@ -1725,7 +1896,10 @@ int wMouseMoveWindow(WWindow * wwin, XEvent * ev)
 
 			if (started) {
 				XEvent e;
-				if (!opaqueMove) {
+
+				if (moveData.snap)
+					do_snap(wwin, &moveData, opaqueMove);
+				else if (!opaqueMove) {
 					drawFrames(wwin, scr->selected_windows,
 						   moveData.realX - wwin->frame_x, moveData.realY - wwin->frame_y);
 					XSync(dpy, 0);
@@ -1809,7 +1983,7 @@ int wMouseMoveWindow(WWindow * wwin, XEvent * ev)
 #define RESIZEBAR	1
 #define HCONSTRAIN	2
 
-static int getResizeDirection(WWindow * wwin, int x, int y, int dx, int dy, int flags)
+static int getResizeDirection(WWindow * wwin, int x, int y, int dy, int flags)
 {
 	int w = wwin->frame->core->width - 1;
 	int cw = wwin->frame->resizebar_corner_width;
@@ -1817,13 +1991,23 @@ static int getResizeDirection(WWindow * wwin, int x, int y, int dx, int dy, int
 
 	/* if not resizing through the resizebar */
 	if (!(flags & RESIZEBAR)) {
+
 		int xdir = (abs(x) < (wwin->client.width / 2)) ? LEFT : RIGHT;
 		int ydir = (abs(y) < (wwin->client.height / 2)) ? UP : DOWN;
-		if (abs(dx) < 2 || abs(dy) < 2) {
-			if (abs(dy) > abs(dx))
-				xdir = 0;
-			else
-				ydir = 0;
+
+		/* How much resize space is allowed */
+		int spacew = abs(wwin->client.width / 3);
+		int spaceh = abs(wwin->client.height / 3);
+
+		/* Determine where x fits */
+		if ((abs(x) > wwin->client.width/2 - spacew/2) &&
+		    (abs(x) < wwin->client.width/2 + spacew/2)) {
+			/* Resize vertically */
+			xdir = 0;
+		} else if ((abs(y) > wwin->client.height/2 - spaceh/2) &&
+		           (abs(y) < wwin->client.height/2 + spaceh/2)) {
+			/* Resize horizontally */
+			ydir = 0;
 		}
 		return (xdir | ydir);
 	}
@@ -1981,38 +2165,36 @@ void wMouseResizeWindow(WWindow * wwin, XEvent * ev)
 						     || abs(orig_y - event.xmotion.y_root) < HRESIZE_THRESHOLD))
 					flags |= HCONSTRAIN;
 
-				res = getResizeDirection(wwin, tx, ty,
-							 orig_x - event.xmotion.x_root,
-							 orig_y - event.xmotion.y_root, flags);
+				res = getResizeDirection(wwin, tx, ty, orig_y - event.xmotion.y_root, flags);
 
 				if (res == (UP | LEFT))
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_TOPLEFTRESIZE], CurrentTime);
+								 wPreferences.cursor[WCUR_TOPLEFTRESIZE], CurrentTime);
 				else if (res == (UP | RIGHT))
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_TOPRIGHTRESIZE], CurrentTime);
+								 wPreferences.cursor[WCUR_TOPRIGHTRESIZE], CurrentTime);
 				else if (res == (DOWN | LEFT))
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_BOTTOMLEFTRESIZE], CurrentTime);
+								 wPreferences.cursor[WCUR_BOTTOMLEFTRESIZE], CurrentTime);
 				else if (res == (DOWN | RIGHT))
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_BOTTOMRIGHTRESIZE], CurrentTime);
+								 wPreferences.cursor[WCUR_BOTTOMRIGHTRESIZE], CurrentTime);
 				else if (res == DOWN || res == UP)
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_VERTICALRESIZE], CurrentTime);
+								 wPreferences.cursor[WCUR_VERTICALRESIZE], CurrentTime);
 				else if (res & (DOWN | UP))
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_VERTICALRESIZE], CurrentTime);
+								 wPreferences.cursor[WCUR_VERTICALRESIZE], CurrentTime);
 				else if (res & (LEFT | RIGHT))
 					XChangeActivePointerGrab(dpy, ButtonMotionMask
 								 | ButtonReleaseMask | ButtonPressMask,
-								 wCursor[WCUR_HORIZONRESIZE], CurrentTime);
+								 wPreferences.cursor[WCUR_HORIZONRESIZE], CurrentTime);
 
 				XGrabKeyboard(dpy, root, False, GrabModeAsync, GrabModeAsync, CurrentTime);
 
@@ -2097,10 +2279,8 @@ void wMouseResizeWindow(WWindow * wwin, XEvent * ev)
 
 #undef LEFT
 #undef RIGHT
-#undef HORIZONTAL
 #undef UP
 #undef DOWN
-#undef VERTICAL
 #undef HCONSTRAIN
 #undef RESIZEBAR
 
@@ -2153,7 +2333,7 @@ void wSelectWindows(WScreen * scr, XEvent * ev)
 
 	if (XGrabPointer(dpy, scr->root_win, False, ButtonMotionMask
 			 | ButtonReleaseMask | ButtonPressMask, GrabModeAsync,
-			 GrabModeAsync, None, wCursor[WCUR_DEFAULT], CurrentTime) != Success) {
+			 GrabModeAsync, None, wPreferences.cursor[WCUR_NORMAL], CurrentTime) != Success) {
 		return;
 	}
 	XGrabServer(dpy);
@@ -2215,7 +2395,7 @@ void InteractivePlaceWindow(WWindow * wwin, int *x_ret, int *y_ret, unsigned wid
 	int junk;
 
 	if (XGrabPointer(dpy, root, True, PointerMotionMask | ButtonPressMask,
-			 GrabModeAsync, GrabModeAsync, None, wCursor[WCUR_DEFAULT], CurrentTime) != Success) {
+			 GrabModeAsync, GrabModeAsync, None, wPreferences.cursor[WCUR_NORMAL], CurrentTime) != Success) {
 		*x_ret = 0;
 		*y_ret = 0;
 		return;
