@@ -4,6 +4,7 @@
  *
  *  Copyright (c) 1997-2003 Alfredo K. Kojima
  *  Copyright (c) 1998-2003 Dan Pascu
+ *  Copyright (c) 2014 Window Maker Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,6 +34,7 @@
 #include <ctype.h>
 #include <time.h>
 #include <dirent.h>
+#include <errno.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -60,15 +62,11 @@
 
 #define MAX_SHORTCUT_LENGTH 32
 
-extern char *Locale;
-extern WDDomain *WDRootMenu;
-extern Cursor wCursor[WCUR_LAST];
-
 static WMenu *readMenuPipe(WScreen * scr, char **file_name);
 static WMenu *readPLMenuPipe(WScreen * scr, char **file_name);
 static WMenu *readMenuFile(WScreen *scr, const char *file_name);
 static WMenu *readMenuDirectory(WScreen *scr, const char *title, char **file_name, const char *command);
-static WMenu *configureMenu(WScreen * scr, WMPropList * definition, Bool includeGlobals);
+static WMenu *configureMenu(WScreen *scr, WMPropList *definition);
 static void menu_parser_register_macros(WMenuParser parser);
 
 typedef struct Shortcut {
@@ -156,7 +154,7 @@ static void execCommand(WMenu * menu, WMenuEntry * entry)
 	cmdline = ExpandOptions(menu->frame->screen_ptr, (char *)entry->clientdata);
 
 	XGrabPointer(dpy, menu->frame->screen_ptr->root_win, True, 0,
-		     GrabModeAsync, GrabModeAsync, None, wCursor[WCUR_WAIT], CurrentTime);
+		     GrabModeAsync, GrabModeAsync, None, wPreferences.cursor[WCUR_WAIT], CurrentTime);
 	XSync(dpy, 0);
 
 	if (cmdline) {
@@ -253,6 +251,10 @@ static void shutdownCommand(WMenu * menu, WMenuEntry * entry)
 
 static void restartCommand(WMenu * menu, WMenuEntry * entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) menu;
+	(void) entry;
+
 	Shutdown(WSRestartPreparationMode);
 	Restart((char *)entry->clientdata, False);
 	Restart(NULL, True);
@@ -260,26 +262,41 @@ static void restartCommand(WMenu * menu, WMenuEntry * entry)
 
 static void refreshCommand(WMenu * menu, WMenuEntry * entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	wRefreshDesktop(menu->frame->screen_ptr);
 }
 
 static void arrangeIconsCommand(WMenu * menu, WMenuEntry * entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	wArrangeIcons(menu->frame->screen_ptr, True);
 }
 
 static void showAllCommand(WMenu * menu, WMenuEntry * entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	wShowAllWindows(menu->frame->screen_ptr);
 }
 
 static void hideOthersCommand(WMenu * menu, WMenuEntry * entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	wHideOtherApplications(menu->frame->screen_ptr->focused_window);
 }
 
 static void saveSessionCommand(WMenu * menu, WMenuEntry * entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	if (!wPreferences.save_session_on_exit)
 		wSessionSaveState(menu->frame->screen_ptr);
 
@@ -288,17 +305,26 @@ static void saveSessionCommand(WMenu * menu, WMenuEntry * entry)
 
 static void clearSessionCommand(WMenu * menu, WMenuEntry * entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	wSessionClearState(menu->frame->screen_ptr);
 	wScreenSaveState(menu->frame->screen_ptr);
 }
 
 static void infoPanelCommand(WMenu * menu, WMenuEntry * entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	wShowInfoPanel(menu->frame->screen_ptr);
 }
 
 static void legalPanelCommand(WMenu * menu, WMenuEntry * entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	wShowLegalPanel(menu->frame->screen_ptr);
 }
 
@@ -309,14 +335,14 @@ static char *getLocalizedMenuFile(const char *menu)
 	char *buffer, *ptr, *locale;
 	int len;
 
-	if (!Locale)
+	if (!w_global.locale)
 		return NULL;
 
-	len = strlen(menu) + strlen(Locale) + 8;
+	len = strlen(menu) + strlen(w_global.locale) + 8;
 	buffer = wmalloc(len);
 
 	/* try menu.locale_name */
-	snprintf(buffer, len, "%s.%s", menu, Locale);
+	snprintf(buffer, len, "%s.%s", menu, w_global.locale);
 	if (access(buffer, F_OK) == 0)
 		return buffer;
 
@@ -352,7 +378,7 @@ Bool wRootMenuPerformShortcut(XEvent * event)
 	int done = 0;
 
 	/* ignore CapsLock */
-	modifiers = event->xkey.state & ValidModMask;
+	modifiers = event->xkey.state & w_global.shortcut.modifiers_mask;
 
 	for (ptr = shortcutList; ptr != NULL; ptr = ptr->next) {
 		if (ptr->keycode == 0 || ptr->menu->menu->screen_ptr != scr)
@@ -546,6 +572,7 @@ static void separateCommand(char *line, char ***file, char **command)
 					*command = wstrdup(tmp);
 				else
 					wwarning(_("%s: missing command"), line);
+				wfree(token);
 				break;
 			}
 			WMAddToArray(array, token);
@@ -575,7 +602,7 @@ static WMenu *constructPLMenu(WScreen *screen, const char *path)
 	if (!pl)
 		return NULL;
 
-	menu = configureMenu(screen, pl, False);
+	menu = configureMenu(screen, pl);
 
 	WMReleasePropList(pl);
 
@@ -601,6 +628,11 @@ static void constructMenu(WMenu * menu, WMenuEntry * entry)
 	separateCommand((char *)entry->clientdata, &path, &cmd);
 	if (path == NULL || *path == NULL || **path == 0) {
 		wwarning(_("invalid OPEN_MENU specification: %s"), (char *)entry->clientdata);
+		if (path) {
+			for (i = 0; path[i] != NULL; i++)
+				wfree(path[i]);
+			wfree(path);
+		}
 		if (cmd)
 			wfree(cmd);
 		return;
@@ -721,6 +753,11 @@ static void constructPLMenuFromPipe(WMenu * menu, WMenuEntry * entry)
 	if (path == NULL || *path == NULL || **path == 0) {
 		wwarning(_("invalid OPEN_PLMENU specification: %s"),
 		    (char *)entry->clientdata);
+		if (path) {
+			for (i = 0; path[i] != NULL; i++)
+				wfree(path[i]);
+			wfree(path);
+		}
 		if (cmd)
 			wfree(cmd);
 		return;
@@ -757,7 +794,7 @@ static void constructPLMenuFromPipe(WMenu * menu, WMenuEntry * entry)
 		wfree(cmd);
 
 }
-static void cleanupWorkspaceMenu(WMenu * menu)
+static void cleanupWorkspaceMenu(WMenu *menu)
 {
 	if (menu->frame->screen_ptr->workspace_menu == menu)
 		menu->frame->screen_ptr->workspace_menu = NULL;
@@ -1012,107 +1049,12 @@ static WMenu *parseCascade(WScreen * scr, WMenu * menu, WMenuParser parser)
 	return NULL;
 }
 
-static WMenu *readMenuFile(WScreen *scr, const char *file_name)
+static WMenu *readMenu(WScreen *scr, const char *flat_file, FILE *file)
 {
 	WMenu *menu = NULL;
-	FILE *file = NULL;
 	WMenuParser parser;
-	char *command, *params, *shortcut, *title;
+	char *title, *command, *params, *shortcut;
 
-	file = fopen(file_name, "rb");
-	if (!file) {
-		werror(_("%s:could not open menu file"), file_name);
-		return NULL;
-	}
-	parser = WMenuParserCreate(file_name, file, DEF_CONFIG_PATHS);
-	menu_parser_register_macros(parser);
-
-	while (WMenuParserGetLine(parser, &title, &command, &params, &shortcut)) {
-
-		if (command == NULL || !command[0]) {
-			WMenuParserError(parser, _("missing command in menu config") );
-			freeline(title, command, params, shortcut);
-			break;
-		}
-		if (strcasecmp(command, "MENU") == 0) {
-			menu = wMenuCreate(scr, M_(title), True);
-			menu->on_destroy = removeShortcutsForMenu;
-			if (!parseCascade(scr, menu, parser)) {
-				wMenuDestroy(menu, True);
-				menu = NULL;
-			}
-			freeline(title, command, params, shortcut);
-			break;
-		} else {
-			WMenuParserError(parser, _("invalid menu file, MENU command is missing") );
-			freeline(title, command, params, shortcut);
-			break;
-		}
-		freeline(title, command, params, shortcut);
-	}
-
-	WMenuParserDelete(parser);
-	fclose(file);
-
-	return menu;
-}
-
-/************    Menu Configuration From Pipe      *************/
-static WMenu *readPLMenuPipe(WScreen * scr, char **file_name)
-{
-	WMPropList *plist = NULL;
-	WMenu *menu = NULL;
-	char *filename;
-	char flat_file[MAXLINE];
-	int i;
-
-	flat_file[0] = '\0';
-
-	for (i = 0; file_name[i] != NULL; i++) {
-		strcat(flat_file, file_name[i]);
-		strcat(flat_file, " ");
-	}
-	filename = flat_file + (flat_file[1] == '|' ? 2 : 1);
-
-	plist = WMReadPropListFromPipe(filename);
-
-	if (!plist)
-		return NULL;
-
-	menu = configureMenu(scr, plist, False);
-
-	WMReleasePropList(plist);
-
-	if (!menu)
-		return NULL;
-
-	menu->on_destroy = removeShortcutsForMenu;
-	return menu;
-}
-
-static WMenu *readMenuPipe(WScreen * scr, char **file_name)
-{
-	WMenu *menu = NULL;
-	FILE *file = NULL;
-	WMenuParser parser;
-	char *command, *params, *shortcut, *title;
-	char *filename;
-	char flat_file[MAXLINE];
-	int i;
-
-	flat_file[0] = '\0';
-
-	for (i = 0; file_name[i] != NULL; i++) {
-		strcat(flat_file, file_name[i]);
-		strcat(flat_file, " ");
-	}
-	filename = flat_file + (flat_file[1] == '|' ? 2 : 1);
-
-	file = popen(filename, "r");
-	if (!file) {
-		werror(_("%s:could not open menu file"), filename);
-		return NULL;
-	}
 	parser = WMenuParserCreate(flat_file, file, DEF_CONFIG_PATHS);
 	menu_parser_register_macros(parser);
 
@@ -1133,7 +1075,7 @@ static WMenu *readMenuPipe(WScreen * scr, char **file_name)
 			freeline(title, command, params, shortcut);
 			break;
 		} else {
-			WMenuParserError(parser, _("no title given for the root menu") );
+			WMenuParserError(parser, _("invalid menu, no menu title given") );
 			freeline(title, command, params, shortcut);
 			break;
 		}
@@ -1142,6 +1084,106 @@ static WMenu *readMenuPipe(WScreen * scr, char **file_name)
 	}
 
 	WMenuParserDelete(parser);
+	return menu;
+}
+
+static WMenu *readMenuFile(WScreen *scr, const char *file_name)
+{
+	WMenu *menu = NULL;
+	FILE *file = NULL;
+
+	file = fopen(file_name, "rb");
+	if (!file) {
+		werror(_("could not open menu file \"%s\": %s"), file_name, strerror(errno));
+		return NULL;
+	}
+	menu = readMenu(scr, file_name, file);
+	fclose(file);
+
+	return menu;
+}
+
+static inline int generate_command_from_list(char *buffer, size_t buffer_size, char **command_elements)
+{
+	char *rd;
+	int wr_idx;
+	int i;
+
+	wr_idx = 0;
+	for (i = 0; command_elements[i] != NULL; i++) {
+
+		if (i > 0)
+			if (wr_idx < buffer_size - 1)
+				buffer[wr_idx++] = ' ';
+
+		for (rd = command_elements[i]; *rd != '\0'; rd++) {
+			if (wr_idx < buffer_size - 1)
+				buffer[wr_idx++] = *rd;
+			else
+				return 1;
+		}
+	}
+	buffer[wr_idx] = '\0';
+	return 0;
+}
+
+/************    Menu Configuration From Pipe      *************/
+static WMenu *readPLMenuPipe(WScreen * scr, char **file_name)
+{
+	WMPropList *plist = NULL;
+	WMenu *menu = NULL;
+	char *filename;
+	char flat_file[MAXLINE];
+
+	if (generate_command_from_list(flat_file, sizeof(flat_file), file_name)) {
+		werror(_("could not open menu file \"%s\": %s"),
+		       file_name[0], _("pipe command for PropertyList is too long"));
+		return NULL;
+	}
+	filename = flat_file + (flat_file[1] == '|' ? 2 : 1);
+
+	plist = WMReadPropListFromPipe(filename);
+
+	if (!plist)
+		return NULL;
+
+	menu = configureMenu(scr, plist);
+
+	WMReleasePropList(plist);
+
+	if (!menu)
+		return NULL;
+
+	menu->on_destroy = removeShortcutsForMenu;
+	return menu;
+}
+
+static WMenu *readMenuPipe(WScreen * scr, char **file_name)
+{
+	WMenu *menu = NULL;
+	FILE *file = NULL;
+	char *filename;
+	char flat_file[MAXLINE];
+
+	if (generate_command_from_list(flat_file, sizeof(flat_file), file_name)) {
+		werror(_("could not open menu file \"%s\": %s"),
+		       file_name[0], _("pipe command is too long"));
+		return NULL;
+	}
+	filename = flat_file + (flat_file[1] == '|' ? 2 : 1);
+
+	/*
+	 * In case of memory problem, 'popen' will not set the errno, so we initialise it
+	 * to be able to display a meaningful message. For other problems, 'popen' will
+	 * properly set errno, so we'll still get a good message
+	 */
+	errno = ENOMEM;
+	file = popen(filename, "r");
+	if (!file) {
+		werror(_("could not open menu file \"%s\": %s"), filename, strerror(errno));
+		return NULL;
+	}
+	menu = readMenu(scr, flat_file, file);
 	pclose(file);
 
 	return menu;
@@ -1344,8 +1386,7 @@ static WMenu *readMenuDirectory(WScreen *scr, const char *title, char **path, co
 		addMenuEntry(menu, M_(data->name), NULL, "OPEN_MENU", buffer, path[data->index]);
 
 		wfree(buffer);
-		if (data->name)
-			wfree(data->name);
+		wfree(data->name);
 		wfree(data);
 	}
 
@@ -1390,8 +1431,7 @@ static WMenu *readMenuDirectory(WScreen *scr, const char *title, char **path, co
 		addMenuEntry(menu, M_(data->name), NULL, "SHEXEC", buffer, path[data->index]);
 
 		wfree(buffer);
-		if (data->name)
-			wfree(data->name);
+		wfree(data->name);
 		wfree(data);
 	}
 
@@ -1422,7 +1462,7 @@ static WMenu *makeDefaultMenu(WScreen * scr)
  *
  *----------------------------------------------------------------------
  */
-static WMenu *configureMenu(WScreen * scr, WMPropList * definition, Bool includeGlobals)
+static WMenu *configureMenu(WScreen *scr, WMPropList *definition)
 {
 	WMenu *menu = NULL;
 	WMPropList *elem;
@@ -1464,7 +1504,7 @@ static WMenu *configureMenu(WScreen * scr, WMPropList * definition, Bool include
 
 		if (!scr->root_menu || stat_buf.st_mtime > scr->root_menu->timestamp
 		    /* if the pointer in WMRootMenu has changed */
-		    || WDRootMenu->timestamp > scr->root_menu->timestamp) {
+		    || w_global.domain.root_menu->timestamp > scr->root_menu->timestamp) {
 
 			if (menu_is_default) {
 				wwarning(_
@@ -1474,7 +1514,7 @@ static WMenu *configureMenu(WScreen * scr, WMPropList * definition, Bool include
 
 			menu = readMenuFile(scr, path);
 			if (menu)
-				menu->timestamp = WMAX(stat_buf.st_mtime, WDRootMenu->timestamp);
+				menu->timestamp = WMAX(stat_buf.st_mtime, w_global.domain.root_menu->timestamp);
 		} else {
 			menu = NULL;
 		}
@@ -1500,20 +1540,6 @@ static WMenu *configureMenu(WScreen * scr, WMPropList * definition, Bool include
 	menu = wMenuCreate(scr, M_(mtitle), False);
 	menu->on_destroy = removeShortcutsForMenu;
 
-#ifdef GLOBAL_SUBMENU_FILE
-	if (includeGlobals) {
-		WMenu *submenu;
-		WMenuEntry *mentry;
-
-		submenu = readMenuFile(scr, GLOBAL_SUBMENU_FILE);
-
-		if (submenu) {
-			mentry = wMenuAddCallback(menu, submenu->frame->title, NULL, NULL);
-			wMenuEntrySetCascade(menu, mentry, submenu);
-		}
-	}
-#endif
-
 	for (i = 1; i < count; i++) {
 		elem = WMGetFromPLArray(definition, i);
 #if 0
@@ -1532,7 +1558,7 @@ static WMenu *configureMenu(WScreen * scr, WMPropList * definition, Bool include
 			WMenuEntry *mentry;
 
 			/* submenu */
-			submenu = configureMenu(scr, elem, True);
+			submenu = configureMenu(scr, elem);
 			if (submenu) {
 				mentry = wMenuAddCallback(menu, submenu->frame->title, NULL, NULL);
 				wMenuEntrySetCascade(menu, mentry, submenu);
@@ -1619,22 +1645,22 @@ void OpenRootMenu(WScreen * scr, int x, int y, int keyboard)
 		return;
 	}
 
-	definition = WDRootMenu->dictionary;
+	definition = w_global.domain.root_menu->dictionary;
 
 	/*
 	   definition = PLGetDomain(domain);
 	 */
 	if (definition) {
 		if (WMIsPLArray(definition)) {
-			if (!scr->root_menu || WDRootMenu->timestamp > scr->root_menu->timestamp) {
-				menu = configureMenu(scr, definition, True);
+			if (!scr->root_menu || w_global.domain.root_menu->timestamp > scr->root_menu->timestamp) {
+				menu = configureMenu(scr, definition);
 				if (menu)
-					menu->timestamp = WDRootMenu->timestamp;
+					menu->timestamp = w_global.domain.root_menu->timestamp;
 
 			} else
 				menu = NULL;
 		} else {
-			menu = configureMenu(scr, definition, True);
+			menu = configureMenu(scr, definition);
 		}
 	}
 
