@@ -38,13 +38,13 @@
 #include <X11/cursorfont.h>
 #include <X11/Xproto.h>
 #include <X11/keysym.h>
-#ifdef SHAPE
+#ifdef USE_XSHAPE
 #include <X11/extensions/shape.h>
 #endif
 #ifdef KEEP_XKB_LOCK_STATUS
 #include <X11/XKBlib.h>
 #endif
-#ifdef HAVE_XRANDR
+#ifdef USE_RANDR
 #include <X11/extensions/Xrandr.h>
 #endif
 
@@ -68,7 +68,7 @@
 #include "wmspec.h"
 #include "event.h"
 #include "switchmenu.h"
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 #include "xdnd.h"
 #endif
 
@@ -84,60 +84,6 @@
 # define SA_NODEFER 0
 #endif
 
-/****** Global Variables ******/
-extern WDDomain *WDWindowMaker;
-extern WDDomain *WDRootMenu;
-extern WDDomain *WDWindowAttributes;
-extern WShortKey wKeyBindings[WKBD_LAST];
-extern int wScreenCount;
-
-#ifdef SHAPE
-extern Bool wShapeSupported;
-extern int wShapeEventBase;
-#endif
-
-#ifdef KEEP_XKB_LOCK_STATUS
-extern Bool wXkbSupported;
-extern int wXkbEventBase;
-#endif
-
-/* contexts */
-extern XContext wWinContext;
-extern XContext wAppWinContext;
-extern XContext wStackContext;
-extern XContext wVEdgeContext;
-
-/* atoms */
-extern Atom _XA_WM_STATE;
-extern Atom _XA_WM_CHANGE_STATE;
-extern Atom _XA_WM_PROTOCOLS;
-extern Atom _XA_WM_TAKE_FOCUS;
-extern Atom _XA_WM_DELETE_WINDOW;
-extern Atom _XA_WM_SAVE_YOURSELF;
-extern Atom _XA_WM_CLIENT_LEADER;
-extern Atom _XA_WM_COLORMAP_WINDOWS;
-extern Atom _XA_WM_COLORMAP_NOTIFY;
-extern Atom _XA_GNUSTEP_WM_ATTR;
-extern Atom _XA_WINDOWMAKER_MENU;
-extern Atom _XA_WINDOWMAKER_WM_PROTOCOLS;
-extern Atom _XA_WINDOWMAKER_STATE;
-extern Atom _XA_WINDOWMAKER_WM_FUNCTION;
-extern Atom _XA_WINDOWMAKER_NOTICEBOARD;
-extern Atom _XA_WINDOWMAKER_COMMAND;
-extern Atom _XA_WINDOWMAKER_ICON_SIZE;
-extern Atom _XA_WINDOWMAKER_ICON_TILE;
-extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
-extern Atom _XA_GNUSTEP_TITLEBAR_STATE;
-extern Atom _XA_WM_IGNORE_FOCUS_EVENTS;
-
-/* cursors */
-extern Cursor wCursor[WCUR_LAST];
-
-#ifndef HAVE_INOTIFY
-/* special flags */
-extern char WDelayedActionSet;
-#endif
-
 /***** Local *****/
 static WScreen **wScreen = NULL;
 static unsigned int _NumLockMask = 0;
@@ -181,33 +127,14 @@ static int catchXError(Display * dpy, XErrorEvent * error)
  */
 static int handleXIO(Display * xio_dpy)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) xio_dpy;
+
 	dpy = NULL;
 	Exit(0);
 	return 0;
 }
 
-#ifndef HAVE_INOTIFY
-/*
- *----------------------------------------------------------------------
- * delayedAction-
- *      Action to be executed after the signal() handler is exited.
- *----------------------------------------------------------------------
- */
-static void delayedAction(void *cdata)
-{
-	if (WDelayedActionSet == 0)
-		return;
-
-	WDelayedActionSet--;
-	/*
-	 * Make the event dispatcher do whatever it needs to do,
-	 * including handling zombie processes, restart and exit
-	 * signals.
-	 */
-	DispatchEvent(NULL);
-}
-#endif
-
 /*
  *----------------------------------------------------------------------
  * handleExitSig--
@@ -239,6 +166,8 @@ static RETSIGTYPE handleExitSig(int sig)
 /* Dummy signal handler */
 static void dummyHandler(int sig)
 {
+	/* Parameter is not used, but tell the compiler that it is ok */
+	(void) sig;
 }
 
 /*
@@ -272,6 +201,9 @@ static RETSIGTYPE buryChild(int foo)
 	int save_errno = errno;
 	sigset_t sigs;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) foo;
+
 	sigfillset(&sigs);
 	/* Block signals so that NotifyDeadProcess() doesn't get fux0red */
 	sigprocmask(SIG_BLOCK, &sigs, NULL);
@@ -403,7 +335,7 @@ wHackedGrabButton(unsigned int button, unsigned int modifiers,
 
 WScreen *wScreenWithNumber(int i)
 {
-	assert(i < wScreenCount);
+	assert(i < w_global.screen_count);
 
 	return wScreen[i];
 }
@@ -412,13 +344,13 @@ WScreen *wScreenForRootWindow(Window window)
 {
 	int i;
 
-	if (wScreenCount == 1)
+	if (w_global.screen_count == 1)
 		return wScreen[0];
 
 	/* Since the number of heads will probably be small (normally 2),
 	 * it should be faster to use this than a hash table, because
 	 * of the overhead. */
-	for (i = 0; i < wScreenCount; i++)
+	for (i = 0; i < w_global.screen_count; i++)
 		if (wScreen[i]->root_win == window)
 			return wScreen[i];
 
@@ -429,7 +361,7 @@ WScreen *wScreenForWindow(Window window)
 {
 	XWindowAttributes attr;
 
-	if (wScreenCount == 1)
+	if (w_global.screen_count == 1)
 		return wScreen[0];
 
 	if (XGetWindowAttributes(dpy, window, &attr))
@@ -462,6 +394,8 @@ static char *atomNames[] = {
 	GNUSTEP_WM_MINIATURIZE_WINDOW,
 	GNUSTEP_TITLEBAR_STATE,
 
+	"_GTK_APPLICATION_OBJECT_PATH",
+
 	"WM_IGNORE_FOCUS_EVENTS"
 };
 
@@ -478,88 +412,87 @@ static char *atomNames[] = {
 void StartUp(Bool defaultScreenOnly)
 {
 	struct sigaction sig_action;
-	int j, max;
-#ifdef HAVE_XRANDR
-	int dummy;
-#endif
-	Atom atom[sizeof(atomNames) / sizeof(atomNames[0])];
+	int i, j, max;
+	char **formats;
+	Atom atom[wlengthof(atomNames)];
 
 	/*
 	 * Ignore CapsLock in modifiers
 	 */
-	ValidModMask = 0xff & ~LockMask;
+	w_global.shortcut.modifiers_mask = 0xff & ~LockMask;
 
 	getOffendingModifiers();
 	/*
 	 * Ignore NumLock and ScrollLock too
 	 */
-	ValidModMask &= ~(_NumLockMask | _ScrollLockMask);
+	w_global.shortcut.modifiers_mask &= ~(_NumLockMask | _ScrollLockMask);
 
 	memset(&wKeyBindings, 0, sizeof(wKeyBindings));
 
-	wWinContext = XUniqueContext();
-	wAppWinContext = XUniqueContext();
-	wStackContext = XUniqueContext();
-	wVEdgeContext = XUniqueContext();
+	w_global.context.client_win = XUniqueContext();
+	w_global.context.app_win = XUniqueContext();
+	w_global.context.stack = XUniqueContext();
 
 	/*    _XA_VERSION = XInternAtom(dpy, "VERSION", False); */
 
 #ifdef HAVE_XINTERNATOMS
-	XInternAtoms(dpy, atomNames, sizeof(atomNames) / sizeof(atomNames[0]), False, atom);
+	XInternAtoms(dpy, atomNames, wlengthof(atomNames), False, atom);
 #else
 
 	{
 		int i;
-		for (i = 0; i < sizeof(atomNames) / sizeof(atomNames[0]); i++)
+		for (i = 0; i < wlengthof(atomNames); i++)
 			atom[i] = XInternAtom(dpy, atomNames[i], False);
 	}
 #endif
 
-	_XA_WM_STATE = atom[0];
-	_XA_WM_CHANGE_STATE = atom[1];
-	_XA_WM_PROTOCOLS = atom[2];
-	_XA_WM_TAKE_FOCUS = atom[3];
-	_XA_WM_DELETE_WINDOW = atom[4];
-	_XA_WM_SAVE_YOURSELF = atom[5];
-	_XA_WM_CLIENT_LEADER = atom[6];
-	_XA_WM_COLORMAP_WINDOWS = atom[7];
-	_XA_WM_COLORMAP_NOTIFY = atom[8];
+	w_global.atom.wm.state = atom[0];
+	w_global.atom.wm.change_state = atom[1];
+	w_global.atom.wm.protocols = atom[2];
+	w_global.atom.wm.take_focus = atom[3];
+	w_global.atom.wm.delete_window = atom[4];
+	w_global.atom.wm.save_yourself = atom[5];
+	w_global.atom.wm.client_leader = atom[6];
+	w_global.atom.wm.colormap_windows = atom[7];
+	w_global.atom.wm.colormap_notify = atom[8];
 
-	_XA_WINDOWMAKER_MENU = atom[9];
-	_XA_WINDOWMAKER_STATE = atom[10];
-	_XA_WINDOWMAKER_WM_PROTOCOLS = atom[11];
-	_XA_WINDOWMAKER_WM_FUNCTION = atom[12];
-	_XA_WINDOWMAKER_NOTICEBOARD = atom[13];
-	_XA_WINDOWMAKER_COMMAND = atom[14];
-	_XA_WINDOWMAKER_ICON_SIZE = atom[15];
-	_XA_WINDOWMAKER_ICON_TILE = atom[16];
+	w_global.atom.wmaker.menu = atom[9];
+	w_global.atom.wmaker.state = atom[10];
+	w_global.atom.wmaker.wm_protocols = atom[11];
+	w_global.atom.wmaker.wm_function = atom[12];
+	w_global.atom.wmaker.noticeboard = atom[13];
+	w_global.atom.wmaker.command = atom[14];
+	w_global.atom.wmaker.icon_size = atom[15];
+	w_global.atom.wmaker.icon_tile = atom[16];
 
-	_XA_GNUSTEP_WM_ATTR = atom[17];
-	_XA_GNUSTEP_WM_MINIATURIZE_WINDOW = atom[18];
-	_XA_GNUSTEP_TITLEBAR_STATE = atom[19];
+	w_global.atom.gnustep.wm_attr = atom[17];
+	w_global.atom.gnustep.wm_miniaturize_window = atom[18];
+	w_global.atom.gnustep.titlebar_state = atom[19];
 
-	_XA_WM_IGNORE_FOCUS_EVENTS = atom[20];
+	w_global.atom.desktop.gtk_object_path = atom[20];
 
-#ifdef XDND
+	w_global.atom.wm.ignore_focus_events = atom[21];
+
+#ifdef USE_DOCK_XDND
 	wXDNDInitializeAtoms();
 #endif
 
 	/* cursors */
-	wCursor[WCUR_NORMAL] = None;	/* inherit from root */
-	wCursor[WCUR_ROOT] = XCreateFontCursor(dpy, XC_left_ptr);
-	wCursor[WCUR_ARROW] = XCreateFontCursor(dpy, XC_top_left_arrow);
-	wCursor[WCUR_MOVE] = XCreateFontCursor(dpy, XC_fleur);
-	wCursor[WCUR_RESIZE] = XCreateFontCursor(dpy, XC_sizing);
-	wCursor[WCUR_TOPLEFTRESIZE] = XCreateFontCursor(dpy, XC_top_left_corner);
-	wCursor[WCUR_TOPRIGHTRESIZE] = XCreateFontCursor(dpy, XC_top_right_corner);
-	wCursor[WCUR_BOTTOMLEFTRESIZE] = XCreateFontCursor(dpy, XC_bottom_left_corner);
-	wCursor[WCUR_BOTTOMRIGHTRESIZE] = XCreateFontCursor(dpy, XC_bottom_right_corner);
-	wCursor[WCUR_VERTICALRESIZE] = XCreateFontCursor(dpy, XC_sb_v_double_arrow);
-	wCursor[WCUR_HORIZONRESIZE] = XCreateFontCursor(dpy, XC_sb_h_double_arrow);
-	wCursor[WCUR_WAIT] = XCreateFontCursor(dpy, XC_watch);
-	wCursor[WCUR_QUESTION] = XCreateFontCursor(dpy, XC_question_arrow);
-	wCursor[WCUR_TEXT] = XCreateFontCursor(dpy, XC_xterm);	/* odd name??? */
-	wCursor[WCUR_SELECT] = XCreateFontCursor(dpy, XC_cross);
+	wPreferences.cursor[WCUR_NORMAL] = None;	/* inherit from root */
+	wPreferences.cursor[WCUR_ROOT] = XCreateFontCursor(dpy, XC_left_ptr);
+	wPreferences.cursor[WCUR_ARROW] = XCreateFontCursor(dpy, XC_top_left_arrow);
+	wPreferences.cursor[WCUR_MOVE] = XCreateFontCursor(dpy, XC_fleur);
+	wPreferences.cursor[WCUR_RESIZE] = XCreateFontCursor(dpy, XC_sizing);
+	wPreferences.cursor[WCUR_TOPLEFTRESIZE] = XCreateFontCursor(dpy, XC_top_left_corner);
+	wPreferences.cursor[WCUR_TOPRIGHTRESIZE] = XCreateFontCursor(dpy, XC_top_right_corner);
+	wPreferences.cursor[WCUR_BOTTOMLEFTRESIZE] = XCreateFontCursor(dpy, XC_bottom_left_corner);
+	wPreferences.cursor[WCUR_BOTTOMRIGHTRESIZE] = XCreateFontCursor(dpy, XC_bottom_right_corner);
+	wPreferences.cursor[WCUR_VERTICALRESIZE] = XCreateFontCursor(dpy, XC_sb_v_double_arrow);
+	wPreferences.cursor[WCUR_HORIZONRESIZE] = XCreateFontCursor(dpy, XC_sb_h_double_arrow);
+	wPreferences.cursor[WCUR_WAIT] = XCreateFontCursor(dpy, XC_watch);
+	wPreferences.cursor[WCUR_QUESTION] = XCreateFontCursor(dpy, XC_question_arrow);
+	wPreferences.cursor[WCUR_TEXT] = XCreateFontCursor(dpy, XC_xterm);	/* odd name??? */
+	wPreferences.cursor[WCUR_SELECT] = XCreateFontCursor(dpy, XC_cross);
 
 	Pixmap cur = XCreatePixmap(dpy, DefaultRootWindow(dpy), 16, 16, 1);
 	GC gc = XCreateGC(dpy, cur, 0, NULL);
@@ -568,15 +501,9 @@ void StartUp(Bool defaultScreenOnly)
 	XSetForeground(dpy, gc, 0);
 	XFillRectangle(dpy, cur, gc, 0, 0, 16, 16);
 	XFreeGC(dpy, gc);
-	wCursor[WCUR_EMPTY] = XCreatePixmapCursor(dpy, cur, cur, &black, &black, 0, 0);
+	wPreferences.cursor[WCUR_EMPTY] = XCreatePixmapCursor(dpy, cur, cur, &black, &black, 0, 0);
 	XFreePixmap(dpy, cur);
 
-
-#ifndef HAVE_INOTIFY
-	/* signal handler stuff that gets called when a signal is caught */
-	WMAddPersistentTimerHandler(500, delayedAction, NULL);
-#endif
-
 	/* emergency exit... */
 	sig_action.sa_handler = handleSig;
 	sigemptyset(&sig_action.sa_mask);
@@ -636,13 +563,13 @@ void StartUp(Bool defaultScreenOnly)
 	WMHookEventHandler(DispatchEvent);
 
 	/* initialize defaults stuff */
-	WDWindowMaker = wDefaultsInitDomain("WindowMaker", True);
-	if (!WDWindowMaker->dictionary)
+	w_global.domain.wmaker = wDefaultsInitDomain("WindowMaker", True);
+	if (!w_global.domain.wmaker->dictionary)
 		wwarning(_("could not read domain \"%s\" from defaults database"), "WindowMaker");
 
 	/* read defaults that don't change until a restart and are
 	 * screen independent */
-	wReadStaticDefaults(WDWindowMaker ? WDWindowMaker->dictionary : NULL);
+	wReadStaticDefaults(w_global.domain.wmaker ? w_global.domain.wmaker->dictionary : NULL);
 
 	/* check sanity of some values */
 	if (wPreferences.icon_size < 16) {
@@ -652,30 +579,30 @@ void StartUp(Bool defaultScreenOnly)
 	}
 
 	/* init other domains */
-	WDRootMenu = wDefaultsInitDomain("WMRootMenu", False);
-	if (!WDRootMenu->dictionary)
+	w_global.domain.root_menu = wDefaultsInitDomain("WMRootMenu", False);
+	if (!w_global.domain.root_menu->dictionary)
 		wwarning(_("could not read domain \"%s\" from defaults database"), "WMRootMenu");
 
-	wDefaultsMergeGlobalMenus(WDRootMenu);
+	wDefaultsMergeGlobalMenus(w_global.domain.root_menu);
 
-	WDWindowAttributes = wDefaultsInitDomain("WMWindowAttributes", True);
-	if (!WDWindowAttributes->dictionary)
+	w_global.domain.window_attr = wDefaultsInitDomain("WMWindowAttributes", True);
+	if (!w_global.domain.window_attr->dictionary)
 		wwarning(_("could not read domain \"%s\" from defaults database"), "WMWindowAttributes");
 
 	XSetErrorHandler((XErrorHandler) catchXError);
 
-#ifdef SHAPE
+#ifdef USE_XSHAPE
 	/* ignore j */
-	wShapeSupported = XShapeQueryExtension(dpy, &wShapeEventBase, &j);
+	w_global.xext.shape.supported = XShapeQueryExtension(dpy, &w_global.xext.shape.event_base, &j);
 #endif
 
-#ifdef HAVE_XRANDR
-	has_randr = XRRQueryExtension(dpy, &randr_event_base, &dummy);
+#ifdef USE_RANDR
+	w_global.xext.randr.supported = XRRQueryExtension(dpy, &w_global.xext.randr.event_base, &j);
 #endif
 
 #ifdef KEEP_XKB_LOCK_STATUS
-	wXkbSupported = XkbQueryExtension(dpy, NULL, &wXkbEventBase, NULL, NULL, NULL);
-	if (wPreferences.modelock && !wXkbSupported) {
+	w_global.xext.xkb.supported = XkbQueryExtension(dpy, NULL, &w_global.xext.xkb.event_base, NULL, NULL, NULL);
+	if (wPreferences.modelock && !w_global.xext.xkb.supported) {
 		wwarning(_("XKB is not supported. KbdModeLock is automatically disabled."));
 		wPreferences.modelock = 0;
 	}
@@ -688,30 +615,41 @@ void StartUp(Bool defaultScreenOnly)
 
 	wScreen = wmalloc(sizeof(WScreen *) * max);
 
-	wScreenCount = 0;
+	w_global.screen_count = 0;
+
+	/* Check if TIFF images are supported */
+	formats = RSupportedFileFormats();
+	if (formats) {
+		for (i = 0; formats[i] != NULL; i++) {
+			if (strcmp(formats[i], "TIFF") == 0) {
+				wPreferences.supports_tiff = 1;
+				break;
+			}
+		}
+	}
 
 	/* manage the screens */
 	for (j = 0; j < max; j++) {
 		if (defaultScreenOnly || max == 1) {
-			wScreen[wScreenCount] = wScreenInit(DefaultScreen(dpy));
-			if (!wScreen[wScreenCount]) {
+			wScreen[w_global.screen_count] = wScreenInit(DefaultScreen(dpy));
+			if (!wScreen[w_global.screen_count]) {
 				wfatal(_("it seems that there is already a window manager running"));
 				Exit(1);
 			}
 		} else {
-			wScreen[wScreenCount] = wScreenInit(j);
-			if (!wScreen[wScreenCount]) {
+			wScreen[w_global.screen_count] = wScreenInit(j);
+			if (!wScreen[w_global.screen_count]) {
 				wwarning(_("could not manage screen %i"), j);
 				continue;
 			}
 		}
-		wScreenCount++;
+		w_global.screen_count++;
 	}
 
 	InitializeSwitchMenu();
 
 	/* initialize/restore state for the screens */
-	for (j = 0; j < wScreenCount; j++) {
+	for (j = 0; j < w_global.screen_count; j++) {
 		int lastDesktop;
 
 		lastDesktop = wNETWMGetCurrentDesktopFromHint(wScreen[j]);
@@ -764,14 +702,14 @@ void StartUp(Bool defaultScreenOnly)
 			wSessionRestoreLastWorkspace(wScreen[j]);
 	}
 
-	if (wScreenCount == 0) {
+	if (w_global.screen_count == 0) {
 		wfatal(_("could not manage any screen"));
 		Exit(1);
 	}
 
 #ifndef HAVE_INOTIFY
 	/* setup defaults file polling */
-	if (!wPreferences.flags.nopolling && !wPreferences.flags.noupdates)
+	if (!wPreferences.flags.noupdates)
 		WMAddTimerHandler(3000, wDefaultsCheckDomains, NULL);
 #endif
 
