@@ -21,9 +21,9 @@
 #ifndef WMSUPERFLUOUS_H
 #define WMSUPERFLUOUS_H
 
-#define PIECES ((64/ICON_KABOOM_PIECE_SIZE)*(64/ICON_KABOOM_PIECE_SIZE))
+#include "dock.h"
 
-extern void DoKaboom(WScreen *scr, Window win, int x, int y);
-extern Pixmap MakeGhostDock(WDock *dock, int sx, int dx, int y);
-extern Pixmap MakeGhostIcon(WScreen *scr, Drawable drawable);
-#endif /* WMSUPERFLUOUS_H */
+void DoKaboom(WScreen *scr, Window win, int x, int y);
+Pixmap MakeGhostIcon(WScreen *scr, Drawable drawable);
+void DoWindowBirth(WWindow *wwin);
+#endif
