@@ -62,23 +62,12 @@ static void wsobserver(void *self, WMNotification * notif);
 static void focusWindow(WMenu * menu, WMenuEntry * entry)
 {
 	WWindow *wwin;
-	WScreen *scr;
-	int x, y, move = 0;
+
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) menu;
 
 	wwin = (WWindow *) entry->clientdata;
-	scr = wwin->screen_ptr;
-
-	wMakeWindowVisible(wwin);
-
-	x = wwin->frame_x;
-	y = wwin->frame_y;
-
-	/* bring window back to visible area */
-	move = wScreenBringInside(scr, &x, &y, wwin->frame->core->width, wwin->frame->core->height);
-
-	if (move) {
-		wWindowConfigure(wwin, x, y, wwin->client.width, wwin->client.height);
-	}
+	wWindowSingleFocus(wwin);
 }
 
 void InitializeSwitchMenu(void)
@@ -388,6 +377,9 @@ static void observer(void *self, WMNotification * notif)
 	const char *name = WMGetNotificationName(notif);
 	void *data = WMGetNotificationClientData(notif);
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	if (!wwin)
 		return;
 
@@ -416,6 +408,9 @@ static void wsobserver(void *self, WMNotification * notif)
 	const char *name = WMGetNotificationName(notif);
 	void *data = WMGetNotificationClientData(notif);
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	if (strcmp(name, WMNWorkspaceNameChanged) == 0) {
 		UpdateSwitchMenuWorkspace(scr, (uintptr_t)data);
 	} else if (strcmp(name, WMNWorkspaceChanged) == 0) {
