@@ -47,9 +47,6 @@
     if (value) {attr->flag = getBool(attrib, value); \
     if (mask) mask->flag = 1;}
 
-/* Global stuff */
-extern WDDomain *WDWindowAttributes;
-
 /* Local stuff */
 
 /* type converters */
@@ -177,8 +174,8 @@ static WMPropList *get_value_from_instanceclass(const char *value)
 
 	WMPLSetCaseSensitive(True);
 
-	if (WDWindowAttributes->dictionary)
-		val = key ? WMGetFromPLDictionary(WDWindowAttributes->dictionary, key) : NULL;
+	if (w_global.domain.window_attr->dictionary)
+		val = key ? WMGetFromPLDictionary(w_global.domain.window_attr->dictionary, key) : NULL;
 
 	if (key)
 		WMReleasePropList(key);
@@ -222,8 +219,8 @@ void wDefaultFillAttributes(const char *instance, const char *class,
 
 	WMPLSetCaseSensitive(True);
 
-	if ((WDWindowAttributes->dictionary) && (useGlobalDefault))
-		da = WMGetFromPLDictionary(WDWindowAttributes->dictionary, AnyWindow);
+	if ((w_global.domain.window_attr->dictionary) && (useGlobalDefault))
+		da = WMGetFromPLDictionary(w_global.domain.window_attr->dictionary, AnyWindow);
 
 	/* get the data */
 	value = get_value(dw, dc, dn, da, ANoTitlebar, No, useGlobalDefault);
@@ -331,7 +328,7 @@ static WMPropList *get_generic_value(const char *instance, const char *class,
 		key = WMCreatePLString(buffer);
 		wfree(buffer);
 
-		dict = WMGetFromPLDictionary(WDWindowAttributes->dictionary, key);
+		dict = WMGetFromPLDictionary(w_global.domain.window_attr->dictionary, key);
 		WMReleasePropList(key);
 
 		if (dict)
@@ -342,7 +339,7 @@ static WMPropList *get_generic_value(const char *instance, const char *class,
 	if (!value && instance) {
 		key = WMCreatePLString(instance);
 
-		dict = WMGetFromPLDictionary(WDWindowAttributes->dictionary, key);
+		dict = WMGetFromPLDictionary(w_global.domain.window_attr->dictionary, key);
 		WMReleasePropList(key);
 
 		if (dict)
@@ -353,7 +350,7 @@ static WMPropList *get_generic_value(const char *instance, const char *class,
 	if (!value && class) {
 		key = WMCreatePLString(class);
 
-		dict = WMGetFromPLDictionary(WDWindowAttributes->dictionary, key);
+		dict = WMGetFromPLDictionary(w_global.domain.window_attr->dictionary, key);
 		WMReleasePropList(key);
 
 		if (dict)
@@ -363,7 +360,7 @@ static WMPropList *get_generic_value(const char *instance, const char *class,
 	/* Search the default icon name - See default_icon argument! */
 	if (!value && default_icon) {
 		/* AnyWindow is "*" - see wdefaults.c */
-		dict = WMGetFromPLDictionary(WDWindowAttributes->dictionary, AnyWindow);
+		dict = WMGetFromPLDictionary(w_global.domain.window_attr->dictionary, AnyWindow);
 
 		if (dict)
 			value = WMGetFromPLDictionary(dict, option);
@@ -375,11 +372,11 @@ static WMPropList *get_generic_value(const char *instance, const char *class,
 }
 
 /* Get the file name of the image, using instance and class */
-char *get_icon_filename(WScreen *scr, const char *winstance, const char *wclass, const char *command,
+char *get_icon_filename(const char *winstance, const char *wclass, const char *command,
 			Bool default_icon)
 {
-	char *file_name = NULL;
-	char *file_path = NULL;
+	char *file_name;
+	char *file_path;
 
 	/* Get the file name of the image, using instance and class */
 	file_name = wDefaultGetIconFile(winstance, wclass, default_icon);
@@ -387,29 +384,32 @@ char *get_icon_filename(WScreen *scr, const char *winstance, const char *wclass,
 	/* Check if the file really exists in the disk */
 	if (file_name)
 		file_path = FindImage(wPreferences.icon_path, file_name);
+	else
+		file_path = NULL;
 
 	/* If the specific icon filename is not found, and command is specified,
 	 * then include the .app icons and re-do the search. */
-	if ((!file_name || !file_path ) && scr && command) {
-		wApplicationExtractDirPackIcon(scr, command, winstance, wclass);
+	if (!file_path && command) {
+		wApplicationExtractDirPackIcon(command, winstance, wclass);
 		file_name = wDefaultGetIconFile(winstance, wclass, False);
+
+		if (file_name) {
+			file_path = FindImage(wPreferences.icon_path, file_name);
+			if (!file_path)
+				wwarning(_("icon \"%s\" doesn't exist, check your config files"), file_name);
+
+			/* FIXME: Here, if file_path does not exist then the icon is still in the
+			 * "icon database" (w_global.domain.window_attr->dictionary), but the file
+			 * for the icon is no more on disk. Therefore, we should remove it from the
+			 * database. Is possible to do that using wDefaultChangeIcon() */
+		}
 	}
 
-	/* Get the full path for the image file */
-	if (file_name) {
-		file_path = FindImage(wPreferences.icon_path, file_name);
-
-		if (!file_path)
-			wwarning(_("icon \"%s\" doesn't exist, check your config files"), file_name);
-
-		/* FIXME: Here, if file_path don't exists, then the icon is in the
-		 * "icon database" (WDWindowAttributes->dictionary), but the icon
-		 * is not en disk. Therefore, we should remove it from the icon
-		 * database. Is possible to do that using wDefaultChangeIcon() */
-
-		/* Don't wfree(file_name) here, because is a pointer to the icon
-		 * dictionary (WDWindowAttributes->dictionary) value. */
-	}
+	/*
+	 * Don't wfree(file_name) because it is a direct pointer inside the icon
+	 * dictionary (w_global.domain.window_attr->dictionary) and not a result
+	 * allocated with wstrdup()
+	 */
 
 	if (!file_path && default_icon)
 		file_path = get_default_image_path();
@@ -477,7 +477,7 @@ RImage *get_icon_image(WScreen *scr, const char *winstance, const char *wclass,
 	char *file_name = NULL;
 
 	/* Get the file name of the image, using instance and class */
-	file_name = get_icon_filename(scr, winstance, wclass, NULL, True);
+	file_name = get_icon_filename(winstance, wclass, NULL, True);
 
 	return get_rimage_from_file(scr, file_name, max_size);
 }
@@ -491,7 +491,7 @@ int wDefaultGetStartWorkspace(WScreen *scr, const char *instance, const char *cl
 	if (!ANoTitlebar)
 		init_wdefaults();
 
-	if (!WDWindowAttributes->dictionary)
+	if (!w_global.domain.window_attr->dictionary)
 		return -1;
 
 	value = get_generic_value(instance, class, AStartWorkspace, True);
@@ -519,7 +519,7 @@ char *wDefaultGetIconFile(const char *instance, const char *class, Bool default_
 	if (!ANoTitlebar)
 		init_wdefaults();
 
-	if (!WDWindowAttributes || !WDWindowAttributes->dictionary)
+	if (!w_global.domain.window_attr || !w_global.domain.window_attr->dictionary)
 		return NULL;
 
 	value = get_generic_value(instance, class, AIcon, default_icon);
@@ -532,9 +532,9 @@ char *wDefaultGetIconFile(const char *instance, const char *class, Bool default_
 	return tmp;
 }
 
-void wDefaultChangeIcon(WScreen *scr, const char *instance, const char *class, const char *file)
+void wDefaultChangeIcon(const char *instance, const char *class, const char *file)
 {
-	WDDomain *db = WDWindowAttributes;
+	WDDomain *db = w_global.domain.window_attr;
 	WMPropList *icon_value = NULL, *value, *attr, *key, *def_win, *def_icon = NULL;
 	WMPropList *dict = db->dictionary;
 	int same = 0;
@@ -568,14 +568,16 @@ void wDefaultChangeIcon(WScreen *scr, const char *instance, const char *class, c
 		icon_value = WMCreatePLDictionary(AIcon, value, NULL);
 		WMReleasePropList(value);
 
-		if ((def_win = WMGetFromPLDictionary(dict, AnyWindow)) != NULL)
+		def_win = WMGetFromPLDictionary(dict, AnyWindow);
+		if (def_win != NULL)
 			def_icon = WMGetFromPLDictionary(def_win, AIcon);
 
 		if (def_icon && !strcmp(WMGetFromPLString(def_icon), file))
 			same = 1;
 	}
 
-	if ((attr = WMGetFromPLDictionary(dict, key)) != NULL) {
+	attr = WMGetFromPLDictionary(dict, key);
+	if (attr != NULL) {
 		if (WMIsPLDictionary(attr)) {
 			if (icon_value != NULL && !same)
 				WMMergePLDictionaries(attr, icon_value, False);
@@ -596,7 +598,7 @@ void wDefaultChangeIcon(WScreen *scr, const char *instance, const char *class, c
 	WMPLSetCaseSensitive(False);
 }
 
-void wDefaultPurgeInfo(WScreen *scr, const char *instance, const char *class)
+void wDefaultPurgeInfo(const char *instance, const char *class)
 {
 	WMPropList *value, *key, *dict;
 	char *buffer;
@@ -611,15 +613,15 @@ void wDefaultPurgeInfo(WScreen *scr, const char *instance, const char *class)
 	sprintf(buffer, "%s.%s", instance, class);
 	key = WMCreatePLString(buffer);
 
-	dict = WMGetFromPLDictionary(WDWindowAttributes->dictionary, key);
+	dict = WMGetFromPLDictionary(w_global.domain.window_attr->dictionary, key);
 
 	if (dict) {
 		value = WMGetFromPLDictionary(dict, AIcon);
 		if (value) {
 			WMRemoveFromPLDictionary(dict, AIcon);
 		}
-		WMRemoveFromPLDictionary(WDWindowAttributes->dictionary, key);
-		UpdateDomainFile(WDWindowAttributes);
+		WMRemoveFromPLDictionary(w_global.domain.window_attr->dictionary, key);
+		UpdateDomainFile(w_global.domain.window_attr);
 	}
 
 	wfree(buffer);
