@@ -22,7 +22,7 @@
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-#ifdef SHAPE
+#ifdef USE_XSHAPE
 #include <X11/extensions/shape.h>
 #endif
 
@@ -49,6 +49,7 @@
 #include "wmspec.h"
 #include "xinerama.h"
 #include "event.h"
+#include "wsmap.h"
 
 #define MC_NEW          0
 #define MC_DESTROY_LAST 1
@@ -56,13 +57,8 @@
 /* index of the first workspace menu entry */
 #define MC_WORKSPACE1   3
 
-#define MAX_SHORTCUT_LENGTH 32
 #define WORKSPACE_NAME_DISPLAY_PADDING 32
 
-extern int ignore_wks_change;
-extern XContext wVEdgeContext;
-extern WShortKey wKeyBindings[WKBD_LAST];
-
 static WMPropList *dWorkspaces = NULL;
 static WMPropList *dClip, *dName;
 
@@ -97,8 +93,15 @@ int wWorkspaceNew(WScreen *scr)
 		wspace->clip = NULL;
 
 		if (!wspace->name) {
-			wspace->name = wmalloc(strlen(_("Workspace %i")) + 8);
-			sprintf(wspace->name, _("Workspace %i"), scr->workspace_count);
+			static const char *new_name = NULL;
+			static size_t name_length;
+
+			if (new_name == NULL) {
+				new_name = _("Workspace %i");
+				name_length = strlen(new_name) + 8;
+			}
+			wspace->name = wmalloc(name_length);
+			snprintf(wspace->name, name_length, new_name, scr->workspace_count);
 		}
 
 		if (!wPreferences.flags.noclip)
@@ -157,6 +160,8 @@ Bool wWorkspaceDelete(WScreen * scr, int workspace)
 		} else {
 			if (scr->workspaces[i]->name)
 				wfree(scr->workspaces[i]->name);
+			if (scr->workspaces[i]->map)
+				RReleaseImage(scr->workspaces[i]->map);
 			wfree(scr->workspaces[i]);
 		}
 	}
@@ -254,15 +259,14 @@ static void showWorkspaceName(WScreen * scr, int workspace)
 	char *name = scr->workspaces[workspace]->name;
 	int len = strlen(name);
 	int x, y;
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 	int head;
 	WMRect rect;
 	int xx, yy;
 #endif
 
-	if (wPreferences.workspace_name_display_position == WD_NONE || scr->workspace_count < 2) {
+	if (wPreferences.workspace_name_display_position == WD_NONE || scr->workspace_count < 2)
 		return;
-	}
 
 	if (scr->workspace_name_timer) {
 		WMDeleteTimerHandler(scr->workspace_name_timer);
@@ -283,7 +287,7 @@ static void showWorkspaceName(WScreen * scr, int workspace)
 	w = WMWidthOfString(scr->workspace_name_font, name, len);
 	h = WMFontHeight(scr->workspace_name_font);
 
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 	head = wGetHeadForPointerLocation(scr);
 	rect = wGetRectForHead(scr, head);
 	if (scr->xine_info.count) {
@@ -298,7 +302,7 @@ static void showWorkspaceName(WScreen * scr, int workspace)
 
 	switch (wPreferences.workspace_name_display_position) {
 	case WD_TOP:
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 		px = xx;
 #else
 		px = (scr->scr_width - (w + 4)) / 2;
@@ -306,7 +310,7 @@ static void showWorkspaceName(WScreen * scr, int workspace)
 		py = WORKSPACE_NAME_DISPLAY_PADDING;
 		break;
 	case WD_BOTTOM:
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 		px = xx;
 #else
 		px = (scr->scr_width - (w + 4)) / 2;
@@ -331,7 +335,7 @@ static void showWorkspaceName(WScreen * scr, int workspace)
 		break;
 	case WD_CENTER:
 	default:
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 		px = xx;
 		py = yy;
 #else
@@ -351,11 +355,9 @@ static void showWorkspaceName(WScreen * scr, int workspace)
 
 	XFillRectangle(dpy, text, WMColorGC(scr->black), 0, 0, w + 4, h + 4);
 
-	for (x = 0; x <= 4; x++) {
-		for (y = 0; y <= 4; y++) {
+	for (x = 0; x <= 4; x++)
+		for (y = 0; y <= 4; y++)
 			WMDrawString(scr->wmscreen, text, scr->white, scr->workspace_name_font, x, y, name, len);
-		}
-	}
 
 	XSetForeground(dpy, scr->mono_gc, 1);
 	XSetBackground(dpy, scr->mono_gc, 0);
@@ -369,8 +371,9 @@ static void showWorkspaceName(WScreen * scr, int workspace)
 
 	WMDrawString(scr->wmscreen, text, scr->white, scr->workspace_name_font, 2, 2, name, len);
 
-#ifdef SHAPE
-	XShapeCombineMask(dpy, scr->workspace_name, ShapeBounding, 0, 0, mask, ShapeSet);
+#ifdef USE_XSHAPE
+	if (w_global.xext.shape.supported)
+		XShapeCombineMask(dpy, scr->workspace_name, ShapeBounding, 0, 0, mask, ShapeSet);
 #endif
 	XSetWindowBackgroundPixmap(dpy, scr->workspace_name, text);
 	XClearWindow(dpy, scr->workspace_name);
@@ -388,10 +391,8 @@ static void showWorkspaceName(WScreen * scr, int workspace)
 	}
 
 	ximg = RGetXImage(scr->rcontext, scr->root_win, px, py, data->text->width, data->text->height);
-
-	if (!ximg || !ximg->image) {
+	if (!ximg)
 		goto erro;
-	}
 
 	XMapRaised(dpy, scr->workspace_name);
 	XFlush(dpy);
@@ -445,7 +446,7 @@ void wWorkspaceRelativeChange(WScreen * scr, int amount)
 	 * still "flying" to its final position and we don't want to
 	 * change workspace before the animation finishes, otherwise
 	 * the window will land in the new workspace */
-	if (ignore_wks_change)
+	if (w_global.ignore_workspace_change)
 		return;
 
 	w = scr->current_workspace + amount;
@@ -470,17 +471,17 @@ void wWorkspaceRelativeChange(WScreen * scr, int amount)
 void wWorkspaceForceChange(WScreen * scr, int workspace)
 {
 	WWindow *tmp, *foc = NULL, *foc2 = NULL;
-	WWindow **toUnmap;
-	int toUnmapSize, toUnmapCount;
 
 	if (workspace >= MAX_WORKSPACES || workspace < 0)
 		return;
 
+	if (wPreferences.enable_workspace_pager && !w_global.process_workspacemap_event)
+		wWorkspaceMapUpdate(scr);
+
 	SendHelperMessage(scr, 'C', workspace + 1, NULL);
 
-	if (workspace > scr->workspace_count - 1) {
+	if (workspace > scr->workspace_count - 1)
 		wWorkspaceMake(scr, workspace - scr->workspace_count + 1);
-	}
 
 	wClipUpdateForWorkspaceChange(scr, workspace);
 
@@ -491,16 +492,20 @@ void wWorkspaceForceChange(WScreen * scr, int workspace)
 
 	wWorkspaceMenuUpdate(scr, scr->clip_ws_menu);
 
-	toUnmapSize = 16;
-	toUnmapCount = 0;
-	toUnmap = wmalloc(toUnmapSize * sizeof(WWindow *));
+	tmp = scr->focused_window;
+	if (tmp != NULL) {
+		WWindow **toUnmap;
+		int toUnmapSize, toUnmapCount;
 
-	if ((tmp = scr->focused_window) != NULL) {
 		if ((IS_OMNIPRESENT(tmp) && (tmp->flags.mapped || tmp->flags.shaded) &&
 		     !WFLAGP(tmp, no_focusable)) || tmp->flags.changing_workspace) {
 			foc = tmp;
 		}
 
+		toUnmapSize = 16;
+		toUnmapCount = 0;
+		toUnmap = wmalloc(toUnmapSize * sizeof(WWindow *));
+
 		/* foc2 = tmp; will fix annoyance with gnome panel
 		 * but will create annoyance for every other application
 		 */
@@ -582,6 +587,26 @@ void wWorkspaceForceChange(WScreen * scr, int workspace)
 		if (!foc)
 			foc = foc2;
 
+		/*
+		 * Check that the window we want to focus still exists, because the application owning it
+		 * could decide to unmap/destroy it in response to unmap any of its other window following
+		 * the workspace change, this happening during our 'ProcessPendingEvents' loop.
+		 */
+		if (foc != NULL) {
+			WWindow *parse;
+			Bool found;
+
+			found = False;
+			for (parse = scr->focused_window; parse != NULL; parse = parse->prev) {
+				if (parse == foc) {
+					found = True;
+					break;
+				}
+			}
+			if (!found)
+				foc = NULL;
+		}
+
 		if (scr->focused_window->flags.mapped && !foc) {
 			foc = scr->focused_window;
 		}
@@ -645,13 +670,19 @@ static void switchWSCommand(WMenu * menu, WMenuEntry * entry)
 	wWorkspaceChange(menu->frame->screen_ptr, (long)entry->clientdata);
 }
 
-static void lastWSCommand(WMenu * menu, WMenuEntry * entry)
+static void lastWSCommand(WMenu *menu, WMenuEntry *entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	wWorkspaceChange(menu->frame->screen_ptr, menu->frame->screen_ptr->last_workspace);
 }
 
-static void deleteWSCommand(WMenu * menu, WMenuEntry * entry)
+static void deleteWSCommand(WMenu *menu, WMenuEntry *entry)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) entry;
+
 	wWorkspaceDelete(menu->frame->screen_ptr, menu->frame->screen_ptr->workspace_count - 1);
 }
 
@@ -659,6 +690,9 @@ static void newWSCommand(WMenu *menu, WMenuEntry *foo)
 {
 	int ws;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) foo;
+
 	ws = wWorkspaceNew(menu->frame->screen_ptr);
 
 	/* autochange workspace */
@@ -769,9 +803,8 @@ void wWorkspaceMenuUpdate(WScreen * scr, WMenu * menu)
 		}
 	} else if (menu->entry_no > scr->workspace_count + MC_WORKSPACE1) {
 		/* removed workspace(s) */
-		for (i = menu->entry_no - 1; i >= scr->workspace_count + MC_WORKSPACE1; i--) {
+		for (i = menu->entry_no - 1; i >= scr->workspace_count + MC_WORKSPACE1; i--)
 			wMenuRemoveItem(menu, i);
-		}
 	}
 
 	for (i = 0; i < scr->workspace_count; i++) {
@@ -787,11 +820,10 @@ void wWorkspaceMenuUpdate(WScreen * scr, WMenu * menu)
 	wMenuRealize(menu);
 
 	/* don't let user destroy current workspace */
-	if (scr->current_workspace == scr->workspace_count - 1) {
+	if (scr->current_workspace == scr->workspace_count - 1)
 		wMenuSetEnabled(menu, MC_DESTROY_LAST, False);
-	} else {
+	else
 		wMenuSetEnabled(menu, MC_DESTROY_LAST, True);
-	}
 
 	/* back to last workspace */
 	if (scr->workspace_count && scr->last_workspace != scr->current_workspace)
@@ -825,10 +857,11 @@ void wWorkspaceSaveState(WScreen * scr, WMPropList * old_state)
 			WMPutInPLDictionary(wks_state, dClip, pstr);
 			WMReleasePropList(pstr);
 		} else if (old_wks_state != NULL) {
-			if ((foo = WMGetFromPLArray(old_wks_state, i)) != NULL) {
-				if ((bar = WMGetFromPLDictionary(foo, dClip)) != NULL) {
+			foo = WMGetFromPLArray(old_wks_state, i);
+			if (foo != NULL) {
+				bar = WMGetFromPLDictionary(foo, dClip);
+				if (bar != NULL)
 					WMPutInPLDictionary(wks_state, dClip, bar);
-				}
 			}
 		}
 		WMAddToPLArray(parr, wks_state);
