@@ -23,6 +23,8 @@
 #define _GNU_SOURCE		/* getopt_long */
 #endif
 
+#include "config.h"
+
 #include <sys/stat.h>
 
 #include <getopt.h>
@@ -34,6 +36,10 @@
 #include <unistd.h>
 #include <X11/Xlib.h>
 
+#ifdef HAVE_STDNORETURN
+#include <stdnoreturn.h>
+#endif
+
 #include <WINGs/WUtil.h>
 
 #include "../src/wconfig.h"
@@ -45,7 +51,6 @@
 char *FontOptions[] = {
 	"IconTitleFont",
 	"ClipTitleFont",
-	"DisplayFont",
 	"LargeDisplayFont",
 	"MenuTextFont",
 	"MenuTitleFont",
@@ -71,7 +76,7 @@ char *CursorOptions[] = {
 	NULL
 };
 
-extern char *__progname;
+static const char *prog_name;
 int ignoreFonts = 0;
 int ignoreCursors = 0;
 
@@ -198,6 +203,7 @@ static void hackPaths(WMPropList * style, const char *prefix)
 		}
 	}
 
+	WMReleasePropList(keys);
 }
 
 static WMPropList *getColor(WMPropList * texture)
@@ -308,6 +314,7 @@ static void hackStyle(WMPropList * style)
 			}
 		}
 	}
+	WMReleasePropList(keys);
 
 	if (!foundIconTitle) {
 		/* set the default values */
@@ -350,9 +357,9 @@ static void hackStyle(WMPropList * style)
 	}
 }
 
-static void print_help(int print_usage, int exitval)
+static noreturn void print_help(int print_usage, int exitval)
 {
-	printf("Usage: %s [OPTIONS] FILE\n", __progname);
+	printf("Usage: %s [OPTIONS] FILE\n", prog_name);
 	if (print_usage) {
 		puts("Reads style/theme configuration from FILE and updates Window Maker.");
 		puts("");
@@ -385,10 +392,11 @@ int main(int argc, char **argv)
 		{ NULL,		0,			NULL,			0 }
 	};
 
+	prog_name = argv[0];
 	while ((ch = getopt_long(argc, argv, "hv", longopts, NULL)) != -1)
 		switch(ch) {
 			case 'v':
-				printf("%s (Window Maker %s)\n", __progname, VERSION);
+				printf("%s (Window Maker %s)\n", prog_name, VERSION);
 				return 0;
 				/* NOTREACHED */
 			case 'h':
@@ -424,7 +432,7 @@ int main(int argc, char **argv)
 	prop = WMReadPropListFromFile(path);
 	if (!prop) {
 		perror(path);
-		printf("%s: could not load WindowMaker configuration file.\n", __progname);
+		printf("%s: could not load WindowMaker configuration file.\n", prog_name);
 		return 1;
 	}
 
@@ -440,17 +448,17 @@ int main(int argc, char **argv)
 			perror(file);
 			return 1;
 		}
-		strncat(buf, "/style", sizeof(buf) - strlen(buf));
+		strncat(buf, "/style", sizeof(buf) - strlen(buf) - 1);
 
 		if (stat(buf, &st) != 0 || !S_ISREG(st.st_mode)) {	/* maybe symlink too? */
-			printf("%s: %s: style file not found or not a file\n", __progname, buf);
+			printf("%s: %s: style file not found or not a file\n", prog_name, buf);
 			return 1;
 		}
 
 		style = WMReadPropListFromFile(buf);
 		if (!style) {
 			perror(buf);
-			printf("%s: could not load style file.\n", __progname);
+			printf("%s: could not load style file.\n", prog_name);
 			return 1;
 		}
 
@@ -473,13 +481,13 @@ int main(int argc, char **argv)
 		style = WMReadPropListFromFile(file);
 		if (!style) {
 			perror(file);
-			printf("%s:could not load style file.\n", __progname);
+			printf("%s:could not load style file.\n", prog_name);
 			return 1;
 		}
 	}
 
 	if (!WMIsPLDictionary(style)) {
-		printf("%s: '%s' is not a style file/theme\n", __progname, file);
+		printf("%s: '%s' is not a style file/theme\n", prog_name, file);
 		return 1;
 	}
 
