@@ -15,7 +15,8 @@
  *
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -134,6 +135,8 @@ int RBlurImage(RImage * image)
 		}
 	}
 
+	free(tmpp);
+
 	return True;
 }
 
